/**
* @file     A31G32x_hal_adc.h
* @brief    Contains all macro definitions and function prototypes support
*           for ADC driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _ADC_H_
#define _ADC_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define ADC_CCR_PDN			((1UL<<7))

#define ADC_CR_START_MASK	((1UL<<0))
#define ADC_CR_STOP_MASK		((1UL<<7))

#define ADC_STAT_COMPIFLG		((1UL<<8))
#define ADC_STAT_DMAOVERRUN		((1UL<<5))
#define ADC_STAT_DMA			((1UL<<4))
#define ADC_STAT_TRIGGER		((1UL<<3))
#define ADC_STAT_SEQ			((1UL<<2))
#define ADC_STAT_SINGLE			((1UL<<0))

#define ADC_INTEN_DMA			((1UL<<4))
#define ADC_INTEN_TRIGGER		((1UL<<3))
#define ADC_INTEN_SEQ			((1UL<<2))
#define ADC_INTEN_SINGLE		((1UL<<0))

#define ADC_DR_RESULT(n)		((n)&0xFFF0)
#define ADC_GET_CH_RESULT(n)	((n)&0xF)

#define ADC_SINGLE_MODE		0  
#define ADC_BURST_MODE		1  
#define ADC_MULTI_MODE		2  

#define ADC_INTERNAL_CLK		0  
#define ADC_EXTERNAL_CLK		1  

#define ADC_TRIGGER_DISABLE	0  
#define ADC_TRIGGER_TIMER		1 

/*******************************************************************************
* Public Typedef
*******************************************************************************/

/**
 * @brief  ADC configuration structure definition
 */
typedef struct {
	/* MR */
	uint32_t Mode;			/**< Mode, should be:
							 - ADC_SINGLE_MODE
							 - ADC_CONTINUOUS_MODE
							 - ADC_BURST_MODE */
	uint32_t DmaOpt;		/**< DmaOpt = ENABLE or DISABLE */
	uint32_t SamplingTime;
	uint32_t SeqCnt;		/**< SeqCnt = count value 2~8	*/
	uint32_t RestartEn;		/**< RestartEn = ENABLE or DISABLE */
	uint32_t TrgSel;

	/* CR1 */
	uint32_t InClkDiv;
} ADC_CFG_Type;

/**
 * @brief  ADC comparator configuration structure definition
 */
typedef struct {
	/*ADC Channel Compare*/
	uint32_t COMPIEN;	/*!< adc channel compare interrupt */
	uint32_t COMPEN;	/*!< adc channel compare enable or disable */
	uint32_t LTE;		/*!< adc Conversion value output timing setting */
	uint32_t CCH;		/*!< adc compare channel */
	uint32_t CVAL;		/*!< adc compare value bit*/
} ADC_CMPR_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct);
HAL_Status_Type HAL_ADC_DeInit(ADC_Type *ADCx);
HAL_Status_Type HAL_ADC_EnterPowerdownMode(ADC_Type *ADCx, FunctionalState NewState);
HAL_Status_Type HAL_ADC_ChannelSel(ADC_Type *ADCx, uint32_t Channel);
HAL_Status_Type HAL_ADC_ChannelSel_1(ADC_Type *ADCx, uint32_t Channel);
HAL_Status_Type HAL_ADC_ChannelSel_2(ADC_Type *ADCx, uint32_t Channel);
HAL_Status_Type HAL_ADC_TriggerlSel(ADC_Type *ADCx, uint32_t Trigger);
HAL_Status_Type HAL_ADC_Start(ADC_Type *ADCx);
HAL_Status_Type HAL_ADC_Stop(ADC_Type *ADCx);
HAL_Status_Type HAL_ADC_EdgeStartConfig(ADC_Type *ADCx, uint8_t EdgeOption);
HAL_Status_Type HAL_ADC_ConfigInterrupt(ADC_Type *ADCx, uint8_t IntType, FunctionalState NewState);
uint16_t HAL_ADC_GetData(ADC_Type *ADCx);
uint32_t HAL_ADC_GetStatus(ADC_Type *ADCx);
HAL_Status_Type HAL_ADC_ClearStatus(ADC_Type *ADCx, uint32_t status);
HAL_Status_Type HAL_ADC_Channel_Compare(ADC_Type *ADCx, ADC_CMPR_Type *ADC_Compare);
#ifdef __cplusplus
}
#endif

#endif
/* --------------------------------- End Of File ------------------------------ */
