/**
* @file     A31G32x_hal_spi.h
* @brief    Contains all macro definitions and function prototypes support
*           for spi driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _TIMER1n_H_
#define _TIMER1n_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"
#include "A31G32x_hal_timer_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
/* Timer 1n Enable/Disable Definition */
#define T1n_DISABLE	(0x0uL<<15)
#define T1n_ENABLE	(0x1uL<<15)

/* Timer 1n Clock Selection Definition */
#define T1n_CLKINT		(0x0uL<<14)
#define T1n_CLKEXT	(0x1uL<<14)

/* Timer 1n Mode Selection Definition */
#define T1n_CNTM		(0x0uL<<12)
#define T1n_CAPM		(0x1uL<<12)
#define T1n_PPGONEM	(0x2uL<<12)
#define T1n_PPGREM	(0x3uL<<12)

/* Timer 1n External Clock Edge Selection Definition */
#define T1n_FEDGE		(0x0uL<<11)
#define T1n_REDGE		(0x1uL<<11)

/* Timer 1n OUT Polarity Selection Definition */
#define T1n_STHIGH	(0x0uL<<8)
#define T1n_STLOW	(0x1uL<<8)

/* Timer 1n Capture Polarity Selection Definition */
#define T1n_CAPFALL	(0x0uL<<6)
#define T1n_CAPRISE	(0x1uL<<6)
#define T1n_CAPBOTH	(0x2uL<<6)

/* Timer 1n Match Interrupt Definition */
#define T1n_MATINTEN	(0x1uL<<5)
#define T1n_MATINTDIS	(0x0uL<<5)

/* Timer 1n Capture Interrupt Definition */
#define T1n_CAPINTEN	(0x1uL<<4)
#define T1n_CAPINTDIS	(0x0uL<<4)

#define T1n_PRS_MASK	0x0FFF

/* Timer n Control register  */
#define TIMER1n_CR_CKSEL_MASK			(0x01<<14)
#define TIMER1n_CR_CKSEL_SET(n)		(n<<14)

#define TIMER1n_CR_MODE_MASK			(0x03<<12)
#define TIMER1n_CR_MODE_SET(n)		(n<<12)

#define TIMER1n_CR_ECE_MASK			(0x01<<11)
#define TIMER1n_CR_ECE_SET(n)			(n<<11)

#define TIMER1n_CR_STARTLVL_MASK		(0x01<<8)
#define TIMER1n_CR_STARTLVL_SET(n)		(n<<8)

#define TIMER1n_CR_CPOL_MASK			(0x03<<6)
#define TIMER1n_CR_CPOL_SET(n)			(n<<6)

/*******************************************************************************
* Public Typedef
*******************************************************************************/
/** @brief counter clock source select */
typedef enum
{
	MCCR1PCLK = 0,		/*!< clock source from pclk or mccr1 */
	ECn = 1				/*!< clock source from ECn pin input. before setting, have to set ECn pin mode */	
} TIMER1n_CKSEL_MODE_OPT;

/** @brief TIMER Interrupt Type definitions **/
typedef enum {
	TIMER1n_INTCFG_MIE = 0,	/*!< Match Interrupt enable*/
	TIMER1n_INTCFG_CIE,		/*!< Capture Interrupt enable*/
} TIMER1n_INT_Type;

/** @brief Configuration structure in TIMER mode */
typedef struct
{
	uint16_t TnADR;	
	uint16_t TnBDR;	
	uint16_t Prescaler;		
	uint8_t StartLevel;		/**< set initial output value, should be:	START_LOW or START_HIGH */		
	uint8_t CkSel;			/**< Counter clock source select, should be: PCLK or  ECn */
	uint8_t ECE;
} TIMER1n_PERIODICCFG_Type;

/** @brief Configuration structure in COUNTER mode */
typedef struct {
	uint16_t TnADR;		
	uint16_t TnBDR;			
	uint16_t Prescaler;		
	uint8_t StartLevel;		/**< set initial output value, should be:	START_LOW or START_HIGH */	
	uint8_t CkSel;			/**< Counter clock source select, should be: PCLK or  ECn */	
	uint8_t ECE;
} TIMER1n_PWMCFG_Type,TIMER1n_ONESHOTCFG_Type;

/** @brief Capture Input configuration structure */
typedef struct {	
	uint16_t TnADR;	
	uint16_t Prescaler;		
	uint8_t ClrMode;		
	uint8_t CkSel;
	uint8_t ECE;
} TIMER1n_CAPTURECFG_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
extern void T1n_SetReg(TIMER1n_Type *, uint32_t, uint32_t);
HAL_Status_Type HAL_TIMER1n_Init(TIMER1n_Type *TIMER1n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct);
HAL_Status_Type HAL_TIMER1n_DeInit (TIMER1n_Type *TIMER1n);
HAL_Status_Type HAL_TIMER1n_Cmd(TIMER1n_Type *TIMER1n, FunctionalState NewState);
HAL_Status_Type HAL_TIMER1n_ConfigInterrupt(TIMER1n_Type *TIMER1n, TIMER1n_INT_Type TIMERIntCfg, FunctionalState NewState);
HAL_Status_Type HAL_TIMER1n_ClearCounter(TIMER1n_Type *TIMER1n);
uint32_t HAL_TIMER1n_GetMatchInterrupt(TIMER1n_Type *TIMER1n);
HAL_Status_Type HAL_TIMER1n_ClearMatchInterrupt(TIMER1n_Type *TIMER1n);
uint32_t HAL_TIMER1n_GetCaptureData(TIMER1n_Type *TIMER1n);
uint32_t HAL_TIMER1n_GetCaptureInterrupt(TIMER1n_Type *TIMER1n);
HAL_Status_Type HAL_TIMER1n_ClearCaptureInterrupt(TIMER1n_Type *TIMER1n);

#ifdef __cplusplus
}
#endif

#endif /*_TIMER1n_H_ */

