/**
* @file     A31G32x_hal_timer2n.h
* @brief    Contains all macro definitions and function prototypes support
*           for timer2n driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _TIMER2n_H_
#define _TIMER2n_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"
#include "A31G32x_hal_timer_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
/** Timer2n Counter clear after Capture bits */
#define TIMER2n_CR_CNCLR_MASK		(0x01<<16)
#define TIMER2n_CR_CNCLR_SET(n)		(n<<16)
	
/** counter clock select bits */
#define TIMER2n_CR_CLK_MASK		(0x01<<14)
#define TIMER2n_CR_CLK_SET(n)		(n<<14)

/** Counter/timer mode bits */
#define TIMER2n_CR_MODE_MASK		(0x03<<12)
#define TIMER2n_CR_MODE_SET(n)	(n<<12)
	
/** External Clock Edge Selection bit */
#define TIMER2n_CR_ECE_MASK		(0x01<<11)
#define TIMER2n_CR_ECE_SET(n)		(n<<11)

/** T2nCAP signal selection bit */
#define TIMER2n_CR_CAPSEL_MASK	(0x03<<9)
#define TIMER2n_CR_CAPSEL_SET(n)	(n<<9)


/** T2nOUT Polarity Selection bit */
#define TIMER2n_CR_OPOL_MASK		(0x01<<8)
#define TIMER2n_CR_OPOL_SET(n)	(n<<8)

/** T2n Capture Polarity Selection bit */
#define TIMER2n_CR_CPOL_MASK		(0x03<<6)
#define TIMER2n_CR_CPOL_SET(n)		(n<<6)

/** T2n Match Interrupt Enable bit */
#define TIMER2n_CR_MIEN_MASK		(0x01<<5)
#define TIMER2n_CR_MIEN_SET(n)		(n<<5)

/** T2n Capture Interrupt Enable bit */
#define TIMER2n_CR_CIEN_MASK		(0x01<<4)
#define TIMER2n_CR_CIEN_SET(n)		(n<<4)

/** T2n Counter Temporary Pause Control bit */
#define TIMER2n_CR_PAU_MASK		(0x01<<1)
#define TIMER2n_CR_PAU_SET(n)		(n<<1)

/** T2n Match Interrupt Flag */
#define TIMER2n_CR_MATCH_FLAG		(1<<3)
#define TIMER2n_CR_CAPTURE_FLAG	(1<<2)


/** Clear Flag */
#define TIMER2n_CR_CLEAR_MATCHINT		(0x01<<3)
#define TIMER2n_CR_CLEAR_CAPTUREINT	(0x01<<2)
#define TIMER2n_CR_CLEAR_CNT_PRED		(0x01<<0)

/** Timer/Counter 2n A Data Register (ADR) */
#define TIMER2n_ADR_MASK	0xFFFFFFFF

/** Timer/Counter 2n B Data Register (BDR) */
#define TIMER2n_BDR_MASK	0xFFFFFFFF

/** Timer/Counter 2n Prescaler Data Register (PREDR) */
#define TIMER2n_PREDR_MASK		(0xFFF<<0)
#define TIMER2n_PREDR_SET(n)		(n<<0)

/** Timer/Counter 2n definitions */
#define TIMER2n_ENABLE	(1)
#define TIMER2n_DISABLE	(0)
#define TIMER2n_CLEAR		(1)


/*******************************************************************************
* Public Typedef
*******************************************************************************/

/** @brief Timer clock source select */
typedef enum
{
	SOSC_TIMER2n_CLK = 0,				/*!< clock source from Internal */
	PCLK_TIMER2n_CLK,					/*!< clock source from External */
} TIMER2n_MASTER_CKSEL_OPT;


/** @brief counter clock source select */
typedef enum
{
	MCCR2PCLK = 0,				/*!< clock source from Internal */
	PCLK = 0,						/*!< clock source from Internal */
	EC2n=1,						/*!< clock source from External */
} TIMER2n_CKSEL_MODE_OPT;

/** @brief capture clock source select */
typedef enum
{
	CAP_EXTERNAL_CLK = 0,			/*!< clock source from Externl */
	CAP_SOSC_CLK,				/*!< clock source from XOSC */
	CAP_WDTRC_CLK,				/*!< Clock Source from WDTRC */
} TIMER2n_CAP_CKSEL_OPT;

/** @brief TIMER Interrupt Type definitions */
typedef enum {
	TIMER2n_CR_MATCH_INTR = 0,	/*!< OVIE Interrupt enable*/
	TIMER2n_CR_CAPTURE_INTR,		/*!< MBIE Interrupt enable*/
} TIMER2n_INT_Type;

/** @brief Configuration structure in TIMER mode */
typedef struct
{
	uint32_t T2nADR;	
	uint32_t T2nBDR;	
	uint16_t Prescaler;
	uint8_t StartLevel;		
	uint8_t CkSel;
	uint8_t ECE;
} TIMER2n_PERIODICCFG_Type;

/** @brief Configuration structure in COUNTER mode */
typedef struct {
	uint32_t T2nADR;
	uint32_t T2nBDR;	
	uint16_t Prescaler;
	uint8_t StartLevel;
	uint8_t CkSel;
	uint8_t ECE;
} TIMER2n_PWMCFG_Type,TIMER2n_ONESHOTCFG_Type;

/** @brief Capture Input configuration structure */
typedef struct {
	uint32_t T2nADR;
	uint16_t Prescaler;
	uint8_t ClrMode;
	uint8_t CkSel;
	uint8_t ECE;	
	uint8_t CAPCkSel;	
	uint8_t CNCLR;
} TIMER2n_CAPTURECFG_Type;

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_TIMER2n_Init(TIMER2n_Type *TIMER2n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct);
HAL_Status_Type HAL_TIMER2n_DeInit (TIMER2n_Type *TIMER2n);
HAL_Status_Type HAL_TIMER2n_Cmd(TIMER2n_Type *TIMER2n, FunctionalState NewState);
HAL_Status_Type HAL_TIMER2n_ConfigInterrupt(TIMER2n_Type *TIMER2n, TIMER2n_INT_Type TIMERIntCfg, FunctionalState NewState);
HAL_Status_Type HAL_TIMER2n_ClearCounter(TIMER2n_Type *TIMER2n);
uint32_t HAL_TIMER2n_GetMatchInterrupt(TIMER2n_Type *TIMER2n);
HAL_Status_Type HAL_TIMER2n_ClearMatchInterrupt(TIMER2n_Type *TIMER2n);
uint32_t HAL_TIMER2n_GetCaptureData(TIMER2n_Type *TIMER2n);
uint32_t HAL_TIMER2n_GetCaptureInterrupt(TIMER2n_Type *TIMER2n);
HAL_Status_Type HAL_TIMER2n_ClearCaptureInterrupt(TIMER2n_Type *TIMER2n);

#ifdef __cplusplus
}
#endif

#endif /* _TIMER2n_H_ */

/* --------------------------------- End Of File ------------------------------ */
