/**
* @file     A31G32x_hal_timer40.h
* @brief    Contains all macro definitions and function prototypes support
*           for timer40 driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _TIMER40_H_
#define _TIMER40_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"
#include "A31G32x_hal_timer_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
/** Timer 40 Enable/Disable Definition */
#define T40_DISABLE		(0x0uL<<15)
#define T40_ENABLE		(0x1uL<<15)

/** Timer 40 Clock Selection Definition   */
#define T40_CLKINT			(0x0uL<<14)
#define T40_CLKEXT		(0x1uL<<14)

/** Timer 40 Mode Selection Definition   */
#define T40_CNTM			(0x0uL<<12)
#define T40_CAPM			(0x1uL<<12)
#define T40_PPGONEM		(0x2uL<<12)
#define T40_PPGREM		(0x3uL<<12)

/** Timer 40 External Clock Edge Selection Definition   */
#define T40_FEDGE			(0x0uL<<11)
#define T40_REDGE			(0x1uL<<11)

/** Timer 40 OUT Polarity Selection Definition   */
#define T40_STHIGH		(0x0uL<<8)
#define T40_STLOW			(0x1uL<<8)

/** Timer 40 Capture Polarity Selection Definition   */
#define T40_CAPFALL		(0x0uL<<6)
#define T40_CAPRISE		(0x1uL<<6)
#define T40_CAPBOTH		(0x2uL<<6)

/** Timer 40 Match Interrupt Definition  */
#define T40_MATINTEN		(0x1uL<<5)
#define T40_MATINTDIS		(0x0uL<<5)

/** Timer 40 Capture Interrupt Definition  */
#define T40_CAPINTEN		(0x1uL<<4)
#define T40_CAPINTDIS		(0x0uL<<4)

#define T40_PRS_MASK	0x0FFF

/** Timer n Control register  */
#define TIMER40_CR_CKSEL_MASK			(0x01<<14)
#define TIMER40_CR_CKSEL_SET(n)		(n<<14)

#define TIMER40_CR_MODE_MASK			(0x03<<12)
#define TIMER40_CR_MODE_SET(n)		(n<<12)

#define TIMER40_CR_ECE_MASK			(0x01<<11)
#define TIMER40_CR_ECE_SET(n)			(n<<11)

#define TIMER40_CR_CAPCHSEL_MASK		(0x03<<9)
#define TIMER40_CR_CAPCHSEL(n)			(n<<9)

#define TIMER40_CR_STARTLVL_MASK		(0x01<<8)
#define TIMER40_CR_STARTLVL_SET(n)		(n<<8)

#define TIMER40_CR_CPOL_MASK			(0x03<<6)
#define TIMER40_CR_CPOL_SET(n)			(n<<6)


/*******************************************************************************
* Public Typedef
*******************************************************************************/
/** @brief counter clock source select */
typedef enum
{
	MCCR6PCLK = 0,			/*!< clock source from pclk or mccr1 */
	EC4n = 1					/*!< clock source from ECn pin input. before setting, have to set ECn pin mode */	
} TIMER40_CKSEL_MODE_OPT;

typedef enum {
	TIMER40_INTCFG_MIE = 0,	/*!< Match Interrupt enable*/
	TIMER40_INTCFG_CIE,		/*!< Capture Interrupt enable*/
} TIMER40_INT_Type;

typedef enum {
	TIMER40_CH1 = 0,
	TIMER40_CH2,
	TIMER40_CH3,
	TIMER40_XOR
} TIMER40_CAPTURE_CHANNEL;

/** @brief Configuration structure in TIMER mode */
typedef struct
{
	uint16_t TnADR;	
	uint16_t TnBDR;	
	uint16_t Prescaler;		
	uint8_t StartLevel;			/**< set initial output value, should be:	START_LOW or START_HIGH */		
	uint8_t CkSel;				/**< Counter clock source select, should be: PCLK or  ECn */
	uint8_t ECE;
} TIMER40_PERIODICCFG_Type;

/** @brief Configuration structure in COUNTER mode */
typedef struct {
	uint16_t TnADR;		
	uint16_t TnBDR;			
	uint16_t Prescaler;		
	uint8_t StartLevel;			/**< set initial output value, should be:	START_LOW or START_HIGH */	
	uint8_t CkSel;				/**< Counter clock source select, should be: PCLK or  ECn */	
	uint8_t ECE;
} TIMER40_PWMCFG_Type,TIMER40_ONESHOTCFG_Type;

/** @brief Capture Input configuration structure */
typedef struct {	
	uint16_t TnADR;	
	uint16_t Prescaler;	
	uint16_t Prescaler_capture;
	uint8_t ClrMode;		
	uint8_t CkSel;
	uint8_t ECE;
	uint8_t Channel;
} TIMER40_CAPTURECFG_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
extern void T40_SetReg(TIMER40_Type *, uint32_t, uint32_t);

HAL_Status_Type HAL_TIMER4n_Init(TIMER40_Type *TIMER4n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct);
HAL_Status_Type HAL_TIMER4n_DeInit (TIMER40_Type *TIMER4n);
HAL_Status_Type HAL_TIMER4n_Cmd(TIMER40_Type *TIMER4n, FunctionalState NewState);
HAL_Status_Type HAL_TIMER4n_ConfigInterrupt(TIMER40_Type *TIMER4n, TIMER40_INT_Type TIMERIntCfg, FunctionalState NewState);
HAL_Status_Type HAL_TIMER4n_ClearCounter(TIMER40_Type *TIMER4n);
uint32_t HAL_TIMER4n_GetMatchInterrupt(TIMER40_Type *TIMER4n);  
HAL_Status_Type HAL_TIMER4n_ClearMatchInterrupt(TIMER40_Type *TIMER4n);     
uint32_t HAL_TIMER4n_GetCaptureData(TIMER40_Type *TIMER4n);  
uint32_t HAL_TIMER4n_GetCaptureInterrupt(TIMER40_Type *TIMER4n);  
HAL_Status_Type HAL_TIMER4n_ClearCaptureInterrupt(TIMER40_Type *TIMER4n);   

#ifdef __cplusplus
}
#endif

#endif /*_TIMER40_H_ */

