/**
* @file     A31G32x_hal_usartn.h
* @brief    Contains all macro definitions and function prototypes support
*           for usartn driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _USARTn_H_
#define _USARTn_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif


/*******************************************************************************
* Public Macro
*******************************************************************************/
#define UST_BLOCKING_TIMEOUT			(0xFFFFFFFFUL)

#define UST_CR1_DRINT_EN        ((uint16_t)(1<<5))      /*!< DR Interrupt enable*/
#define UST_CR1_TXCINT_EN       ((uint16_t)(1<<4)) 	    /*!< TXC Interrupt enable*/
#define UST_CR1_RXCINT_EN       ((uint16_t)(1<<3)) 	    /*!< RXC Interrupt enable*/
#define UST_CR1_WAKEINT_EN      ((uint16_t)(1<<2)) 	    /*!< WAKE Interrupt enable*/
#define UST_CR1_INT_BITMASK     ((uint16_t)(0x3C))      /*!< USART CR1 Interrupt register bit mask */

#define UST_CR1_TX_EN           ((uint16_t)(1<<1)) 	    /*!< TX enable*/
#define UST_CR1_RX_EN           ((uint16_t)(1<<0)) 	    /*!< RX enable*/

#define USART_CR1_USTnMS_Pos    14
#define USART_CR1_USTnP_Pos     13
#define USART_CR1_USTnS_Pos     9
#define USART_CR1_ORDn_Pos      8
#define USART_CR1_CPOLn_Pos     7
#define USART_CR1_CPHAn_Pos     6

#define UST_CR2_DMA_TXINT_EN    ((uint16_t)(1<<14)) 	/*!< DMA TX Interrupt enable*/
#define UST_CR2_DMA_RXINT_EN    ((uint16_t)(1<<13)) 	/*!< DMA RX Interrupt enable*/
#define UST_CR2_RTOINT_EN       ((uint16_t)(1<<12)) 	/*!< RTO Interrupt enable*/

#define UST_CR2_RTO_EN          ((uint16_t)(1<<11)) 	/*!< RTO enable*/
#define UST_CR2_FPC_EN          ((uint16_t)(1<<10)) 	/*!< Floating Point Counter enable*/
#define UST_CR2_UST_EN          ((uint16_t)(1<<9)) 	    /*!< USART block enable*/
#define UST_CR2_DBLS            ((uint16_t)(1<<8)) 	    /*!< Select receiver sampling rate (only UART mode)*/
#define UST_CR2_MASTER          ((uint16_t)(1<<7)) 	    /*!< Selects master or slave in SPI or synchronous mode and controls the direction of SCK pin*/
#define UST_CR2_LOOPS           ((uint16_t)(1<<6)) 	    /*!< Control the Loop Back mode of USART for test mode*/
#define UST_CR2_DISSCK          ((uint16_t)(1<<5)) 	    /*!< In synchronous mode operation, select the waveform of SCK output*/
#define UST_CR2_SSEN            ((uint16_t)(1<<4)) 	    /*!< SS Pin operation select (only SPI mode)*/
#define UST_CR2_FXCH            ((uint16_t)(1<<3)) 	    /*!< SPI port function exchange control (only SPI mode)*/
#define UST_CR2_STOPBIT         ((uint16_t)(1<<2)) 	    /*!< Select the length of stop bit (0: 1 stop bit, 1: 2 stop bit) */
#define UST_CR2_TX8             ((uint16_t)(1<<1)) 	    /*!< The ninth bit of TX data frame (0: MSB 9th bit 0, 1: MSB 9th bit 1 */
#define UST_CR2_RX8             ((uint16_t)(1<<0)) 	    /*!< The ninth bit of RX data frame (0: MSB 9th bit 0, 1: MSB 9th bit 1 */
#define UST_CR2_CONTROL_BITMASK ((uint16_t)(0x07FF))    /*!< USART CR2 mode control register bit mask */
#define UST_CR2_INT_BITMASK     ((uint16_t)(0x7800))    /*!< USART CR2 Interrupt register bit mask */
#define UST_CR2_USTnSB_Pos      2

#define UST_SR_DMATX            ((uint16_t)(1<<9)) 	    /*!<Interrupt identification: DMA TX*/
#define UST_SR_DMARX            ((uint16_t)(1<<8)) 	    /*!<Interrupt identification: DMA RX*/
#define UST_SR_DRE              ((uint16_t)(1<<7)) 	    /*!<Interrupt identification: Data Register empty*/
#define UST_SR_TXC              ((uint16_t)(1<<6))  		/*!<Interrupt identification: Tx Complete*/
#define UST_SR_RXC              ((uint16_t)(1<<5)) 	    /*!<Interrupt identification: Rx Complete*/
#define UST_SR_WAKE             ((uint16_t)(1<<4)) 	    /*!<Interrupt identification: Wake*/
#define UST_SR_RTO              ((uint16_t)(1<<3)) 	    /*!<Interrupt identification: RTO*/
#define UST_SR_DOR              ((uint16_t)(1<<2))	    /*!<Interrupt identification: Data OverRun */
#define UST_SR_FE               ((uint16_t)(1<<1))	    /*!<Interrupt identification: Frame Error */
#define UST_SR_PE               ((uint16_t)(1<<0))	    /*!<Interrupt identification: Parity Error */
#define UST_SR_BITMASK          ((uint16_t)(0x3FF))	    /*!<USART status register bit mask */


/*******************************************************************************
* Public Typedef
*******************************************************************************/
/**  @brief USART Databit type definitions */
typedef enum {
	USART_UART_MODE     = 0,        /*!< UART mode */
	USART_SYNC_MODE,                /*!< Syncronous Mode mode*/
	USART_SPI_MODE      = 3         /*!< USART 9 bit data mode */
} USART_OPMODE_Type;

/**  @brief USART Parity type definitions */
typedef enum {
	USART_PARITY_NONE   = 0,        /*!< No parity */
	USART_PARITY_EVEN   = 2,        /*!< Even parity */
	USART_PARITY_ODD    = 3         /*!< Odd parity */
} USART_PARITY_Type;

/**  @brief USART First data bit type definitions (Only SPI mode) */
typedef enum {
	USART_SPI_LSB       = 0,        /*!< SPI LSB First */
	USART_SPI_MSB,                  /*!< SPI MSB First */
} USART_SPI_ORDER_Type;

/**  @brief USART Clock Polarity of ack type definitions Synchronous or SPI mode) */
typedef enum {
	USART_SPI_TX_RISING = 0,        /*!< Txd Change : Rising / Rxd Change : Falling */
	USART_SPI_TX_FALLING,           /*!< Txd Change : Falling / Rxd Change : Rising */
} USART_ACK_Type;

/**  @brief USART Clock Polarity of ack type definitions Synchronous or SPI mode) */
typedef enum {
	USART_SPI_TX_LEADEDGE_SAMPLE  = 0,  /*!< Leading edge : Sample / Trailing edge : Setup */
	USART_SPI_TX_LEADEDGE_SETUP,        /*!< Leading edge : Setup / Trailing edge : Sample */
} USART_EDGE_Type;

/**  @brief USART Clock Polarity of ack type definitions Synchronous or SPI mode) */
typedef enum {
	USART_DATABIT_5     = 0,            /*!< USART 5 bit data mode */
	USART_DATABIT_6,		     		/*!< USART 6 bit data mode */
	USART_DATABIT_7,		     		/*!< USART 7 bit data mode */
	USART_DATABIT_8,                    /*!< USART 8 bit data mode */
	USART_DATABIT_9     = 7             /*!< USART 9 bit data mode */
} USART_DATABIT_Type;

/**  @brief USART Stop bit type definitions */
typedef enum {
	USART_STOPBIT_1		= (0),          /*!< USART 1 Stop Bits Select */
	USART_STOPBIT_2                     /*!< USART 2 Stop Bits Select */
} USART_STOPBIT_Type;

/** @brief USART Data Control type definition */
typedef enum {
	USART_CONTROL_USTRX8	= 0,    // 0
	USART_CONTROL_USTTX8,           // 1
	USART_CONTROL_USTSB,            // 2
	USART_CONTROL_FXCH,             // 3
	USART_CONTROL_USTSSEN,          // 4
	USART_CONTROL_DISSCK,           // 5
	USART_CONTROL_LOOPS,            // 6
	USART_CONTROL_MASTER,           // 7
	USART_CONTROL_DBLS,	            // 8
	USART_CONTROL_USTEN,            // 9
	USART_CONTROL_FPCEN,            // 10
	USART_CONTROL_RTOEN             // 11
} USART_CONTROL_Type;

typedef enum {
	USART_STATUS_PE	= 0,        // 0 
	USART_STATUS_FE,            // 1
	USART_STATUS_DOR,           // 2
	USART_STATUS_WAKE,          // 3
	USART_STATUS_RTO,           // 4
	USART_STATUS_RXC,           // 5
	USART_STATUS_TXC,           // 6
	USART_STATUS_DRE,           // 7
	USART_STATUS_DMARX,         // 8
	USART_STATUS_DMATX          // 9
} USART_STATUS_Type;

typedef enum {
    // USART CR1
    USART_INTCFG_WAKE = 0,	    // 0    /*!< Wake-up Interrupt enable*/
	USART_INTCFG_RXC,           // 1    /*!< Receive Complete Interrupt enable*/
	USART_INTCFG_TXC,           // 2    /*!< Transmit Complete line status interrupt enable*/
	USART_INTCFG_DR,	        // 3    /*!< Data Register Empty interrupt */
    // USART CR2
    USART_INTCFG_RTO,	        // 4    /*!< RTO Interrput enable */
	USART_INTCFG_DMARX,	        // 5    /*!< DMA RX interrupt */
	USART_INTCFG_DMATX	        // 6    /*!< DMA TX interrupt */
} USART_INT_Type;

typedef struct {
  uint32_t Baud_rate;   		/**< USART baud rate */
  USART_OPMODE_Type Mode;
  USART_SPI_ORDER_Type Order;
  USART_ACK_Type ACK;
  USART_EDGE_Type Edge;
  USART_DATABIT_Type Databits; /**< Number of data bits, should be:
							   - USART_DATABIT_5: USART 5 bit data mode
							   - USART_DATABIT_6: USART 6 bit data mode
							   - USART_DATABIT_7: USART 7 bit data mode
							   - USART_DATABIT_8: USART 8 bit data mode
                               - USART_DATABIT_9: USART 9 bit data mode
							   */	
  USART_PARITY_Type Parity;    /**< Parity selection, should be:
							   - USART_PARITY_NONE: No parity
							   - USART_PARITY_ODD: Odd parity
							   - USART_PARITY_EVEN: Even parity
							   - USART_PARITY_SP_1: Forced "1" stick parity
							   - USART_PARITY_SP_0: Forced "0" stick parity
							   */
  USART_STOPBIT_Type Stopbits; /**< Number of stop bits, should be:
							   - USART_STOPBIT_1: USART 1 Stop Bits Select
							   - USART_STOPBIT_2: USART 2 Stop Bits Select
							   */
} USART_CFG_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_USART_Init(USART_Type *USTx, USART_CFG_Type *USART_ConfigStruct);
HAL_Status_Type HAL_USART_DeInit(USART_Type* USTx);
HAL_Status_Type HAL_USART_UART_Mode_Config(USART_CFG_Type *USART_InitStruct);
HAL_Status_Type HAL_USART_USRT_Mode_Config(USART_CFG_Type *USART_InitStruct);
HAL_Status_Type HAL_USART_SPI_Mode_Config(USART_CFG_Type *USART_InitStruct);
HAL_Status_Type HAL_USART_TransmitByte(USART_Type* USTx, uint8_t Data);
uint8_t HAL_USART_ReceiveByte(USART_Type* USTx);

uint32_t HAL_USART_Transmit(USART_Type *USTx, uint8_t *txbuf, uint32_t buflen);
uint32_t HAL_USART_Receive(USART_Type *USTx, uint8_t *rxbuf, uint32_t buflen);

HAL_Status_Type HAL_USART_ConfigInterrupt(USART_Type *USTx, USART_INT_Type USTIntCfg, FunctionalState NewState);
uint32_t HAL_USART_GetStatus(USART_Type* USTx);
HAL_Status_Type HAL_USART_ClearStatus(USART_Type* USTx, USART_STATUS_Type Status);
FlagStatus HAL_USART_CheckBusy(USART_Type *USTx);
HAL_Status_Type HAL_USART_DataControlConfig(USART_Type *USTx, USART_CONTROL_Type Mode, FunctionalState NewState);
HAL_Status_Type HAL_USART_Enable(USART_Type* USTx, FunctionalState state);
HAL_Status_Type HAL_USART_RTOEnable(USART_Type* USTx, FunctionalState state);

#ifdef __cplusplus
}
#endif

#endif /*  */

