/**
* @file		A31G32x_hal_adc.c
* @brief	Contains all functions support for ADC firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/


/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_adc.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief      Initialize ADC(Analog/Digital Convertor) peripheral
* @param      ADCx : ADC peripheral selected, should be
*              - ADC : ADC peripheral
* @param      ADC_ConfigStruct : Pointer contains configuration of ADC
* @return     HAL_Status
*/
HAL_Status_Type HAL_ADC_Init(ADC_Type *ADCx, ADC_CFG_Type *ADC_ConfigStruct)
{
	uint32_t tempreg;

   /* Check ADC handle */
   if(ADCx == NULL)
   {
     return HAL_ERROR;
   }
  
	SCU->PER2&=~(1<<20);
	SCU->PCER2&=~(1<<20);
	/* Set up peripheral clock for ADC module */
	SCU->PER2|=(1<<20);
	SCU->PCER2|=(1<<20);

	tempreg=0;
	switch(ADC_ConfigStruct->Mode){
		case ADC_SINGLE_MODE:
			tempreg = 0
				| (0<<18)									// DMACH
				| (0<<17)									// DMADIABLE
				| ((ADC_ConfigStruct->SamplingTime & 0x1f)<<12)	// STSEL
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)		// SEQCNT
				| (1<<7)									// ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)		// ARST
				| (0<<4)									// ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)			// TRGSRC
				;
			break;
		case ADC_BURST_MODE:
			tempreg = 0
				| (ADC_ConfigStruct->DmaOpt<<18)				// DMACH
				| (1<<17)									// DMAEN
				| ((ADC_ConfigStruct->SamplingTime & 0x1f)<<12)	// STSEL //
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)		// SEQCNT//	| ((ADC_ConfigStruct->SeqCnt+3)<<8)  // SEQCNT
				| (1<<7)									// ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)		// ARST
				| (1<<4)									// ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)			// TRGSRC
				;
			break;
		case ADC_MULTI_MODE:
			tempreg = 0
				| (ADC_ConfigStruct->DmaOpt<<18)				// DMACH
				| (1<<17)									// DMAEN
				| ((ADC_ConfigStruct->SamplingTime & 0x1F)<<12)	// STSEL
				| (((ADC_ConfigStruct->SeqCnt-1) & 7)<<8)		// SEQCNT//	| ((ADC_ConfigStruct->SeqCnt+3)<<8)  // SEQCNT
				| (1<<7)									// ADCEN
				| (((ADC_ConfigStruct->RestartEn) & 1)<<6)		// ARST
				| (2<<4)									// ADCMOD
				| (((ADC_ConfigStruct->TrgSel) & 3)<<0)			// TRGSRC
				;	
		default:
			break;
	}
	tempreg |=  (1<<21)	// TRGINFO ( Trigger information option enable )
		| (1<<20)		// CHINFO ( Channel information option enable )
		| (1<<17);		// DMAEN ( DMA Enable bit)
	ADCx->MR = tempreg;

	tempreg = 0
		| (ADC_ConfigStruct->InClkDiv<<8)		// CKDIV  48MHz
		| (0<<7)		// ADCPD
		| (0<<5)		// CLKINVT
		;

	ADCx->CCR = tempreg;
	ADCx->CSCR = 0;

	ADCx->CR =ADC_CR_STOP_MASK;
	
	return HAL_OK;
}


/**
* @brief      De-Initialize ADC peripheral
* @param      ADCx : ADC peripheral selected, should be
*              - ADC : ADC peripheral
* @return     HAL_Status
*/
HAL_Status_Type HAL_ADC_DeInit(ADC_Type *ADCx)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->CR = (1<<7); // stop
	ADCx->CCR = 0;
	ADCx->IER = 0;
	ADCx->MR = 0;
	
	// Disable ADC peripheral & clock
	SCU->PER2&=~(1<<20);
	SCU->PCER2&=~(1<<20);
	return HAL_OK;
}

/**
* @brief      Start A/D conversion
* @param      ADCx : ADC peripheral selected, should be
*              - ADC : ADC peripheral
* @return     HAL_Status
*/
HAL_Status_Type HAL_ADC_Start(ADC_Type *ADCx)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->CR =ADC_CR_START_MASK;
	return HAL_OK;
}

/**
* @brief      Stop A/D conversion  
*             If this function called after a conversion cycle starts,  
*             the current conversion is completed
* @param      ADCx : ADC peripheral selected, should be
*              - ADC : ADC peripheral
* @return     HAL_Status
*/
HAL_Status_Type HAL_ADC_Stop(ADC_Type *ADCx)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->CR =ADC_CR_STOP_MASK;
	return HAL_OK;
}

/**
* @brief		Get Result conversion from A/D data register
* @param      	ADCx : ADC peripheral selected, should be
*              	 - ADC : ADC peripheral
* @return		Result of ADC conversion data
*/
uint16_t HAL_ADC_GetData(ADC_Type *ADCx)
{
	uint16_t adc_value;
	
    /* Check ADC handle */
   if(ADCx == NULL)
   {
     return HAL_ERROR;
   }

	adc_value = ADCx->DDR;
	return ADC_DR_RESULT(adc_value);
	
}


/**
* @brief		Set ADC conversion in power mode
* @param      	ADCx : ADC peripheral selected, should be
*              	 - ADC : ADC peripheral
* @param		NewState
* 				 - DISABLE : AD converter is normal mode
* 				 - ENABLE: AD Converter is in power down mode
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_EnterPowerdownMode(ADC_Type *ADCx, FunctionalState NewState)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->CCR &= ~ADC_CCR_PDN;
	if (NewState){
		ADCx->CCR |= ADC_CCR_PDN;
	}
	return HAL_OK;
}

/**
* @brief		ADC interrupt configuration
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		IntType: type of interrupt, should be:
* 					- ADC_INTEN_DMA
* 					- ADC_INTEN_TRIGGER
* 					- ADC_INTEN_SEQ
* 					- ADC_INTEN_SINGLE
* @param		NewState:
* 					- ENABLE : enable ADC interrupt
* 					- DISABLE: disable ADC interrupt
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_ConfigInterrupt(ADC_Type *ADCx, uint8_t IntType, FunctionalState NewState)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->IER &= ~IntType;
	if (NewState){
		ADCx->IER |= IntType;
	}
	return HAL_OK;
}

/**
* @brief		Select ADC channel number(1st of channel selection registers)
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		Channel(0~15) : Sequence index selected, should be
*				 - SEQ3[28:24] : channel << 24
*				 - SEQ2[20:16] : channel << 16
*				 - SEQ1[12:8]  : channel <<  8
*				 - SEQ0[4:0]   : channel <<  0
*				Use Bitwise OR(|) to set SEQ0~3
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_ChannelSel_1(ADC_Type *ADCx, uint32_t Channel)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->SCSR1 = Channel;
	return HAL_OK;
}

/**
* @brief		Select ADC channel number(2nd of channel selection registers)
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		Channel(0~15) : Sequence index selected, should be
*				 - SEQ7[28:24] : channel << 24
*				 - SEQ6[20:16] : channel << 16
*				 - SEQ5[12:8]  : channel <<  8
*				 - SEQ4[4:0]   : channel <<  0
*				Use Bitwise OR(|) to set SEQ0~3
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_ChannelSel_2(ADC_Type *ADCx, uint32_t Channel)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->SCSR2 = Channel;
	return HAL_OK;
}

/**
* @brief		Select ADC Trigger source
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		Trigger(0~15) : Trigger source(8th) selected, should be
*				 - SEQTRG[num] : Trigger << (4*num)
*				Use Bitwise OR(|) to set SEQTRG0~7
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_TriggerlSel(ADC_Type *ADCx, uint32_t Trigger)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->TRG = Trigger;
	return HAL_OK;
}

/**
* @brief		Get information of ADC status
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @return		ADC status register(ADC_SR) 
*                - ADC_STAT_COMPIFLG   : Compare Interrupt
*                - ADC_STAT_DMAOVERRUN : ADC DMA OVERRUN Interrupt
*                - ADC_STAT_DMA        : ADC DMA Done Interrupt
*                - ADC_STAT_TRIGGER    : ADC Trigger Interrupt
*                - ADC_STAT_SEQ        : ADC Sequence Interrupt
*                - ADC_STAT_SINGLE     : ADC Single Interrupt
*/
uint32_t HAL_ADC_GetStatus(ADC_Type *ADCx)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	return (ADCx->SR);
}

/**
* @brief		Clear ADC status, register(ADC_SR) 
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		status : ADC status register(ADC_SR)
*                - ADC_STAT_COMPIFLG   : Compare Interrupt
*                - ADC_STAT_DMAOVERRUN : ADC DMA OVERRUN Interrupt
*                - ADC_STAT_DMA        : ADC DMA Done Interrupt
*                - ADC_STAT_TRIGGER    : ADC Trigger Interrupt
*                - ADC_STAT_SEQ        : ADC Sequence Interrupt
*                - ADC_STAT_SINGLE     : ADC Single Interrupt
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_ClearStatus(ADC_Type *ADCx, uint32_t status)
{
	/* Check ADC handle */
	if(ADCx == NULL)
	{
	  return HAL_ERROR;
	}

	ADCx->SR=status;
	return HAL_OK;
}

/**
* @brief		Configure ADC Channel Compare
* @param		ADCx pointer to ADC_Type, should be: ADCx
* @param		ADC_Compare : Pointer contains configuration of ADC_CMPR_Type
* @return		HAL_Status
*/
HAL_Status_Type HAL_ADC_Channel_Compare(ADC_Type *ADCx, ADC_CMPR_Type *ADC_Compare)
{

	uint32_t temp;
    /* Check ADC handle */
   if(ADCx == NULL)
   {
     return HAL_ERROR;
   }
	temp = 0
		| (ADC_Compare->COMPIEN << ADC_CMPR_COMPIEN_Pos)
		| (ADC_Compare->COMPEN << ADC_CMPR_COMPEN_Pos)
		| (ADC_Compare->LTE << ADC_CMPR_LTE_Pos)
		| (ADC_Compare->CCH << ADC_CMPR_CCH_Pos)
		| (ADC_Compare->CVAL << ADC_CMPR_CVAL_Pos)
		;
	ADC->CMPR &= 0;
	ADC->CMPR |= temp;
	return HAL_OK;
	
}

void HAL_ADC_Start_DMA(ADC_Type *pADCx){}
void HAL_ADC_Stop_DMA(ADC_Type *pADCx){}
void HAL_ADC_Start_IT(ADC_Type *pADCx){}
void HAL_ADC_Stop_IT(ADC_Type *pADCx){}



/* --------------------------------- End Of File ------------------------------ */

