/**
* @file		A31G32x_hal_comp.c
* @brief	Contains all functions support for Comparator firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/


/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_comp.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief      Initialize comparator peripheral
* @param      None
* @return     None
*/
HAL_Status_Type HAL_COMP_Init(void)
{
	SCU->PER2 &= ~(1<<23);
	SCU->PCER2 &= ~(1<<23);
	
	SCU->PER2 |= (1<<23);
	SCU->PCER2 |= (1<<23);	
	//comp init 
	return HAL_OK;
}

/**
* @brief      De-Initialize comparator peripheral
* @param      None
* @return     None
*/
HAL_Status_Type HAL_COMP_DeInit(void)
{
	
	COMP->COMP0CR &= ~COMP_CR_EN;
	COMP->COMP1CR &= ~COMP_CR_EN;
	
	SCU->PER2 &= ~(1<<23);
	SCU->PCER2 &= ~(1<<23);
	
	return HAL_OK;
}

/**
* @brief      Start comparator
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_COMP_Start(void)
{
	return HAL_OK;

}

/**
* @brief      Stop comparator
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_COMP_Stop(void)
{
	return HAL_OK;
}
/**
* @brief      Start comparator for interrupt
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_COMP_Start_IT(void)
{
	return HAL_OK;

}


/**
* @brief      Stop comparator for interrupt
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_COMP_Stop_IT(void)
{
	return HAL_OK;
}



/**
* @brief		Get Comparator Interrupt Status
* @param		comp_no : Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @return		COMPIST register value
*/
uint32_t HAL_COMP_GetStatus_IT(uint32_t comp_no)
{
	if(comp_no==COMP0)
	{
		return COMP->COMPIST & 0x01;
	}
	else {
		return COMP->COMPIST & 0x02;
	}
}

/**
* @brief		Clear Comparator interrupt status 
* @param		comp_no : Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_ClearStatus_IT(uint32_t comp_no)
{
	if(comp_no==COMP0){
		COMP->COMPICLR = COMP_ICLR_C0ICLR ;
	}
	else{
		COMP->COMPICLR = COMP_ICLR_C1ICLR;
	}
	return HAL_OK;
}

/**
* @brief		Clear All Comparator interrupt status
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_COMP_ClearStatusAll_IT()
{
	COMP->COMPICLR = COMP_ICLR_BITMASK;
	return HAL_OK;
}

/**
* @brief		Configure comparator hysteris 
* @param		comp_no	: Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param		hyssel	: Comparator hysterisis select, should be :
*						- COMP_CR_HYSSEL_OFF = 0,    					
*						- COMP_CR_HYSSEL_LOW = 1,     						
*						- COMP_CR_HYSSEL_MEDIUM = 2,    			
*						- COMP_CR_HYSSEL_HIGH = 3    				
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_ConfigHysterisis(uint32_t comp_no, uint32_t hyssel)
{
	if(comp_no==COMP0){
		COMP->COMP0CR = (COMP->COMP0CR & ~(COMP_CR_HYSSEL_BITMASK));
		COMP->COMP0CR |= (hyssel<<16);
	}
	else{
		COMP->COMP1CR = (COMP->COMP1CR & ~(COMP_CR_HYSSEL_BITMASK));
		COMP->COMP1CR |= (hyssel<<16);
	}
	return HAL_OK;
}
	
/**
* @brief		Configure comparator reference/input voltage
* @param		comp_no	: Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param		inputsel : Comparator Reference(input -) Selection bit, should be :
*							- COMP_INPUT_CP0_N2	= 0,   					
*							- COMP_INPUT_CP0_N0,
*							- COMP_INPUT_CP0_N1,	
*							- COMP_INPUT_Vref12MV,
*							- COMP_INPUT_Vref9MV,
*							- COMP_INPUT_Vref6MV,
*							- COMP_INPUT_Vref3MV,
*							- COMP_INPUT_DAC_OUT3
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_ConfigInputLevel(uint32_t comp_no, uint32_t inputsel)
{
	if(comp_no==COMP0){
		COMP->COMP0CR = (COMP->COMP0CR & ~(COMP_CR_INNSEL_BITMASK));
		COMP->COMP0CR |= (inputsel<<4);
	}
	else{
		COMP->COMP1CR = (COMP->COMP1CR & ~(COMP_CR_INNSEL_BITMASK));
		COMP->COMP1CR |= (inputsel<<4);
	}
	return HAL_OK;
}

/**
* @brief		Enable/disable Comparator operate
* @param		comp_no	: Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param		NewState : enable/disable should be,
* 						- DISABLE= 0
* 						- ENABLE	= 1
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_Cmd(uint32_t comp_no, FunctionalState NewState)
{
	if(comp_no==COMP0){
		COMP->COMP0CR = (COMP->COMP0CR & ~COMP_CR_EN);
		if(NewState == ENABLE){
			COMP->COMP0CR |= COMP_CR_EN;
		}
	}
	else{
		COMP->COMP1CR = (COMP->COMP1CR & ~COMP_CR_EN);
		if(NewState == ENABLE){
			COMP->COMP1CR |= COMP_CR_EN;
		}
	}
	return HAL_OK;
}

/**
* @brief		Set Comparator Debounce 
* @param		comp_no	: Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param		dbncshift	Debounce shift selection, should be in range from 0 to 0xF
* @param		dbncclksel	Debounce clock selection, should be in range from SYSTEMCLK, LSI750
* @param		dbnccounter	Debounce time base counter, should be in range from 0 to 0xFFFF
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_ConfigDebounce(uint32_t comp_no, uint32_t dbncshift, uint32_t dbncclksel, uint32_t dbnccounter)
{
	if(comp_no==COMP0){
		COMP->COMPDBNC &=  ~(COMP_DBNC_C0DBNC_BITMASK | COMP_DBNC_DBNCTB_BITMASK);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbncshift << 0);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbncclksel << 8);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbnccounter << 16);

	}
	else{
		COMP->COMPDBNC &= ~(COMP_DBNC_C1DBNC_BITMASK | COMP_DBNC_DBNCTB_BITMASK);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbncshift << 4);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbncclksel << 8);
		COMP->COMPDBNC = COMP->COMPDBNC | (dbnccounter << 16);
	}
	return HAL_OK;
}

/**
* @brief		Enable/disable Comparator Interrupt
* @param		comp_no	: Comparator block number selected, should be
*                - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_COMP_InterruptCmd(uint32_t comp_no, FunctionalState NewState)
{
	if(comp_no==COMP0){
		if(NewState == ENABLE){
			COMP->COMPIEN |= COMP_IEN_C0_EN;
		}
		else{
			COMP->COMPIEN &= ~COMP_IEN_C0_EN;
		}
	}
	else{
		if(NewState == ENABLE){
			COMP->COMPIEN |= COMP_IEN_C1_EN;
		}
		else{
			COMP->COMPIEN &= ~COMP_IEN_C1_EN;
		}
	}
	return HAL_OK;
}

/**
* @brief	Configure comparator interrupt 
* @param	comp_no	: Comparator block number selected, should be
*  		         - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param	tpol :	Comparator Trigger output polarity(to trigger other IP), should be
*				- COMP_TRIOUT_NORMAL		= 0   					
*				- COMP_TRIOUT_INVERTED		= 1
* @param	ipol :	Comparator interrupt polarity(level mode), should be
*				- COMP_INTOUT_HIGH		= 0 					
*				- COMP_INTOUT_LOW		= 1
* @param	mode :	Comparator Interrupt mode select, should be
*				- ICON_LEVEL				= 0     		
*				- ICON_RISING_EDGE_INT	    = 1		     			
*				- ICON_FALLING_EDGE_INT		= 2		     		
*				- ICON_BOTH_EDGE_INT		= 3
* @return	HAL_Status
*/
HAL_Status_Type HAL_COMP_ConfigInterrupt(uint32_t comp_no, uint32_t tpol, uint32_t ipol, uint32_t mode)
{
	if(comp_no==COMP0){
		COMP->COMPICON = COMP->COMPICON & ~(COMP_ICON_IPOL0_BITMASK | COMP_ICON_TPOL0_BITMASK | COMP_ICON_C0IMODE_BITMASK);
		COMP->COMPICON |= (tpol << 12);
		COMP->COMPICON |= (ipol << 8);
		COMP->COMPICON |= (mode << 0);
	}
	else{
		COMP->COMPICON = COMP->COMPICON & ~(COMP_ICON_IPOL1_BITMASK | COMP_ICON_TPOL1_BITMASK | COMP_ICON_C1IMODE_BITMASK);
		COMP->COMPICON |= (tpol << 13);
		COMP->COMPICON |= (ipol << 9);
		COMP->COMPICON |= (mode << 2);
	}
	return HAL_OK;
}

/**
* @brief	Comparator Mode selection
* @param	comp_no	: Comparator block number selected, should be
*  		         - CMP0 : Comarator 0 block
*                - CMP1 : Comarator 1 block
* @param	modesel	: 	should be
*						- COMP_MODE_UL = 0
*						- COMP_MODE_Low = 1
*						- COMP_MODE_Medium = 2
*						- COMP_MODE_High = 3
* @return		HAL_Status
*/
HAL_Status_Type HAL_COMP_SetModeLevel(uint32_t comp_no, uint32_t modesel)
{
	if(comp_no==COMP0){
		COMP->COMP0CR |= (modesel<<20); 
	}
	else{
		COMP->COMP1CR |= (modesel<<20); 
	}
	return HAL_OK;
}

/**
* @brief		Enable/disable Comparator Internal Reference voltage
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_COMP_SetVoltage(FunctionalState NewState)
{
	if(NewState == ENABLE){
		COMP->COMP0CR |= COMP_CR_PREF_EN;
	}
	else{
		COMP->COMP0CR &= ~COMP_CR_PREF_EN;
	}
	return HAL_OK;
}

/**
* @brief		Enable/disable Comparator Window Mode
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_COMP_SetWindowMode(FunctionalState NewState)
{
	if(NewState == ENABLE){
		COMP->COMP0CR |= COMP_CR_WIND_EN;
	}
	else{
		COMP->COMP0CR &= ~COMP_CR_WIND_EN;
	}
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */
