/**
* @file		A31G32x_hal_crc.c
* @brief	Contains all functions support for CRC firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/


/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_crc.h"


/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/


/**
* @brief      Initialize CRC peripheral
* @param      None
* @return     HAL_Status
*/
HAL_Status_Type HAL_CRC_Init(void)
{
    SCU->PER2 &=~(1UL<<31);
	SCU->PCER2&=~(1UL<<31);
    /* Set up peripheral clock for CRC module */
    SCU->PER2 |= (1UL<<31);
    SCU->PCER2|= (1UL<<31);
	
	return HAL_OK;
}

/**
* @brief      DeInitialize CRC peripheral
* @param      None
* @return     HAL_Status
*/
HAL_Status_Type HAL_CRC_DeInit(void)
{
	// Disable CRC peripheral & clock
	SCU->PER2 &= ~(1UL<<31);
	SCU->PCER2 &= (1UL<<31);
	
	return HAL_OK;
}


/**
* @brief		CRC/Checksum User Mode Start
*				This function sets the control register for CRC/Checksum User mode
* @param		u32SEL		CRC/Checksum Selection			:	CRC_CRC, CRC_CHECKSUM
* @param		u32POLY		CRC Polynomial Selection		:	CRC_CCITT, CRC_16
* @param		u32AdrInc	Auto Increment of Start Address	:	CRC_NOINC, CRC_AUTOINC
* @param		u321stBit	CRC 1st Shifted-in bit			:	CRC_MSB, CRC_LSB
*
* @return		HAL_Status
*/
HAL_Status_Type HAL_CRC_ConfigUserMode(uint32_t u32SEL, uint32_t u32POLY, uint32_t u32FirstBit)
{
	CRC->CR |= (0x1uL<<CRC_CR_RLTCLR_Pos)                              
	             |CRC_USER_M     										                                 
	             |u32SEL
	             |u32POLY
	             |u32FirstBit;

	return HAL_OK;

}

/**
* @brief		CRC/Checksum Input on User Mode
* @param		u32Input	:	CRC/Checksum Input
* @return		HAL_Status
*/
uint32_t HAL_CRC_UserInput(uint32_t u32Input)
{
	CRC_InData(u32Input);
	return HAL_OK;
}

/**
* @brief	CRC/Checksum Get Data
* @param	None
* @return	CRC/Checksum Result
*/
uint32_t HAL_CRC_GetData(void)
{
	uint32_t	u32Result;
	
	u32Result = CRC->RLT;
	
	return u32Result;
}


/**
* @brief		CRC/Checksum User Init Setting
* @param		InitVal	 :	CRC/Checksum Init value
* @return		HAL_Status
*/
HAL_Status_Type HAL_CRC_SetUserInput(uint32_t InitVal)
{
	CRC->INIT = InitVal;
	return HAL_OK;
}


/**
* @brief		CRC/Checksum Interrupt Setting
* @param		IntSel	:  CRC/Checksum Interrup Setting
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_CRC_ConfigInterrupt(uint32_t IntSel)
{
	uint32_t	tempreg;
	
	tempreg = CRC->CR;
	
	tempreg &= ~(1<<9);
	
	if (IntSel)
	{
		tempreg |= (1<<9);
	}
	
	CRC->CR = tempreg;
	return HAL_OK;
}

/**
* @brief		Get CRC/Checksum Interrupt Status
* @param		None
* @return		CRC interrupt status
*/
uint32_t HAL_CRC_GetState_IT(void)
{
	return (CRC->CR & (1<<8));
}

/**
* @brief		Clear CRC/Checksum Interrupt Status
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_CRC_ClearState_IT(void)
{
	CRC->CR |= (1<<8);
	return HAL_OK;
}


/* --------------------------------- End Of File ------------------------------ */

