/**
* @file		A31G32x_hal_dac.c
* @brief	Contains all functions support for dac firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_dac.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief	Initial for DAC
* @param	DACx pointer to DAC_Type, should be: DACx
* @param	DAC_ConfigStruct : Pointer contains configuration of DAC
* @param	DAC_OutSel 	: 	should be
*						- 0 -> Disable
*						- 1 -> COMP1 Enable
*						- 2 -> COMP2 Enable
*						- 3 -> Not Support
* @return	HAL_Status
*/
HAL_Status_Type HAL_DAC_Init(DAC_Type *DACx, DAC_CFG_Type *DAC_ConfigStruct, uint32_t DAC_OutSel)
{
	uint32_t	tempreg;

	
	SCU->PER2 &= ~(1<<22);
	SCU->PCER2 &= ~(1<<22);
	
	// Set-up Peripheral Clock for DAC module
	SCU->PER2 |= (1<<22);
	SCU->PCER2 |= (1<<22);
	
	// PGA Control Setting
	HAL_DAC_SetGain(DAC_ConfigStruct->PGASel);
	
	// DAC Control Setting
	if(DAC_OutSel == 1)
	{
		tempreg = 0
		| (1<<11)		// DAC_OUT1 output enable
		| ((DAC_ConfigStruct->RefSel&(0x1))<<10)		// DAC Buffer Selection
		| ((DAC_ConfigStruct->RefSel&(0x1))<<8)		// DAC Reference Selection
		| ((DAC_ConfigStruct->IntSel&(0x1))<<7)		// DAC Interrupt Enable/Disable
		| ((DAC_ConfigStruct->DataChangeSel&(0x1))<<5)	// DAC Data Change Selection
		| (1<<4)								// DAC Clear Buffer
		| ((DAC_ConfigStruct->ReloadSel&(0x3))<<1)	// DAC Reload Selection
		| (0<<0)								// D/A Converter Disable
		;
	}
	else if(DAC_OutSel == 2)
	{
		tempreg = 0
		| (1<<12)		// DAC_OUT2 output enable
		| ((DAC_ConfigStruct->RefSel&(0x1))<<10)		// DAC Buffer Selection
		| ((DAC_ConfigStruct->RefSel&(0x1))<<8)		// DAC Reference Selection
		| ((DAC_ConfigStruct->IntSel&(0x1))<<7)		// DAC Interrupt Enable/Disable
		| ((DAC_ConfigStruct->DataChangeSel&(0x1))<<5)	// DAC Data Change Selection
		| (1<<4)								// DAC Clear Buffer
		| ((DAC_ConfigStruct->ReloadSel&(0x3))<<1)	// DAC Reload Selection
		| (0<<0)								// D/A Converter Disable
		;
	}
	else if(DAC_OutSel == 3)
	{
		tempreg = 0
		| (1<<13)		// DAC_OUT3 output enable
		| ((DAC_ConfigStruct->RefSel&(0x1))<<10)		// DAC Buffer Selection
		| ((DAC_ConfigStruct->RefSel&(0x1))<<8)		// DAC Reference Selection
		| ((DAC_ConfigStruct->IntSel&(0x1))<<7)		// DAC Interrupt Enable/Disable
		| ((DAC_ConfigStruct->DataChangeSel&(0x1))<<5)	// DAC Data Change Selection
		| (1<<4)								// DAC Clear Buffer
		| ((DAC_ConfigStruct->ReloadSel&(0x3))<<1)	// DAC Reload Selection
		| (0<<0)								// D/A Converter Disable
		;
	}
	else
	{
		tempreg = 0
		| (0<<13)		// DAC_OUT3 output disable
		| (0<<12)		// DAC_OUT3 output disable
		| (0<<11)		// DAC_OUT3 output disable
		| ((DAC_ConfigStruct->BufSel&(0x1))<<10)		// DAC Buffer Selection
		| ((DAC_ConfigStruct->RefSel&(0x1))<<8)		// DAC Reference Selection
		| ((DAC_ConfigStruct->IntSel&(0x1))<<7)		// DAC Interrupt Enable/Disable
		| ((DAC_ConfigStruct->DataChangeSel&(0x1))<<5)	// DAC Data Change Selection
		| (1<<4)								// DAC Clear Buffer
		| ((DAC_ConfigStruct->ReloadSel&(0x3))<<1)	// DAC Reload Selection
		| (0<<0)								// D/A Converter Disable
		;
	}
	DAC->CR = tempreg;
	
	
	// DAC Offset Setting
	HAL_DAC_SetOffset(DAC_ConfigStruct->OffsetSel, DAC_ConfigStruct->OffsetDIRSel, DAC_ConfigStruct->OffsetVal);
	DAC->OFSCR = tempreg;
	return HAL_OK;
	
}

/**
* @brief	DeInitialize for DAC
* @param	DACx pointer to DAC_Type, should be: DACx
* @return	HAL_Status
*/
HAL_Status_Type HAL_DAC_DeInit(DAC_Type *DACx)
{
	DAC->CR = 0;
	
	SCU->PER2 &= ~(1<<22);
	SCU->PCER2 &= ~(1<<22);
	return HAL_OK;
}

/**
* @brief		Set start mode for DAC
* @param		DACx pointer to DAC_Type, should be: DACx
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_DAC_Start(DAC_Type *DACx, FunctionalState NewState)
{
	DACx->CR |= (NewState<<0);
	return HAL_OK;
}

/**
* @brief      Stop DAC
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_DAC_Stop(void)
{
	return HAL_OK;

}

/**
* @brief      Start D/A conversion DMA
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_DAC_Start_DMA(void)
{
	return HAL_OK;
}

/**
* @brief      Stop D/A conversion DMA
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type HAL_DAC_Stop_DMA(void)
{
	return HAL_OK;
}


/**
* @brief	Get D/A Data Register
* @param	DACx pointer to DAC_Type, should be: DACx
* @return	Result of DAC value
*/
uint16_t	HAL_DAC_GetData(DAC_Type *DACx)
{
	uint16_t	dac_value;
	
	dac_value = (DACx->DR>>4);
	
	return DAC_DR_RESULT(dac_value);
}


/**
* @brief		Set D/A Data Register
* @param		DACx pointer to DAC_Type, should be: DACx
* @param		DAC value(0x0 ~ 0x3FF)
* @return		None
*/
void	HAL_DAC_SetData(DAC_Type *DACx, uint16_t data_value)
{
	DACx->DR = (data_value<<4);
}

/**
* @brief		DAC interrupt configuration
* @param		DACx pointer to DAC_Type, should be: DACx
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_DAC_ConfigInterrup(DAC_Type *DACx, FunctionalState NewState)
{
	uint32_t	tempreg;
	
	tempreg = DACx->CR;
	tempreg &= ~(1<<7);
	
	if (NewState)
	{
		tempreg |= (1<<7);
	}
	DACx->CR = tempreg;
	return HAL_OK;
	
}


/**
* @brief		Get DAC status 
* @param		DACx pointer to DAC_Type, should be: DACx
* @return		DAC status register
*/
uint32_t	HAL_DAC_GetState(DAC_Type *DACx)
{
	return ((DACx->CR>>6)&1);
}


/**
* @brief		Clear DAC Chanel status
* @param		DACx pointer to DAC_Type, should be: DACx
* @return		HAL_Status
*/
HAL_Status_Type HAL_DAC_ClearState(DAC_Type *DACx)
{
	uint32_t	tempreg;
	
	tempreg = DACx->CR;
	tempreg &= ~(1<<6);
	
	DACx->CR = tempreg;
	return HAL_OK;
}


/**
* @brief		Get DAC Buffer
* @param		DACx pointer to DAC_Type, should be: DACx
* @return		The value of DAC Buffer register
*/
uint16_t	HAL_DAC_GetBuffer(DAC_Type *DACx)
{
	uint16_t	dac_fvalue;
	
	dac_fvalue = (DACx->BR>>4);
	
	return (dac_fvalue);
}

/**
* @brief		Set DAC output offset for DAC
* @param		enable	: Offset enable = 1 / disable = 0
* @param		dir		: Offset direction
*							0 - Minus (-)
*							1 - Plus (+)
* @param		value	: Offset value (0x0 ~ 0xF)
* @return		HAL_Status
*/
HAL_Status_Type HAL_DAC_SetOffset(uint32_t enable, uint32_t dir, uint32_t value){
	uint32_t tempreg;
	tempreg = 0
	| ((enable&(0x1))<<7)	// DAC Offset Enable/Disable
	| ((dir&(0x1))<<6)	// DAC Offset Direction Selection
	| ((value&(0xF))<<0)	// DAC Offset Value
	;
	
	DAC->OFSCR = tempreg;
	return HAL_OK;
}

/**
* @brief      Set offset of DAC output
* @param      pDACx : DAC peripheral selected, should be
*              - DAC : DAC peripheral
* @param      Gain : programmable gain, should be (-30dB ~ +30dB)
*              - DAC_GAIN_M_30DB : - 30dB
*              - DAC_GAIN_0DB    :    0dB
*              - DAC_GAIN_P_30DB : + 30dB
* @return     HAL_Status
*/
HAL_Status_Type HAL_DAC_SetGain(uint32_t gain){
	DAC->PGSR = gain;
	return HAL_OK;
}



/* --------------------------------- End Of File ------------------------------ */

