/**
* @file		A31G32x_hal_dac.c
* @brief	Contains all functions support for dac firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _DEBUG_FRMWRK_
#define _DEBUG_FRMWRK_


/*******************************************************************************
* Included File
*******************************************************************************/
#include <stdarg.h>
#include <stdio.h>
#include "A31G32x_hal_debug_frmwrk.h"
#include "A31G32x_hal_pcu.h"


/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
#ifdef _DEBUG_MSG
void (*_db_msg)(USART_Type *USARTx, const void *s);	// USARTPuts
void (*_db_msg_)(USART_Type *USARTx, const void *s);	// USARTPuts_   (add /n/r)
void (*_db_char)(USART_Type *USARTx, uint8_t ch);		// USARTPutChar
void (*_db_dec)(USART_Type *USARTx, uint8_t decn);		// USARTPutDec
void (*_db_dec_16)(USART_Type *USARTx, uint16_t decn);	// USARTPutDec16
void (*_db_dec_32)(USART_Type *USARTx, uint32_t decn);	// USARTPutDec32
void (*_db_hex)(USART_Type *USARTx, uint8_t hexn);		// USARTPutHex
void (*_db_hex_16)(USART_Type *USARTx, uint16_t hexn);	// USARTPutHex16
void (*_db_hex_32)(USART_Type *USARTx, uint32_t hexn);	// USARTPutHex32
uint8_t (*_db_get_char)(USART_Type *USARTx);			// USARTGetChar
uint8_t (*_db_get_ch)(USART_Type *USARTx, uint8_t *ch);	// USARTGetCh

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Puts a character to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		ch		Character to put
* @return		None
*/
void USARTPutChar (USART_Type *USARTx, uint8_t ch)
{
	HAL_USART_Transmit(USARTx, &ch, 1);
}

/**
* @brief		Get a character to USART port
* @param		USARTx	Pointer to USART peripheral
* @return		character value that returned
*/
uint8_t USARTGetChar (USART_Type *USARTx)
{
	uint8_t tmp = 0;
	HAL_USART_Receive(USARTx, &tmp, 1);
	return(tmp);
}

/**
* @brief		Get a character to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		*ch		Character to gett
* @return		if getting value, return '1'. if not, return '0'
*/
uint8_t USARTGetCh(USART_Type *USARTx, uint8_t *ch)
{
	if (!(USARTx->ST & UST_SR_RXC)){
		*ch=0;
		return(0);		
	}
	else {
		*ch = HAL_USART_ReceiveByte(USARTx);
		return(1);	
	}
}


/**
* @brief		Puts a string to USART port
* @param		USARTx 	Pointer to USART peripheral
* @param		str 	string to put
* @return		None
*/
void USARTPuts(USART_Type *USARTx, const void *str)
{
	uint8_t *s = (uint8_t *) str;

	while (*s)
	{
		USARTPutChar(USARTx, *s++);
	}
}


/**
* @brief		Puts a string to USART port and print new line
* @param		USARTx	Pointer to USART peripheral
* @param		str		String to put
* @return		None
*/
void USARTPuts_(USART_Type *USARTx, const void *str)
{
	USARTPuts (USARTx, str);
	USARTPuts (USARTx, "\n\r");
}


/**
* @brief		Puts a decimal number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		decnum	Decimal number (8-bit long)
* @return		None
*/
void USARTPutDec(USART_Type *USARTx, uint8_t decnum)
{
	uint8_t c1=decnum%10;
	uint8_t c2=(decnum/10)%10;
	uint8_t c3=(decnum/100)%10;
	USARTPutChar(USARTx, '0'+c3);
	USARTPutChar(USARTx, '0'+c2);
	USARTPutChar(USARTx, '0'+c1);
}

/**
* @brief		Puts a decimal number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		decnum	Decimal number (8-bit long)
* @return		None
*/
void USARTPutDec16(USART_Type *USARTx, uint16_t decnum)
{
	uint8_t c1=decnum%10;
	uint8_t c2=(decnum/10)%10;
	uint8_t c3=(decnum/100)%10;
	uint8_t c4=(decnum/1000)%10;
	uint8_t c5=(decnum/10000)%10;
	USARTPutChar(USARTx, '0'+c5);
	USARTPutChar(USARTx, '0'+c4);
	USARTPutChar(USARTx, '0'+c3);
	USARTPutChar(USARTx, '0'+c2);
	USARTPutChar(USARTx, '0'+c1);
}

/**
* @brief		Puts a decimal number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		decnum	Decimal number (8-bit long)
* @return		None
*/
void USARTPutDec32(USART_Type *USARTx, uint32_t decnum)
{
	uint8_t c1=decnum%10;
	uint8_t c2=(decnum/10)%10;
	uint8_t c3=(decnum/100)%10;
	uint8_t c4=(decnum/1000)%10;
	uint8_t c5=(decnum/10000)%10;
	uint8_t c6=(decnum/100000)%10;
	uint8_t c7=(decnum/1000000)%10;
	uint8_t c8=(decnum/10000000)%10;
	uint8_t c9=(decnum/100000000)%10;
	uint8_t c10=(decnum/1000000000)%10;
	USARTPutChar(USARTx, '0'+c10);
	USARTPutChar(USARTx, '0'+c9);
	USARTPutChar(USARTx, '0'+c8);
	USARTPutChar(USARTx, '0'+c7);
	USARTPutChar(USARTx, '0'+c6);
	USARTPutChar(USARTx, '0'+c5);
	USARTPutChar(USARTx, '0'+c4);
	USARTPutChar(USARTx, '0'+c3);
	USARTPutChar(USARTx, '0'+c2);
	USARTPutChar(USARTx, '0'+c1);
}

/**
* @brief		Puts a hex number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		hexnum	Hex number (8-bit long)
* @return		None
*/
void USARTPutHex (USART_Type *USARTx, uint8_t hexnum)
{
	uint8_t nibble, i;

//	USARTPuts(USARTx, "0x");
	i = 1;
	do {
		nibble = (hexnum >> (4*i)) & 0x0F;
		USARTPutChar(USARTx, (nibble > 9) ? ('A' + nibble - 10) : ('0' + nibble));
	} while (i--);
}


/**
* @brief		Puts a hex number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		hexnum	Hex number (16-bit long)
* @return		None
*/
void USARTPutHex16 (USART_Type *USARTx, uint16_t hexnum)
{
	uint8_t nibble, i;

//	USARTPuts(USARTx, "0x");
	i = 3;
	do {
		nibble = (hexnum >> (4*i)) & 0x0F;
		USARTPutChar(USARTx, (nibble > 9) ? ('A' + nibble - 10) : ('0' + nibble));
	} while (i--);
}

/**
* @brief		Puts a hex number to USART port
* @param		USARTx	Pointer to USART peripheral
* @param		hexnum	Hex number (32-bit long)
* @return		None
*/
void USARTPutHex32 (USART_Type *USARTx, uint32_t hexnum)
{
	uint8_t nibble, i;

//	USARTPuts(USARTx, "0x");
	i = 7;
	do {
		nibble = (hexnum >> (4*i)) & 0x0F;
		USARTPutChar(USARTx, (nibble > 9) ? ('A' + nibble - 10) : ('0' + nibble));
	} while (i--);
}

/**
* @brief		print function that supports format as same as printf()
*				function of <stdio.h> library
* @param		format formated string to be print
* @return		None
*/
void  cprintf (const  char *format, ...)
{
    char  buffer[512 + 1];
            va_list     vArgs;
    va_start(vArgs, format);
    vsprintf((char *)buffer, (char const *)format, vArgs);
    va_end(vArgs);

    _DBG(buffer);
}

/**
* @brief		Initialize Debug frame work through initializing USART port
* @param		None
* @return		None
*/
void debug_frmwrk_init(void)
{
	USART_CFG_Type USARTConfigStruct;

#if (USED_USART_DEBUG_PORT==0)
	/*
	 * Initialize USART10 pin connect
	 */
	// Initialize USART10 (UART) pin connection
	HAL_GPIO_ConfigOutput(PB, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, FUNC0);    // USART10 TXD
	HAL_GPIO_ConfigPullup(PB, 6, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PB, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, FUNC0);    // USART10 RXD
	HAL_GPIO_ConfigPullup(PB, 7, ENPU);   // enable Pull up	
	
	// Initialize USART10 (UART) pin connection
//	HAL_GPIO_ConfigOutput(PA, 9, ALTERN_FUNC);
//	HAL_GPIO_ConfigFunction(PA, 9, FUNC0);    // USART10 TXD
//	HAL_GPIO_ConfigPullup(PA, 9, ENPU);   // enable Pull up
//	
//	HAL_GPIO_ConfigOutput(PA, 10, ALTERN_FUNC);
//	HAL_GPIO_ConfigFunction(PA, 10, FUNC0);    // USART10 RXD
//	HAL_GPIO_ConfigPullup(PA, 10, ENPU);   // enable Pull up	
	
#elif (USED_USART_DEBUG_PORT==1)
	/*
	 * Initialize USART11 pin connect
	 */
	HAL_GPIO_ConfigOutput(PA, 2, ALTERN_FUNC); // USART11 TXD
	HAL_GPIO_ConfigFunction(PA, 2, FUNC0);
	HAL_GPIO_ConfigPullup(PA, 2, ENPU);	
    
  HAL_GPIO_ConfigOutput(PA, 3, ALTERN_FUNC); // USART11 RXD
	HAL_GPIO_ConfigFunction(PA, 3, FUNC0);
	HAL_GPIO_ConfigPullup(PA, 3, ENPU);
	
//	HAL_GPIO_ConfigOutput(PA, 14, ALTERN_FUNC); // USART11 TXD
//	HAL_GPIO_ConfigFunction(PA, 14, FUNC0);
//	HAL_GPIO_ConfigPullup(PA, 14, ENPU);	
//    
//	HAL_GPIO_ConfigOutput(PA, 15, ALTERN_FUNC); // USART11 RXD
//	HAL_GPIO_ConfigFunction(PA, 15, FUNC0);
//	HAL_GPIO_ConfigPullup(PA, 15, ENPU);
	
#endif
#if (USED_USART_DEBUG_PORT==2)
	/*
	 * Initialize USART12 pin connect
	 */
	// Initialize USART12 (UART) pin connection
//	HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
//	HAL_GPIO_ConfigFunction(PB, 10, FUNC0);    // USART12 TXD
//	HAL_GPIO_ConfigPullup(PB, 10, ENPU);   // enable Pull up
//	
//	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
//	HAL_GPIO_ConfigFunction(PB, 11, FUNC0);    // USART12 RXD
//	HAL_GPIO_ConfigPullup(PB, 11, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PC, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 4, FUNC0);    // USART12 TXD
	HAL_GPIO_ConfigPullup(PC, 4, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PC, 5, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 5, FUNC0);    // USART12 RXD
	HAL_GPIO_ConfigPullup(PC, 5, ENPU);   // enable Pull up
	
#elif (USED_USART_DEBUG_PORT==3)
	/*
	 * Initialize USART13 pin connect
	 */
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC); // USART13 TXD
	HAL_GPIO_ConfigFunction(PA, 0, FUNC0);
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);	
    
  HAL_GPIO_ConfigOutput(PA, 1, ALTERN_FUNC); // USART13 RXD
	HAL_GPIO_ConfigFunction(PA, 1, FUNC0);
	HAL_GPIO_ConfigPullup(PA, 1, ENPU);
#endif
	/* Initialize USART Configuration parameter structure to default state:
	 * Baudrate = 38400bps
	 * 8 data bit
	 * 1 Stop bit
	 * None parity
	 */
	 // default: 38400-8-N-1	
	USARTConfigStruct.Mode = USART_UART_MODE;
	USARTConfigStruct.Baud_rate = 38400;
	USARTConfigStruct.Databits = USART_DATABIT_8;
	USARTConfigStruct.Parity = USART_PARITY_NONE;
	USARTConfigStruct.Stopbits = USART_STOPBIT_1;
	
	// Initialize DEBUG_USART_PORT peripheral with given to corresponding parameter
	HAL_USART_Init((USART_Type*)DEBUG_USART_PORT, &USARTConfigStruct);
    
    // Interrupt mode selection
    #if 0
    HAL_USART_ConfigInterrupt((USART_Type*)DEBUG_USART_PORT, USART_INTCFG_TXC, ENABLE);
    HAL_USART_ConfigInterrupt((USART_Type*)DEBUG_USART_PORT, USART_INTCFG_RXC, ENABLE);
    #endif
    
	_db_msg     = USARTPuts;
	_db_msg_    = USARTPuts_;
	_db_char    = USARTPutChar;
	_db_hex     = USARTPutHex;
	_db_hex_16  = USARTPutHex16;
	_db_hex_32  = USARTPutHex32;
	_db_dec     = USARTPutDec;
	_db_dec_16  = USARTPutDec16;
	_db_dec_32  = USARTPutDec32;
	_db_get_char = USARTGetChar;
	_db_get_ch  = USARTGetCh;	
}
#endif 

#endif /* _DEBUG_FRMWRK_ */

/* --------------------------------- End Of File ------------------------------ */
