/**
* @file		A31G32x_hal_dmac.c
* @brief	Contains all functions support for dma firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_dmac.h"
#include "A31G32x_hal_scu.h"


/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initialize DMA controller
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_DMAC_Init(void)
{
	/* to be defined Enable DMA and clock */
	// enabled default on reset
	SCU->PER1&=~SCU_PER1_DMA_Msk; // (1<<4) DMA peri enable 
	SCU->PCER1&=~SCU_PCER1_DMA_Msk; // (1<<4) DMA peri clock enable 
	
	SCU->PER1|=SCU_PER1_DMA_Msk; // (1<<4) DMA peri enable 
	SCU->PCER1|=SCU_PCER1_DMA_Msk; // (1<<4) DMA peri clock enable 
	return HAL_OK;
}

/**
* @brief      	Initialize DMAC(Direct Memory Access Controller) peripheral
* @param    	None
* @return     	HAL_Status
*/
HAL_Status_Type HAL_DMAC_DeInit(void)
{
	SCU->PER1&=~SCU_PER1_DMA_Msk; // (1<<4) DMA peri enable 
	SCU->PCER1&=~SCU_PCER1_DMA_Msk; // (1<<4) DMA peri clock enable 
	
	return HAL_OK;
}
/**
* @brief      Start DMA
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type DMAC_Start(void)
{
	return HAL_OK;

}

/**
* @brief      Stop DMA
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type DMAC_Stop(void)
{
	return HAL_OK;

}

/**
* @brief      Start DMA for interrupt
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type DMAC_Start_IT(void)
{
	return HAL_OK;

}

/**
* @brief      Stop DMA for interrupt
* @param      Not Support
* @return     HAL_Status
*/
HAL_Status_Type DMAC_Stop_IT(void)
{
	return HAL_OK;

}


/**
* @brief		Setup DMAC channel peripheral according to the specified
*				parameters in the GPDMAChannelConfig.
* @param		DMACx	Pointer to selected DMAC peripheral, should be:
* 					- DMAC0~3	:DMAC0~3 peripheral
* @param		DMACChannelConfig : Pointer to a DMAC_CH_CFG_Type structure
* 				that contains the configuration information for the specified
* 				channel peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_DMAC_Setup(DMAC_Type *DMACx, DMAC_Ch_CFG_Type *DMACChConfig)
{
	/* Check the DMA handle allocation */
	if(NULL == DMACx)
	{
	  return HAL_ERROR;
	}

	DMACx->CR = ((DMACChConfig->TransferCnt & 0xfff)<<16) 
		| ((DMACChConfig->PeriSel & 0x0f)<<8)	
		| ((DMACChConfig->TransferWidth & 0x03)<<2)
		| ((DMACChConfig->Dir & 0x01)<<1)	
		;
	DMACx->PAR = DMACChConfig->PeriAddr;	
	DMACx->MAR = DMACChConfig->MemAddr;
	return HAL_OK;
}

/**
* @brief		Enable/Disable DMA channel
* @param		DMACx	Pointer to selected DMAC peripheral, should be:
* 					- DMAC0~3	:DMAC0~3 peripheral
* @param		NewState	New State of this command, should be:
* 					- ENABLE.
* 					- DISABLE.
* @return		HAL_Status
*/
HAL_Status_Type HAL_DMAC_ChCmd(DMAC_Type *DMACx, FunctionalState NewState)
{
	/* Check the DMA handle allocation */
	if(NULL == DMACx)
	{
	  return HAL_ERROR;
	}

	if (NewState == ENABLE) {
		DMACx->SR |= DMAC_SR_DMAEN_Msk;
	} else {
		DMACx->SR &= ~DMAC_SR_DMAEN_Msk;
	}
	return HAL_OK;
}

/**
* @brief		Check if corresponding channel does have an active interrupt
* 				request or not
* @param	DMACx	Pointer to selected DMAC peripheral, should be:
* 					- DMAC0~3	:DMAC0~3 peripheral
* @return	FlagStatus	status of DMA channel interrupt after masking
*			Should be:
* 				- SET	: all data is transferred 
* 				- RESET	: data to be transferred is existing 
*/
FlagStatus HAL_DMAC_GetState(DMAC_Type *DMACx)
{

	if (DMACx->SR & DMAC_SR_EOT_Msk){	
		return SET;
	}
	else {
		return RESET;
	}
}

/* --------------------------------- End Of File ------------------------------ */

