/**
* @file		A31G32x_hal_pwr.c
* @brief	Contains all functions support for pwr firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_scu.h"
#include "A31G32x_hal_pwr.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief	Enter Sleep mode with co-operated instruction by the Cortex-M0+.
* @param	None
* @return	None
*/
void HAL_PWR_EnterSleepMode(void)
{
	/* Sleep Mode*/
	SCB->SCR &= ~(1<<2);
	__DSB();
#ifdef  __CC_ARM
	 __wfi();		// Use only keil arm compiler
#elif	__ICCARM__
	 asm("WFI");	// Use only iar arm compiler
#endif
	__NOP();
	__NOP();
	__NOP();
	__NOP();	
}

/**
* @brief		Enter Power Down mode with co-operated instruction by the Cortex-M0+.
* @param		None
* @return		None
*/
void HAL_PWR_EnterPowerDownMode(void)
{
	SCU->VDCCON = 0xA8000000;
	
    /* Deep-Sleep Mode, set SLEEPDEEP bit */
	SCB->SCR = 0x4;
	/* Power Down Mode*/
#ifdef  __CC_ARM
	 __wfi();		// Use only keil arm compiler
#elif	__ICCARM__
	 asm("WFI");	// Use only iar arm compiler
#endif
	__NOP();
	__NOP();
	__NOP();
	__NOP();
}

/**
* @brief		Enter Standby mode
* @param		None
* @return		None
*/
void HAL_PWR_EnterStandbyMode(void)
{
	SCU->VDCCON = 0xA8000000;
	SCU->VDCCON |= (0x15<<18);
	
    /* Deep-Sleep Mode, set SLEEPDEEP bit */
	SCB->SCR = 0x4;
	/* Power Down Mode*/
#ifdef  __CC_ARM
	 __wfi();		// Use only keil arm compiler
#elif	__ICCARM__
	 asm("WFI");	// Use only iar arm compiler
#endif
	__NOP();
	__NOP();
	__NOP();
	__NOP();
}
	
/**
* @brief		Enable/Disable Backup Power mode
* @param		state : Enable/Disable
* @return		None
*/
void HAL_PWR_Backup_Power_Mode(FunctionalState state)
{
	if(state == ENABLE) {
		SCU->PDRCON = ((0xA<<8) | (0x1<<7));	// VBAT Enable // when VDDEXT < PDR, Enter the VBAT Mode
	} else {
		SCU->PDRCON = ((0xA<<8) | (0x0<<7));	// VBAT Disable // when VDDEXT < PDR, Enter the VBAT Mode
	}
}

/**
* @brief		Enable/Disable PC13 Wake-up in Standby Mode(Only)
* @param		state : Enable/Disable
* @return		None
*/
void HAL_PWR_StandbyMode_PC13_Wakeup(FunctionalState state)
{
	if(state == ENABLE) {
		RTC->RTCPFCR |= ((3<<9) | (1<<0));	// Enable PC13 Wake-up
	} else {
		RTC->RTCPFCR &= ~(0x301);	// Disable PC13 Wake-up, When PC13 is used as Standby mode wakeup Pin, PC13 Wake-up must be Disable
	}
}

/**
* @brief		Check PC13 Wake-up in Standby Mode(Only)
* @param	None
* @return		Bool : True/False
*/
Bool HAL_PC13_Wakeup_State()
{
	if(SCU->WUSR & 0x80)	return TRUE;
	else					return FALSE;
	
}

/**
* @brief		Check Backup Power Wakeup State
* @param	None
* @return		Bool : True/False
*/
Bool HAL_Backup_Power_Wakeup_State()
{
	if((SCU->SMR & 0x60)==0x60)	return TRUE;
	else					return FALSE;
	
}

/**
* @brief		Backup_Register Value Setting
* @param	uint32_t	value
*                      uint8_t number
*                        - 1 : BKUPDR1
*                        - 2 : BKUPDR2
*                        - 3 : BKUPDR3
*                        - 4 : BKUPDR4
* @return		None
*/
void HAL_Backup_Register_Write(uint32_t value, uint8_t number)
{
	switch(number)
	{
		case 1:	RTC->BKUPDR1=value;	break;
		case 2:	RTC->BKUPDR2=value;	break;
		case 3:	RTC->BKUPDR3=value;	break;
		case 4:	RTC->BKUPDR4=value;	break;
		default :
			break;
	}	
}

/**
* @brief		Backup_Register Value Read
* @param	uint8_t number
*                        - 1 : BKUPDR1
*                        - 2 : BKUPDR2
*                        - 3 : BKUPDR3
*                        - 4 : BKUPDR4
* @return		uint32_t	value
*/
uint32_t HAL_Backup_Register_Read(uint8_t number)
{
	uint32_t result;
	
	switch(number)
	{
		case 1:	result=RTC->BKUPDR1;	break;
		case 2:	result=RTC->BKUPDR2;	break;
		case 3:	result=RTC->BKUPDR3;	break;
		case 4:	result=RTC->BKUPDR4;	break;
		default :
			break;
	}
	
	return result;
}

/* --------------------------------- End Of File ------------------------------ */
