/**
* @file		A31G32x_hal_rtc.c
* @brief	Contains all functions support for rtc firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_scu.h"
#include "A31G32x_hal_pcu.h"
#include "A31G32x_hal_rtc.h"
#include "A31G32x_hal_debug_frmwrk.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/**
* @brief		Initialize RTC peripheral
* @param      	None
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_Init(void)
{
	SCU->PER2 |= (1<<29);		
	SCU->PCER2 |= (1<<29);	
	SCU->PRER2 |= (1<<29);
	
	return HAL_OK;
}

/**
* @brief		De-initializes the RTC peripheral registers to their
*				default reset values.
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_DeInit(void)
{
	RTC->RTCCR = 0x00;
	
	SCU->PER2 &= ~(1<<29);		
	SCU->PCER2 &= ~(1<<29);	
	SCU->PRER2 &= ~(1<<29);

	return HAL_OK;
}

/**
* @brief		Set start mode for RTC
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_Start(FunctionalState NewState )
{
	if (NewState == ENABLE)
	{
		RTC->RTCCR |=  (1<<30);
	}
	else
	{
		RTC->RTCCR &=  ~(1<<30);
	}
	return HAL_OK;
}

/**
* @brief		Set stop mode for RTC
* @param		Not Support
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_Stop(void)
{
	 return HAL_OK;
}

/**
* @brief		RTC interrupt configuration
* @param		RTC_ConfigStruct	pointer to RTC_CONFIGURE_Type
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_ConfigInterrupt(void *RTC_ConfigStruct)
{
	RTC_CONFIGURE_Type *pRTCCfg;
	unsigned int count=0;
	
	pRTCCfg=RTC_ConfigStruct;
	
	RTC->RTCCR |= RTC_RTCCR_RTCEN_Msk;
	for(count=0;count<2500;count++);	// RTC wait must
	
	RTC->SUBDR &= ~(0xFFFFFFFF);
	RTC->SUBDR |= pRTCCfg->RTC_SUBDR;

	RTC->SEC = pRTCCfg->RTC_Sec;
	
	RTC->MIN = pRTCCfg->RTC_Min;
	
	RTC->DAY = pRTCCfg->RTC_Day;
	
	RTC->HOUR = pRTCCfg->RTC_Hour;
	
	RTC->WEEK = pRTCCfg->RTC_Week;
	
	RTC->MONTH = pRTCCfg->RTC_Month;
	
	RTC->YEAR = pRTCCfg->RTC_Year;
	return HAL_OK;
}

/**
* @brief 		Get flags of RTC Match Interrupt
* @param		None
* @return		return flags of RTC Match(0, 1) 
*/
uint32_t HAL_RTC_GetMatch_IT(void)
{
	if (RTC->RTCCR & (1<<16)){
		return 1;
	}
	else {
		return 0;
	}
}

/**
* @brief 		clear flags of RTC Match Interrupt
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_ClearMatch_IT(void)
{
	RTC->RTCCR &= ~(1<<16);
	return HAL_OK;
}


/**
* @brief 		Get flags of RTC Timestamp Interrupt
* @param		None
* @return		return flags of RTC Timestamp(0, 1) 
*/
uint32_t HAL_RTC_GetTS_IT(void)
{
	if (RTC->RTCCR & (1<<12))
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**
* @brief 		clear flags of RTC Timestamp Interrupt
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_ClearTS_IT(void)
{
	RTC->RTCCR &= ~(1<<12);
	return HAL_OK;
}

/**
* @brief 		Get value of RTC Timestamp(RTC->TIME_TS) 
* @param		None
* @return		return value of RTC Timestamp(RTC->TIME_TS) 
*/
uint32_t HAL_RTC_GetTS_Time(void)
{
	return RTC->TIME_TS;
}

/**
* @brief 		Get value of RTC Timestamp(RTC->DATE_TS) 
* @param		None
* @return		return value of RTC Timestamp(RTC->DATE_TS) 
*/
uint32_t HAL_RTC_GetTS_Date(void)
{
	return RTC->DATE_TS;
}

/**
* @brief 		Get flags of RTC Alarm Interrupt
* @param		None
* @return		return flags of RTC Alarm(0, 1) 
*/
uint32_t HAL_RTC_GetAlarm_IT()
{
	if (RTC->RTCCR & (1<<8)){
		return 1;
	}
	else {
		return 0;
	}
}

/**
* @brief 		Clear flags of RTC Alarm Interrupt
* @param		None
* @return		None
*/
void HAL_RTC_ClearAlarm_IT(void)
{
	RTC->RTCCR &= ~(1<<8);

}

/**
* @brief 		Set RW mode set
* @param		None
* @return		None
*/
void HAL_RTC_SetRwalt(void)
{
	RTC->RTCCR |= (1<<0);
	while(!(RTC->RTCCR & (1<<1)));
}


/**
* @brief 		Clear RW mode set
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_ClearRwalt(void)
{
	RTC->RTCCR &= ~(1<<0);
	while((RTC->RTCCR & (1<<1)));
	return HAL_OK;
}


/**
* @brief 		Set Period selection of constant-period matching
* @param		period : 0x0 ~ 0x6 should be,
* 					- 0x0 : Disable
* 					- 0x1 : Once per 0.5 sec
* 					- 0x2 : Once per 1 sec
* 					- 0x3 : Once per 1 min
* 					- 0x4 : Once per 1 hour
* 					- 0x5 : Once per 1 day
* 					- 0x6 : Once per 1 month
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_SetConstantPeriod(uint8_t period)
{
	RTC->RTCCR |= (period<<17);		// CT
	return HAL_OK;
}


/**
* @brief 		Set Hour system selection(AM/PM)
* @param		period : 0, 1 should be,
* 					- 0 : 12-hour system(AM/PM)
* 					- 1 : 24-hour system
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_SetAmpm(uint8_t state)
{
	if(state==0x1)
	{
		RTC->RTCCR |= (1<<20);
	}
	else
	{
		RTC->RTCCR &= ~(1<<20);
	}
	return HAL_OK;
}

/**
* @brief 		Set Alarm configuration
* @param		RTC_ALARM_Config : pointer to RTC_Alarm_Type
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_SetAlarm(void *RTC_ALARM_Config, FunctionalState NewState)
{
	RTC_Alarm_Type *pRTCAlarmCfg;
	
	pRTCAlarmCfg=RTC_ALARM_Config;
	
	RTC->ALARMWM = (pRTCAlarmCfg->RTC_ALARMWW<<16)
				| (pRTCAlarmCfg->RTC_ALARMWH<<8)
				| (pRTCAlarmCfg->RTC_ALARMWM<<0);
	
	if(NewState)
	{
		RTC->RTCCR |= (1<<11);
	}
	else
	{
		RTC->RTCCR &= ~(1<<11);
	}
	return HAL_OK;
	
}

/**
* @brief 		Set Timestamp functions
* @param		NewState : enable/disable should be,
* 					- DISABLE	= 0
* 					- ENABLE	= 1
* @param		edge : Timestamp event active edge selection should be,
* 					- 0 : Falling edge
* 					- 1 : Rising edge
* 					- 2 : Falling/rising
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_SetTimestamp(FunctionalState NewState, uint8_t edge)
{
	if(NewState==0x01)
	{
		RTC->RTCCR |= (1<<15);
	}
	else
	{
		RTC->RTCCR &= ~(1<<15);
	}
	
	switch(edge)
	{
		case 0 :			// Falling Edge
			RTC->RTCCR &= ~(3<<13);
			break;
		case 1:			// Rising Edge
			RTC->RTCCR &= ~(3<<13);
			RTC->RTCCR |= (1<<13);
			break;
		case 2:			// Both Edge
			RTC->RTCCR &= ~(3<<13);
			RTC->RTCCR |= (2<<13);
			break;
		default :
			break;
	}
	return HAL_OK;
}

/**
* @brief 		Set Timestamp functions
* @param		RTC_ERRCOR_Config : pointer to RTC_ERRCOR_Type
* @return		HAL_Status
*/
HAL_Status_Type HAL_RTC_SetErrcor(void *RTC_ERRCOR_Config)
{
	RTC_ERRCOR_Type *pRTCErrcorCfg;
	pRTCErrcorCfg=RTC_ERRCOR_Config;
	
	RTC->ERRCOR = 0x00000000
				| (pRTCErrcorCfg->RTC_DEV << 31)
				| (pRTCErrcorCfg->RTC_DIR <<30)
				| (pRTCErrcorCfg->RTC_COREN << 28)
				| (pRTCErrcorCfg->RTC_CORDATA << 0)
				;	
	return HAL_OK;
}

/**
* @brief 		Get Year counter
* @param		None
* @return		return value of Year counter
*/
uint8_t HAL_RTC_Get_YEAR()
{
	return RTC->YEAR;
}

/**
* @brief 		Get Month counter
* @param		None
* @return		return value of Month counter
*/
uint8_t HAL_RTC_Get_MONTH()
{
	return RTC->MONTH;
}

/**
* @brief 		Get Week counter
* @param		None
* @return		return value of Week counter
*/
uint8_t HAL_RTC_Get_WEEK()
{
	return RTC->WEEK;
}

/**
* @brief 		Get Day counter
* @param		None
* @return		return value of Day counter
*/
uint8_t HAL_RTC_Get_DAY()
{
	return RTC->DAY;
}

/**
* @brief 		Get Hour counter
* @param		None
* @return		return value of Hour counter
*/
uint8_t HAL_RTC_Get_HOUR()
{
	return RTC->HOUR;
}

/**
* @brief 		Get Minute counter
* @param		None
* @return		return value of Minute counter
*/
uint8_t HAL_RTC_Get_MIN()
{
	return RTC->MIN;
}

/**
* @brief 		Get Sec counter
* @param		None
* @return		return value of Sec counter
*/
uint8_t HAL_RTC_Get_SEC()
{
	return RTC->SEC;
}

/**
* @brief 		Get change value of hour according to the specified time system.
* @param		hour(value of hour register)
* @return		return value of hour(24 hour system)
*/
uint8_t HAL_RTC_AMPM_Display(uint8_t hour)
{
	uint8_t result;
	
	result=hour;
	
	if((RTC->RTCCR & (RTC_RTCCR_AMPM_Msk))==AMPM_12HOUR)
	{
		switch(result)
		{
			case 0x12:	result=0x00;		break;
			case 0x21:	result=0x13;		break;
			case 0x22:	result=0x14;		break;
			case 0x23:	result=0x15;		break;
			case 0x24:	result=0x16;		break;
			case 0x25:	result=0x17;		break;
			case 0x26:	result=0x18;		break;
			case 0x27:	result=0x19;		break;
			case 0x28:	result=0x20;		break;
			case 0x29:	result=0x21;		break;
			case 0x30:	result=0x22;		break;
			case 0x31:	result=0x23;		break;
			case 0x32:	result=0x12;		break;
			default :		
				break;
		}
	}	
	return result;
}

/**
* @brief		Print RTC time
* @param		None
* @return		None
*/
void HAL_RTC_Print(void)
{	
	uint32_t Hour_temp;
	
	HAL_RTC_ClearMatch_IT();
	HAL_RTC_SetRwalt();		

	_DBG("\n\r");
	_DBG("20");
	_DBH(HAL_RTC_Get_YEAR());
	_DBG("-");
	_DBH(HAL_RTC_Get_MONTH());
	_DBG("-");
	_DBH(HAL_RTC_Get_DAY());
	_DBG("\t");
	
	Hour_temp=HAL_RTC_Get_HOUR();
	
	HAL_RTC_AMPM_Display(Hour_temp);

	_DBH(Hour_temp);
	_DBG(" : ");
	_DBH(HAL_RTC_Get_MIN());
	_DBG(" : ");
	_DBH(HAL_RTC_Get_SEC());
	_DBG("\t");	
	_DBG("WEEK : ");	_DBH(HAL_RTC_Get_WEEK());	
	_DBG("\n\r");
			
	HAL_RTC_ClearRwalt();
}

/**
* @brief		Print RTC timestamp
* @param		None
* @return		None
*/
void HAL_RTC_Time_Print(void)
{
	_DBG("Timestamp....\n\r");
	_DBH32(HAL_RTC_GetTS_Time());	_DBG("\t");	_DBH32(HAL_RTC_GetTS_Date());
	_DBG("\n\r");
}



/* --------------------------------- End Of File ------------------------------ */
