/**
* @file		A31G32x_hal_rtc.c
* @brief	Contains all functions support for rtc firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief 		Configure external sub clock  
*            	Before exteranl crystal enable setting, you have to set clock ports as SXIN,SXOUT in MOD,AFSRx reg
* @param		esosccon 	Function mode, should be:
* 					- EXSOSC_STOP	: 0
* 					- EXSOSC_EN		: 8
* 					- EXSOSC_EN_DIV2	: 9
* 					- EXSOSC_EN_DIV2	: 10 
* @return		None
*/
void HAL_SCU_LSE_ClockConfig(uint32_t esosccon)
{
	uint32_t reg_val32;
	
// enable external sub clock  
// Before exteranl crystal enable setting, you have to set clock ports as SXIN,SXOUT in MOD,AFSRx reg
	reg_val32=SCU->CSCR;
	reg_val32&=~(0x0f<<12); // 
	reg_val32|=(esosccon<<12); // 0:Stop SOSC, 8:Enable SOSC/1, 9:Enable SOSC/2, 10:Enable SOSC/4
	SCU->CSCR=reg_val32 | (0xa507UL<<16);		
}
 

/**
* @brief 		Configure low speed internal osc clock  
* @param		ringosc 	Function mode, should be:
* 					- LSIOSC_STOP        : 0
* 					- LSIOSC_EN          : 8
* 					- LSIOSC_EN_DIV2     : 9
* 					- LSIOSC_EN_DIV4     : 10
* @return		None
*/
void HAL_SCU_LSI_ClockConfig(uint32_t ringosc)
{
	uint32_t reg_val32;
	
// enable internal clock  
	reg_val32=SCU->CSCR;
	reg_val32&=~(0x0f<<8); // 
	reg_val32|=(ringosc<<8); // 0:Stop LSI, 8:Enable LSI/1, 9:Enable LSI/2, 10:Enable LSI/4
	SCU->CSCR=reg_val32 | (0xa507UL<<16);		
}

/**
* @brief 		Configure high speed internal osc clock  
* @param		hsiosc 	Function mode, should be:
* 					- HSIOSC_STOP        : 0
* 					- HSIOSC_EN          : 8
* 					- HSIOSC_EN_DIV2     : 9
* 					- HSIOSC_EN_DIV4     : 10
* 					- HSIOSC_EN_DIV8     : 11
* 					- HSIOSC_EN_DIV16    : 12
* 					- HSIOSC_EN_DIV32    : 13
* @return		None
 */
void HAL_SCU_HSI_ClockConfig(uint32_t hsiosc)
{
	uint32_t reg_val32;
	
// enable internal clock  
	reg_val32=SCU->CSCR;
	reg_val32&=~(0x0f<<4); // 
	reg_val32|=(hsiosc<<4); // 0:Stop LSI, 8:Enable LSI/1, 9:Enable LSI/2, 10:Enable LSI/4, 11:Enable LSI/8, 12:Enable LSI/16, 13:Enable LSI/32
	SCU->CSCR=reg_val32 | (0xa507UL<<16);		
}

/**
* @brief 		Configure external clock  
*            	 Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MR.
* @param		eosccon 	Function mode, should be:
* 					- EXOSC_STOP        : 0
* 					- EXOSC_EN          : 8
* 					- EXOSC_EN_DIV2     : 9
* 					- EXOSC_EN_DIV4     : 10
* @return		None
*/
void HAL_SCU_HSE_ClockConfig(uint32_t eosccon)
{
	uint32_t reg_val32;
	
// enable external clock  
// Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MOD,AFSRx reg
	reg_val32=SCU->CSCR;
	reg_val32&=~(0x0f<<0); // 
	reg_val32|=(eosccon<<0); // 0:Stop MOSC, 8:Enable MOSC/1, 9:Enable MOSC/2, 10:Enable MOSC/4
	SCU->CSCR=reg_val32 | (0xa507UL<<16);	
}

/**
* @brief 	monitoring External sub oscillator  
* @param  	none
* @return	status enumeration value:
*              - ERROR
*              - SUCCESS
*/
Status HAL_SCU_LSE_ClockMonitoring(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;
		
	// monitoring External sub oscillator 
	SCU->CMR|=(1<<11); // 0:External sub oscillator monitoring disabled, 1:External sub oscillator monitoring enable
	
	/* Wait till SOSC is ready and if timeout is reached exit */
//	do 
//	{
//		status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
//		StartUpCounter++;  
//	} while((StartUpCounter != SOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_SOSCSTS_Msk));	
	
	/* need more Wait for SOSC stable, check SXOUT pin oscillation  */	
	
	while(!(SCU->CMR & (1<<8)));
	for (i=0;i<0x1000;i++)
	{
		__NOP();
	}
	
//	status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
//	if (status != SCU_CMR_SOSCSTS_Msk){
//		return ERROR;
//	}
//	else {
		return SUCCESS;
//	}
}


/**
* @brief	monitoring External oscillator  
* @param  	none
* @return	status 	enumeration value:
*              - ERROR
*              - SUCCESS
*/
Status HAL_SCU_HSE_ClockMonitoring (void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;

	// monitoring External oscillator 
	SCU->CMR|=(1<<3); // 0:External oscillator monitoring disabled, 1:External oscillator monitoring enable
	
	/* Wait till MOSC is ready and if timeout is reached exit */
	do 
	{
		status = (SCU->CMR & SCU_CMR_MOSCSTS_Msk);
		StartUpCounter++;  
	} while((StartUpCounter != MOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_MOSCSTS_Msk));	
	
	/* need more Wait for MOSC stable, check XOUT pin oscillation  */	
	for (i=0;i<0x800;i++)
	{
		__NOP();
	}
	
	status = (SCU->CMR & SCU_CMR_MOSCSTS_Msk);
	if (status != SCU_CMR_MOSCSTS_Msk){
		return ERROR;
	}
	else {
		return SUCCESS;
	}
}

/**
* @brief 		change system clock control   
* @param		sysclkcon mode, should be:
* 					- SC_LSIOSC		: 0
* 					- SC_SOSC		: 1 
* 					- SC_HSIOSC		: 2
* 					- SC_HSIOSCPLL	: 3
* 					- SC_EXOSC		: 6
* 					- SC_EXOSCPLL	: 7
* @return		None
*/
void HAL_SCU_SystemClockChange(uint32_t sysclkcon)
{ 
	uint32_t reg_val32;
	
	reg_val32=SCU->SCCR;
	reg_val32&=~(7<<0); //
	reg_val32|=sysclkcon;  
	SCU->SCCR=reg_val32 | (0x570AUL<<16);		
}

/**********************************************************************//**
 * @brief 		change system clock control   
 * @param[in]	sysclkcon mode, should be:
 * 					- SC_FIN_HSI		         : 0
 * 					- SC_FIN_HSE 	           : 1 
 * @return		None
 **********************************************************************/
void HAL_SCU_SystemClockFinClock(uint32_t sysclkcon)
{ 
  uint32_t reg_val32;
	
	reg_val32=SCU->SCCR;
	reg_val32&=~(1<<2); 
	reg_val32|=(sysclkcon<<2);  
	SCU->SCCR=reg_val32 | (0x570aUL<<16);		
}

/**
* @brief 		Configure clock out  on PF0
*       		Before this function setting, you have to set PF4 as CLKO in PF.MODand PF.AFSR0.
* @param		divval(=CLKODIV) : 0~15  // CLKO=MCLK/(2*(CLKODIV+1))
*                   ex) MCLK=20Mh, CLKODIV=4,    20Mhz/ (2*(4+1)) = 2Mhz
* @param		endis, should be:
* 					- DISABLE		: 0
* 					- ENABLE		: 1
* @return		None
*/
void HAL_SCU_ClockOutput(uint8_t divval, FunctionalState endis)
{
	SCU->COR=(endis<<4)|(divval&0xf);      
}

/**
 * @brief 	 monitoring External oscillator  
 * @param 	 none
 * @return	 status 	enumeration value:
 *              - ERROR
 *              - SUCCESS

PLL_DEALY value must be changed according to the system clock used by the user.
Set to about 200us based on the system clock.
1. LSI750kHz		: 150 
2. HSE16MHz	: 3500
3. HSI48MHz		: 10000


*/
#define PLL_DELAY	1750	// @HSE 8MHz

Status HAL_SCU_PLL_ClockConfig(FunctionalState pllsetstate, uint8_t selbypass, uint8_t selfin,  uint8_t PREDIV, uint8_t POSTDIV1, uint8_t POSTDIV2, uint8_t OUTDIV)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint32_t status;
	uint32_t i;
	
	if (pllsetstate == DISABLE){
		SCU->PLLCON=0;
		return SUCCESS;
	}
	else {
		/* PLLenable    PLL freq = (FIN or FIN/2) * M / N */
		SCU->PLLCON = 0 |(1<<22); // PLLEn	
		__NOP();                  // 1us wait
		SCU->PLLCON |= (1<<23)    // PLL reset is negated
		|((selbypass&1)<<21)     //BYPASS 0:FOUT is bypassed as FIN, 1:FOUT is PLL output
		|((selfin&1)<<20)             //PLL VCO mode  0:VCO is the same with FOUT, 1:VCO frequency is x2 of FOUT   D
		|((PREDIV&0x7)<<16)      //PREDIV        R
		|((POSTDIV1&0xff)<<8)    //POSTDIV1   N1			
		|((POSTDIV2&0xf)<<4)     //POSTDIV2   N2
		|((OUTDIV&0xf)<<0)         //OUTDIV	     P 
			;

		/*wait over 190us*/		 
		for (i=0;i<PLL_DELAY;i++); 
		
		/* Check the PLLCON State */
		while((SCU->PLLCON & (1UL<<31)) == RESET )
		{
			/* Time out for HSE start up, in 1ms */
			if(StartUpCounter++ > PLL_STARTUP_TIMEOUT)
			{
				return ERROR;
			}
		}
		return SUCCESS;
	}
}

/**
* @brief 	LSI 40kHz Clock
* @param	LSI40clk
*				- ENABLE
*				- DISABLE
* @return	None
*/
void HAL_SCU_WDTLSI40K_ClockConfig(uint32_t lsi40clk)
{
	uint32_t temp;
	
	temp = SCU->MCCR3;
	if(lsi40clk == ENABLE)
	{
		temp |= (1<<11);
	}
	else
	{
		temp &= ~(1<<11);		
	}	
	SCU->MCCR3 = temp;
}

/**
* @brief 		LSITS Clock
* @param		LSITS
*					- ENABLE
*					- DISABLE
* @return	None
*/
void HAL_SCU_TSLSI40K_ClockConfig(uint32_t lsits)
{
	uint32_t temp;
	
	temp = SCU->MCCR5;

	if(lsits == ENABLE)
	{
		temp |= (1<<11);
	}
	else
	{
		temp &= ~(1<<11);		
	}
	
	SCU->MCCR5 = temp;

}

/**
* @brief 		TempSensor MCCR5
* @param		Temptype :
*					- Temp Sensor Reference clock
*					- Temp Sensor Sense clock
*			
*				clksel :
*					- TSREFCLK 
*					00 : TSREFCLK_HSI_I (Output clock set by CSCR from HSI)
*					01 : TSREFCLK_MCLK
*					10 : TSREFCLK_MOSC
*					11 : TSREFCLK_SOSC
*					- TSSENSECLK
*					00 : TSSENSECLK_LSITS
*					01 : TSSENSECLK_LSI750kHz
*					10 : TSSENSECLK_LSI40kHz
*					11 : TSSENSECLK_HSI_I (Output clock set by CSCR from HSI)
* @return	None
*/
void HAL_SCU_TempSensor_ClockConfig (uint32_t Temptype, uint32_t clksel)
{
	uint32_t temp;
	
	temp = SCU->MCCR5;
	
	if(Temptype==TSREF_TYPE)
	{
		temp &= ~(3<<14);
		temp |= (clksel<<14);
	}
	else if(Temptype==TSSENSE_TYPE)
	{
		temp &= ~(3<<12);
		temp |= (clksel<<12);
	}
	
	SCU->MCCR5 = temp;
}

/**
* @brief 		Configure peri clock setting
* @param		mccrnum: select REG number
*                     - MCCR1 : 1, * - MCCR2 : 2
*                     - MCCR3 : 3, * - MCCR4 : 4
*                     - MCCR5 : 5, * - MCCR6 : 6 
* @param		type : 
*	                    SYSTICK_TYPE, TIMER1N_TYPE, 
*	                    TIMER20_TYPE, TIMER21_TYPE, 
*			     		TIMER30_TYPE, TIMER40_TYPE,
*			     		WDT_TYPE, WT_TYPE, 
*			     		PD0_TYPE
*						USB_TYPE
* @param		clksrc : 
*						- CLKSRC_LSI=0,
*						- CLKSRC_SOSC=3,
*						- CLKSRC_MCLK=4,	
*						- CLKSRC_HSI=5,
*						- CLKSRC_MOSC=6,
*                     	- CLKSRC_PLL=7	
* @param		clkdiv : 0x0 ~ 0xFF
* @return		None
*/
 void HAL_SCU_MiscClockConfig( uint8_t mccrnum, uint8_t type, uint8_t clksrc, uint8_t clkdiv)
{
	uint32_t tmp=0;
	
	switch(mccrnum){
		case 1:
			tmp=SCU->MCCR1;
			if (type==SYSTICK_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TIMER1N_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}			
			SCU->MCCR1=tmp;
			break;
			
		case 2:	
			tmp=SCU->MCCR2;
			if (type==TIMER20_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TIMER30_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR2=tmp;			
			break;
			
		case 3:
			tmp=SCU->MCCR3;
			if (type==WDT_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==WT_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
			}
			SCU->MCCR3=tmp;			
			break;
			
		case 4:
			tmp=SCU->MCCR4;
			if (type==PD_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			SCU->MCCR4=tmp;			
			break;
		
		case 5:
			tmp=SCU->MCCR5;
			if(type==RTC_TYPE) {
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));
			}
			SCU->MCCR5=tmp;
			break;
		
		case 6:	
			tmp=SCU->MCCR6;
			if (type==TIMER40_TYPE){
				tmp&=0xFFFFF800;
				tmp|= (((clksrc&7)<<8)|((clkdiv&0xFF)<<0));		
			}
			else if (type==TIMER21_TYPE){
				tmp&=0xF800FFFF;
				tmp|= (((clksrc&7)<<24)|((clkdiv&0xFF)<<16));					
				tmp|=((clksrc&7)<<24);
				tmp|=((clkdiv&0xFF)<<16);
			}
			SCU->MCCR6=tmp;			
			break;
	}
}


/**
* @brief 		set LVR level   
* @param		bodlevel, should be:
* 					LVR_LVL_1_60V			(0xFuL<<0)
* 					LVR_LVL_1_69V			(0xEuL<<0)
* 					LVR_LVL_1_78V			(0xDuL<<0)
* 					LVR_LVL_1_90V			(0xCuL<<0)
* 					LVR_LVL_1_99V			(0xBuL<<0)
* 					LVR_LVL_2_12V			(0xAuL<<0)
* 					LVR_LVL_2_30V			(0x9uL<<0)
* 					LVR_LVL_2_47V			(0x8uL<<0)
* 					LVR_LVL_2_67V			(0x7uL<<0)
* 					LVR_LVL_3_04V			(0x6uL<<0)
* 					LVR_LVL_3_18V			(0x5uL<<0)
* 					LVR_LVL_3_59V			(0x4uL<<0)
* 					LVR_LVL_3_72V			(0x3uL<<0)
* 					LVR_LVL_4_03V			(0x2uL<<0)
* 					LVR_LVL_4_20V			(0x1uL<<0)
* 					LVR_LVL_4_48V			(0x0uL<<0)
* @return		None
 */
void HAL_SCU_SetLVRLevel(uint32_t lvdlevel)
{
	SCULV->LVRCNFIG = ((0x72A5<<16) | (0x00<<8) | (lvdlevel<<0));
}

/**
* @brief	 	enable/disable LVR
* @param		NewState
* 					- ENABLE  	:Set LVD enable
* 					- DISABLE 	:Disable LVD
* @return 		None
*/
void HAL_SCU_LVRCmd(FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->RSER |= (1<<0);		// LVD Enable
		SCULV->LVRCR = 0x00;
		SCULV->LVRCNFIG = ((0x72A5<<16) | (0x00<<8));
	}
	else
	{
		SCULV->LVRCR = 0x55;
		SCULV->LVRCNFIG = ((0x72A5<<16) | (0xAA<<8) | (0xF<<0));
		SCU->RSER &= ~(1<<0);		// LVD Disable
	}
}

/**
* @brief		Low Voltage Indicator Control
* @param		LVI level, should be:
* 					LVI_LVL_1_88V			(0x3uL<<0)
* 					LVI_LVL_2_00V			(0x4uL<<0)
* 					LVI_LVL_2_13V			(0x5uL<<0)
* 					LVI_LVL_2_28V			(0x6uL<<0)
* 					LVI_LVL_2_46V			(0x7uL<<0)
* 					LVI_LVL_2_67V			(0x8uL<<0)
* 					LVI_LVL_3_04V			(0x9uL<<0)
* 					LVI_LVL_3_20V			(0xAuL<<0)
* 					LVI_LVL_3_55V			(0xBuL<<0)
* 					LVI_LVL_3_75V			(0xCuL<<0)
* 					LVI_LVL_3_99V			(0xDuL<<0)
* 					LVI_LVL_4_25V			(0xEuL<<0)
* 					LVI_LVL_4_55V			(0xFuL<<0)
* @return	None
*/
void HAL_SCU_LVICmd(LVI_CFG_Type *LVI_ConfigStruct)
{
	uint32_t	tempreg;
	
	tempreg = SCULV->LVICR;
	tempreg = 0
	| ((LVI_ConfigStruct->EnSel&0x1)<<7)
	| ((LVI_ConfigStruct->IntSel&0x1)<<5)
	| ((LVI_ConfigStruct->LvlSel&0xF)<<0)	
	;
	
	SCULV->LVICR = tempreg;
}

/**
* @brief	 	Get value of LVI interrupt flag
* @param		None
* @return 		return LVI interrupt flag
*/
uint32_t HAL_SCULV_GetLviFlag(void)
{
	return (SCULV->LVICR & (1<<4));	
}

/**
* @brief	 	Clear value of LVI interrupt flag
* @param		None
* @return 		None
*/
void HAL_SCULV_ClrLviFlag(void)
{
	SCULV->LVICR |= (1<<4);
}

/**
* @brief	 	enable/disable WakeUp Source
* @param		WakeUpSrc, oring is possible:
* 					- WAKEUP_GPIOF 	:    (1UL<<13)
* 					- WAKEUP_USART3 :    (1UL<<12)
* 					- WAKEUP_USART2 :    (1UL<<11)
* 					- WAKEUP_GPIOC 	:    (1UL<<10)
* 					- WAKEUP_GPIOB 	:    (1UL<<9)
* 					- WAKEUP_GPIOA 	:    (1UL<<8)
*					- WAKEUP_COMP   :    (1UL<<6)
* 					- WAKEUP_RTC    :    (1UL<<5)
* 					- WAKEUP_USART1 :    (1UL<<4)
* 					- WAKEUP_USART0 :    (1UL<<3)
* 					- WAKEUP_WT    	:    (1UL<<2)
* 					- WAKEUP_WDT    :    (1UL<<1)
* 					- WAKEUP_LVD    :    (1UL<<0)
* @param		NewState
* 					- ENABLE  	:Set WakeUp Source enable
* 					- DISABLE 	:Disable WakeUp Source
* @return 		None
*/
void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->WUER |=  (WakeUpSrc);
	}
	else
	{
		SCU->WUER &= ~(WakeUpSrc);
	}
}

/**
* @brief 		Get current value of WakeUpSRC Status
* @param		None
* @return		Current value of WakeUpSRC Status register
*/
uint32_t HAL_SCU_GetWakeUpSRCStatus(void)
{
	return (SCU->WUSR);
}

/**
* @brief	 	enable/disable WakeUp Source
* @param		WakeUpSrc
* @return 		None
*/
void HAL_SCU_ClearWakeUpSRC(uint32_t WakeUpStatus)
{
	SCU->WUSR = WakeUpStatus;
}

/**
* @brief 		Get current value of NMI Status
* @param		None
* @return		Current value of NMI Status register
*/
uint32_t HAL_SCU_GetNMIStatus(void)
{
	return (SCU->NMIR);
}

/**
* @brief	 	enable/disable NMI Source
* @param		nmisrc
* @return 		None
*/
void HAL_SCU_SetNMI(uint32_t nmisrc)
{
	SCU->NMIR = (0xA32CUL<<16) | (nmisrc & 0x1F);
}

/**
* @brief	 	enable/disable Reset Source
* @param		RstSrc, oring is possible:
* 					- RST_PINRST	:	(1UL<<6)
* 					- RST_CPURST	:	(1UL<<5)
* 					- RST_SWRST		:	(1UL<<4)
* 					- RST_WDTRST	:	(1UL<<3)
*					- RST_MCKFRST	:	(1UL<<2)
* 					- RST_MOFRST	:	(1UL<<1)
* 					- RST_LVDRST	:	(1UL<<0)
* @param		NewState
* 					- ENABLE  	:Set Reset Source enable
* 					- DISABLE 	:Disable Reset Source
* @return 		None
*/
void HAL_SCU_SetResetSrc(uint32_t RstSrc, FunctionalState NewState)
{
	if (NewState == ENABLE)
	{
		SCU->RSER |=  (RstSrc);
	}
	else
	{
		SCU->RSER &= ~(RstSrc);
	}
}

/**
* @brief 		Get current value of RSSR Status
* @param		None
* @return		Current value of RSSR Status register
*/
uint32_t HAL_SCU_GetResetSrc(void)
{
	return (SCU->RSSR);
}

/**
* @brief	 	clear Reset Status
* @param		rststatus
* @return 		None
*/
void HAL_SCU_ClearResetStatus(uint32_t rststatus)
{
	SCU->RSSR = rststatus;
}

/**
* @brief		Set Timer1n Clock
* @param		t1nclk                	T1NCLK_MCCR1, T1NCLK_PCLK
* @return 		None
*/
void HAL_SCU_Timer1n_ClockConfig(uint32_t t1nclk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<22);
	temp |= (t1nclk<<22);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Timer20 Clock
* @param		t20clk                	T20CLK_MCCR2, T20CLK_PCLK
* @return 		None
*/
void HAL_SCU_Timer20_ClockConfig(uint32_t t20clk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<20);
	temp |= (t20clk<<20);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Timer21 Clock
* @param		t21clk                	T21CLK_MCCR6, T21CLK_PCLK
* @return 		None
*/
void HAL_SCU_Timer21_ClockConfig(uint32_t t21clk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<19);
	temp |= (t21clk<<19);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Timer30 Clock
* @param		t30clk                	T30CLK_MCCR2, T30CLK_PCLK
* @return 		None
*/
void HAL_SCU_Timer30_ClockConfig(uint32_t t30clk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<17);
	temp |= (t30clk<<17);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Timer40 Clock
* @param		t40clk                	T40CLK_MCCR2, T40CLK_PCLK
* @return 		None
*/
void HAL_SCU_Timer40_ClockConfig(uint32_t t40clk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<16);
	temp |= (t40clk<<16);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set USB Clock
* @param		t40clk                	USBCLK_MCCR6, USBCLK_PCLK
* @return 		None
*/
void HAL_SCU_USB_ClockConfig(uint32_t usbclk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<9);
	temp |= (usbclk<<9);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Real Time Clock
* @param		rtcclk                	RTCCLK_LSI40kHz, RTCCLK_MCCR5
* @return 		None
*/
void HAL_SCU_RTC_ClockConfig(uint32_t rtcclk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<5);
	temp |= (rtcclk<<5);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set Watch Timer Clock
* @param		wtclk     WTCLK_MCCR3, WTCLK_SOSC, WTCLK_WDTRC(LSI40kHz)
* @return 		None
*/
void HAL_SCU_WT_ClockConfig(uint32_t wtclk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(3<<3);
	temp |= (wtclk<<3);
	SCU->PPCLKSR = temp;	      
}

/**
* @brief		Set WatchDog Timer Clock
* @param		wdtclk     WDTCLK_WDTRC, WDTCLK_MCCR3
* @return 		None
*/
void HAL_SCU_WDT_ClockConfig(uint32_t wdtclk)
{
	uint32_t temp;
	
	temp = SCU->PPCLKSR;
	temp &= ~(1<<0);
	temp |= (wdtclk<<0);
	SCU->PPCLKSR = temp;	      
}

/* --------------------------------- End Of File ------------------------------ */
