/**
* @file		A31G32x_hal_tempsensor.c
* @brief	Contains all functions support for tempsensor firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_tempsensor.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
float slope, offset;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief      Initialize TS(temperature sensor) peripheral
* @param      None
* @return     HAL_Status
*/
HAL_Status_Type HAL_TS_Init(void)
{
	SCU->PER2   &=  ~(SCU_PER2_TSENSE_Msk);
	SCU->PCER2  &=  ~(SCU_PER2_TSENSE_Msk);		
	
	SCU->PER2   |=  (SCU_PER2_TSENSE_Msk);
	SCU->PCER2  |=  (SCU_PER2_TSENSE_Msk);		
	
	TEMP->TSENSECON |=  TSENSECON_TSSEN;
	return HAL_OK;
}

/**
* @brief      De-Initialize TS peripheral
* @param      None
* @return     HAL_Status
*/
HAL_Status_Type HAL_TS_Deinit(void)
{
	TEMP->TSENSECON &= ~TSENSECON_TSSEN;
	
	SCU->PER2		&=	~(SCU_PER2_TSENSE_Msk);
	SCU->PCER2	&=	~(SCU_PER2_TSENSE_Msk);	
	return HAL_OK;
}

/**
* @brief    Start TS peripheral
* @param	NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return   HAL_Status
*/
HAL_Status_Type HAL_TS_Start(FunctionalState NewState)
{
	if(NewState == ENABLE)
		TEMP->TSENSECON |=  TSENSECON_TSSTART;
	else
		TEMP->TSENSECON &= ~TSENSECON_TSSTART;
	return HAL_OK;
}

/**
* @brief    Stop TS peripheral
* @param	None
* @return   HAL_Status
*/
HAL_Status_Type HAL_TS_Stop(void)
{
	return HAL_OK;
}

/**
* @brief      Clear match flag of TS
* @param      None
* @return     HAL_Status
*/
HAL_Status_Type HAL_TS_ClearStatus(void)
{
	TEMP->TSENSECON |= TSENSECON_TSSFLAG; //(1<<1) : clear for TSSFLAG
	return HAL_OK;
}

/**
* @brief      	Configure TS interrupt
* @param		NewState : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return    	HAL_Status
*/
HAL_Status_Type HAL_TS_ConfigInterrupt(FunctionalState NewState)
{
	if(NewState == ENABLE)
		TEMP->TSENSECON |=  TSENSECON_TSSINTEN;
	else
		TEMP->TSENSECON &= ~TSENSECON_TSSINTEN;
	return HAL_OK;
}

/**
* @brief      	Get TS status
* @param		None
* @return    	Return get TS control status
*/
uint8_t HAL_TS_GetStatus(void)
{
	return TEMP->TSENSECON;
	
}

/**
* @brief      	Set TS Reference Period(1kHz)
* @param		None
* @return    	HAL_Status
*/
HAL_Status_Type HAL_TS_SetReferencePeriod(void)
{
	volatile uint32_t top, bottom;
	
	TEMP->TSREFPEROID =	SystemCoreClock/1000;
	
	bottom = *(uint32_t *)(0x3F0103F4);
	top = *(uint32_t *)(0x3F0103F8);
		
	bottom = (bottom*1000) / 0x2000;
	top = (top*1000) / 0x2000;

	slope = (float)(75/((float)top-(float)bottom));
	offset = (float)(30-((float)slope*(float)bottom));
	return HAL_OK;
}

/**
* @brief      	Get TS Reference Peroid
* @param		None
* @return    	Return get TS Reference Peroid
*/
uint32_t HAL_TS_GetReferencePeroid(void)
{
	return TEMP->TSREFPEROID;
}

/**
* @brief      Get sensing count of TS
* @param      None
* @return     Sensing Count : 0x0000 ~ 0xFFFF
*/	
uint16_t HAL_TS_GetSenseCount(void)
{
	return TEMP->TSSENSECNT;
}

/**
* @brief      Convert to Celsius
* @param      None
* @return     Celsius value
*/	
float HAL_TS_ConvertToCelsius(void)
{
	return (slope*TEMP->TSSENSECNT+offset);
}

/* --------------------------------- End Of File ------------------------------ */

