/**
* @file		A31G32x_hal_timer1n.c
* @brief	Contains all functions support for timer1n firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_timer1n.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initial Timer/Counter device
* 				 	Set Clock frequency for Timer
* 					Set initial configuration for Timer
* @param		TIMER1n Timer selection, should be:
* 					- TIMER1n peripheral n=0..4
* @param		TimerCounterMode Timer counter mode, should be:
* 					- PERIODIC_MODE			:Timer mode
* 					- PWM_MODE	:Counter rising mode
* 					- ONESHOT_MODE	:Counter falling mode
* 					- CAPTURE_MODE		:Counter on both edges
* @param		TIMER_ConfigStruct pointer to TIMER_ConfigStruct
*				that contains the configuration information for the
*				specified Timer peripheral.
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_Init(TIMER1n_Type *TIMER1n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct)
{
	TIMER1n_PERIODICCFG_Type *pTimeCfg;
	TIMER1n_PWMCFG_Type *pPwmOneshotCfg;
	TIMER1n_CAPTURECFG_Type *pCaptureCfg;
	uint16_t reg_val16;
	
	if (TIMER1n== TIMER10)
	{
		SCU->PER1 &= ~(1<<16);
		SCU->PCER1 &= ~(1<<16);
		
		SCU->PER1 |= (1<<16);
		SCU->PCER1 |= (1<<16);		
	}
	else if (TIMER1n== TIMER11)
	{
		SCU->PER1 &= ~(1<<17);
		SCU->PCER1 &= ~(1<<17);

		SCU->PER1 |= (1<<17);
		SCU->PCER1 |= (1<<17);	
	}
	else if (TIMER1n== TIMER12)
	{
		SCU->PER1 &= ~(1<<18);
		SCU->PCER1 &= ~(1<<18);
		
		SCU->PER1 |= (1<<18);
		SCU->PCER1 |= (1<<18);
	}
	else if (TIMER1n== TIMER13)
	{
		SCU->PER1 &= ~(1<<19);
		SCU->PCER1 &= ~(1<<19);

		SCU->PER1 |= (1<<19);
		SCU->PCER1 |= (1<<19);		
	}

	if (TimerCounterMode == PERIODIC_MODE)
	{
		pTimeCfg = (TIMER1n_PERIODICCFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			| TIMER1n_CR_CKSEL_SET(pTimeCfg->CkSel)
			| TIMER1n_CR_MODE_SET(TimerCounterMode)
			| TIMER1n_CR_STARTLVL_SET(pTimeCfg->StartLevel)
			;
		if (pTimeCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER1n_CR_ECE_SET(pTimeCfg->ECE);
		}
		TIMER1n->CR = reg_val16;
		
		TIMER1n->PREDR = ((pTimeCfg->Prescaler-1) & T1n_PRS_MASK);
		TIMER1n->ADR = pTimeCfg->TnADR;
	}
	else if ((TimerCounterMode == PWM_MODE) || (TimerCounterMode == ONESHOT_MODE))
	{
		pPwmOneshotCfg = (TIMER1n_PWMCFG_Type *)TIMER_ConfigStruct;

		reg_val16 = 0
			| TIMER1n_CR_CKSEL_SET(pPwmOneshotCfg->CkSel)
			| TIMER1n_CR_MODE_SET(TimerCounterMode)
			| TIMER1n_CR_STARTLVL_SET(pPwmOneshotCfg->StartLevel)
			;
		if (pPwmOneshotCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER1n_CR_ECE_SET(pPwmOneshotCfg->ECE);
		}		
		TIMER1n->CR = reg_val16;

		TIMER1n->PREDR = ((pPwmOneshotCfg->Prescaler-1) & T1n_PRS_MASK);
		TIMER1n->ADR = pPwmOneshotCfg->TnADR;
		TIMER1n->BDR = pPwmOneshotCfg->TnBDR;		
	}
	else if (TimerCounterMode == CAPTURE_MODE)
	{
		pCaptureCfg = (TIMER1n_CAPTURECFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			| TIMER1n_CR_CKSEL_SET(pCaptureCfg->CkSel)	// (n<<14)
			| TIMER1n_CR_MODE_SET(TimerCounterMode)	// (n<<12)
			| TIMER1n_CR_CPOL_SET(pCaptureCfg->ClrMode) // (n<<6)
			;
		if (pCaptureCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER1n_CR_ECE_SET(pCaptureCfg->ECE);
		}
		TIMER1n->CR = reg_val16;
		
		TIMER1n->PREDR = ((pCaptureCfg->Prescaler-1) & T1n_PRS_MASK);
		TIMER1n->ADR = pCaptureCfg->TnADR;		
	}
	TIMER1n->CR |= 0x1; //  timer counter clear
	return HAL_OK;
}

/**
* @brief		Close Timer/Counter device
* @param		TIMER1n  Pointer to timer device, should be:
*					- TIMER1n peripheral n=0..4
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_DeInit (TIMER1n_Type *TIMER1n)
{
	// Disable timer/counter
	TIMER1n->CR = 0x00;

	if (TIMER1n== TIMER10)
	{
		SCU->PER1 &= ~(1<<16);
		SCU->PCER1 &= ~(1<<16);
	}
	else if (TIMER1n== TIMER11)
	{
		SCU->PER1 &= ~(1<<17);
		SCU->PCER1 &= ~(1<<17);
	}
	else if (TIMER1n== TIMER12)
	{
		SCU->PER1 &= ~(1<<18);
		SCU->PCER1 &= ~(1<<18);
	}
	else if (TIMER1n== TIMER13)
	{
		SCU->PER1 &= ~(1<<19);
		SCU->PCER1 &= ~(1<<19);
	}
	return HAL_OK;
}

/**
* @brief		Start/Stop Timer/Counter device
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @param		NewState
* 					- ENABLE  	:Set timer enable
* 					- DISABLE 	:Disable timer
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_Cmd(TIMER1n_Type *TIMER1n, FunctionalState NewState)
{
	if (NewState == ENABLE) {
		TIMER1n->CR |=  T1n_ENABLE;	
	}
	else {
		TIMER1n->CR &= ~T1n_ENABLE;
	}
	return HAL_OK;
}



/**
* @brief		Enable or disable TIMER interrupt.
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @param		TIMERIntCfg	Specifies the interrupt flag,
* 				should be one of the following:
*						- TIMER1n_INTCFG_MIE : OVIE Interrupt enable
*						- TIMER1n_INTCFG_CIE : MBIE Interrupt enable
* @param		NewState New state of specified TIMER1n interrupt type,
* 				should be:
* 					- ENALBE	:Enable this interrupt type.
* 					- DISALBE :Disable this interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_ConfigInterrupt(TIMER1n_Type *TIMER1n, TIMER1n_INT_Type TIMERIntCfg, FunctionalState NewState)
{
	uint8_t tmp;

	switch(TIMERIntCfg){
		case TIMER1n_INTCFG_MIE:
			tmp = T1n_MATINTEN;
			break;
		case TIMER1n_INTCFG_CIE:
			tmp = T1n_CAPINTEN;
			break;			
	}

	if (NewState == ENABLE)
	{
		TIMER1n->CR |= tmp;
	}
	else // DISABLE
	{
		TIMER1n->CR &= (~tmp);
	}
	return HAL_OK;
}

/**
* @brief		Timer 1n Counter and Prescaler clear
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_ClearCounter(TIMER1n_Type *TIMER1n)
{
	TIMER1n->CR |= (1<<0);
	return HAL_OK;
}

/**
* @brief		Get flags of Timer 1n Match Interrupt
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		HAL_Status
*/
uint32_t HAL_TIMER1n_GetMatchInterrupt(TIMER1n_Type *TIMER1n)
{
	if (TIMER1n->CR & (1<<3)){
		return 1;
	}
	else {
		return 0;
	}
}

/**
* @brief		Clear flags of Timer 1n Match Interrupt
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_ClearMatchInterrupt(TIMER1n_Type *TIMER1n)
{
	TIMER1n->CR |= (1<<3);
	return HAL_OK;
}

/**
* @brief		Get Timer 1n Capture Data Register
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		Capture Data
*/
uint32_t HAL_TIMER1n_GetCaptureData(TIMER1n_Type *TIMER1n)
{
	return (TIMER1n->CAPDR);
}

/**
* @brief		Get flags of Timer 1n Capture Interrupt
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		Capture Interrupt flag
*/
uint32_t HAL_TIMER1n_GetCaptureInterrupt(TIMER1n_Type *TIMER1n)
{
	if (TIMER1n->CR & (1<<2)){
		return 1;
	}
	else {
		return 0;
	}	
}

/**
* @brief		Clear flags of Timer 1n Capture Interrupt
* @param		TIMER1n Pointer to timer device, should be:
* 					- TIMER1n peripheral n=0..4
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER1n_ClearCaptureInterrupt(TIMER1n_Type *TIMER1n)
{
	TIMER1n->CR |= (1<<2);
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */

