/**
* @file		A31G32x_hal_timer2n.c
* @brief	Contains all functions support for timer2n firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_timer2n.h"


/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initial Timer/Counter device
* 				 	Set Clock frequency for Timer
* 					Set initial configuration for Timer
* @param		TIMER2n  Timer selection, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @param		TimerCounterMode Timer counter mode, should be:
* 					- PERIODIC_MODE			:Timer mode
* 					- PWM_MODE	:Counter rising mode
* 					- ONESHOT_MODE	:Counter falling mode
* 					- CAPTURE_MODE		:Counter on both edges
* @param		TIMER_ConfigStruct pointer to TIM_TIMERCFG_Type
*				that contains the configuration information for the
*				specified Timer peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_Init(TIMER2n_Type *TIMER2n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct)
{
	TIMER2n_PERIODICCFG_Type *pTimeCfg;
	TIMER2n_PWMCFG_Type *pPwmOneshotCfg;
	TIMER2n_CAPTURECFG_Type *pCaptureCfg;
	uint32_t reg_val16;
	
	if (TIMER2n== TIMER20)
	{
		SCU->PER1 &= ~(1<<25);
		SCU->PCER1 &= ~(1<<25);
		
		SCU->PER1 |= (1<<25);
		SCU->PCER1 |= (1<<25);		
	}
	else if (TIMER2n== TIMER21)
	{
		SCU->PER1 &= ~(1<<26);
		SCU->PCER1 &= ~(1<<26);
		
		SCU->PER1 |= (1<<26);
		SCU->PCER1 |= (1<<26);		
	}
	
	if (TimerCounterMode == PERIODIC_MODE)
	{
		pTimeCfg = (TIMER2n_PERIODICCFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			|TIMER2n_CR_CLK_SET(pTimeCfg->CkSel)
			|TIMER2n_CR_MODE_SET(TimerCounterMode)		
			|TIMER2n_CR_OPOL_SET(pTimeCfg->StartLevel)
			;
		if (pTimeCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER2n_CR_ECE_SET(pTimeCfg->ECE);
		}		
		TIMER2n->CR = reg_val16;
		
		TIMER2n->PREDR = ((pTimeCfg->Prescaler-1) & TIMER2n_PREDR_MASK);
		TIMER2n->ADR = pTimeCfg->T2nADR;
	}
	else if ((TimerCounterMode == PWM_MODE) || (TimerCounterMode == ONESHOT_MODE))
	{
		pPwmOneshotCfg = (TIMER2n_PWMCFG_Type *)TIMER_ConfigStruct;

		reg_val16 = 0
			|TIMER2n_CR_CLK_SET(pPwmOneshotCfg->CkSel)
			|TIMER2n_CR_MODE_SET(TimerCounterMode)			
			|TIMER2n_CR_OPOL_SET(pPwmOneshotCfg->StartLevel)
			;
		if (pTimeCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER2n_CR_ECE_SET(pPwmOneshotCfg->ECE);
		}			
		TIMER2n->CR = reg_val16;
		
		TIMER2n->PREDR = ((pPwmOneshotCfg->Prescaler-1) & TIMER2n_PREDR_MASK);
		TIMER2n->ADR = pPwmOneshotCfg->T2nADR;
		TIMER2n->BDR = pPwmOneshotCfg->T2nBDR;
	}
	else if (TimerCounterMode == CAPTURE_MODE)
	{
		pCaptureCfg = (TIMER2n_CAPTURECFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			|TIMER2n_CR_CLK_SET(pCaptureCfg->CkSel)
			|TIMER2n_CR_MODE_SET(TimerCounterMode)			
			|TIMER2n_CR_CPOL_SET(pCaptureCfg->ClrMode)
			|TIMER2n_CR_CAPSEL_SET(pCaptureCfg->CAPCkSel)
			|TIMER2n_CR_CNCLR_SET(pCaptureCfg->CNCLR)
			;
		if (pTimeCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER2n_CR_ECE_SET(pCaptureCfg->ECE);
		}			
		TIMER2n->CR = reg_val16;
		
		TIMER2n->PREDR = ((pCaptureCfg->Prescaler-1) & TIMER2n_PREDR_MASK);
		TIMER2n->ADR = pCaptureCfg->T2nADR;
	}
	
	TIMER2n->CR |= TIMER2n_CLEAR;
	return HAL_OK;
}

/**
* @brief 		Close Timer/Counter device
* @param		TIMx  Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_DeInit (TIMER2n_Type *TIMER2n)
{
	// Disable timer/counter
	TIMER2n->CR = 0;
	
	if (TIMER2n== TIMER20)
	{
		SCU->PER1 &= ~(1<<25);
		SCU->PCER1 &= ~(1<<25);
	}
	else if (TIMER2n== TIMER21)
	{
		SCU->PER1 &= ~(1<<26);
		SCU->PCER1 &= ~(1<<26);
	}	
	return HAL_OK;
}

/**
* @brief	 	Start/Stop Timer/Counter device
* @param		TIMER2n Pointer to timer device, should be:
*					- TIMER20	:TIMER20 peripheral
*					- TIMER21	:TIMER21 peripheral
* @param		NewState
*					- ENABLE  	:Set timer enable
*					- DISABLE 	:Disable timer
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_Cmd(TIMER2n_Type *TIMER2n, FunctionalState NewState)
{
	if (NewState == ENABLE) {
		TIMER2n->CR |= (1<<15);
	}
	else {
		TIMER2n->CR &= ~(1<<15);
	}
	return HAL_OK;
}

/**
* @brief 		Enable or disable TIMER interrupt.
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @param		UARTIntCfg	Specifies the interrupt flag,
* 				should be one of the following:
*						- TIMER_CR_MATCH_INTR : Match interrupt enable
*						- TIMER_CR_CAPTURE_INTR : Capture interrupt enable 
* @param		NewState New state of specified UART interrupt type,
* 				should be:
* 					- ENALBE	:Enable this interrupt type.
* 					- DISALBE	:Disable this interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_ConfigInterrupt(TIMER2n_Type *TIMER2n, TIMER2n_INT_Type TIMERIntCfg, FunctionalState NewState)
{
	switch(TIMERIntCfg){
		case TIMER2n_CR_MATCH_INTR:
			if (NewState == ENABLE)
				TIMER2n->CR |= (1<<5);
			else if (NewState == DISABLE)
				TIMER2n->CR &= ~(1<<5);
			break;
		case TIMER2n_CR_CAPTURE_INTR:
			if (NewState == ENABLE)
				TIMER2n->CR |= (1<<4);
			else if (NewState == DISABLE)
				TIMER2n->CR &= ~(1<<4);
			break;
	}
	return HAL_OK;
}

/**
* @brief 		Clear Timer/Counter device,
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_ClearCounter(TIMER2n_Type *TIMER2n)
{
	TIMER2n->CR |= (1<<0); //CLEAR;
	return HAL_OK;
}

/**
* @brief 		Get flags of Timer 2n Match Interrupt
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		return flag
*/
uint32_t HAL_TIMER2n_GetMatchInterrupt(TIMER2n_Type *TIMER2n)
{
	if (TIMER2n->CR & (1<<3)){
		return (1);
	}
	else {
		return (0);
	}
}

/**
* @brief 		Clear flags of Timer 2n Match Interrupt
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_ClearMatchInterrupt(TIMER2n_Type *TIMER2n)
{
	TIMER2n->CR |= (1<<3);
	return HAL_OK;
}

/**
* @brief 		Get Timer 2n Capture Data Register
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		return Capture Data
*/
uint32_t HAL_TIMER2n_GetCaptureData(TIMER2n_Type *TIMER2n)
{
	return (TIMER2n->CAPDR);
}

/**
* @brief 		Get flags of Timer 2n Capture Interrupt
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		return flag
*/
uint32_t HAL_TIMER2n_GetCaptureInterrupt(TIMER2n_Type *TIMER2n)
{
	if (TIMER2n->CR & (1<<2)){
		return (1);
	}
	else {
		return (0);
	}
}

/**
* @brief 		Clear flags of Timer 2n Capture Interrupt
* @param		TIMER2n Pointer to timer device, should be:
* 					- TIMER20	:TIMER20 peripheral
* 					- TIMER21	:TIMER21 peripheral
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER2n_ClearCaptureInterrupt(TIMER2n_Type *TIMER2n)
{
	TIMER2n->CR |= (1<<2);
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */
