/**
* @file		A31G32x_hal_timer30.c
* @brief	Contains all functions support for timer30 firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_timer30.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initializes the Timer30 peripheral according to the specified
*				parameters.
* @param		TIMER30		Timer 30 peripheral selected, should be:
*						- TIMER30	:Timer30 peripheral
* @param		TIMER_ConfigStruct pointer to TIMER3n_CFG_Type
*				that contains the configuration information for the
*				specified Timer peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_Init(TIMER3n_Type* TIMER3n, void *TIMER_ConfigStruct)
{
	TIMER3n_CFG_Type *pTimeConfig;
	uint16_t reg_val16;
	pTimeConfig = (TIMER3n_CFG_Type *)TIMER_ConfigStruct;	
	
	//timer30 peri enable 
	SCU->PER1 &= ~(1<<24);
	SCU->PCER1 &= ~(1<<24);
		
	SCU->PER1 |= (1<<24);
	SCU->PCER1 |= (1<<24);	
	
	reg_val16 = 0
	| pTimeConfig->CLK		// T30CLK[14]
	| pTimeConfig->MS		// T30MS[13:12]
	| pTimeConfig->ECE	// ECE[11]
	| pTimeConfig->FORCA	// FORCA[10]
	| pTimeConfig->UPDT	// UPDT[5:4]
	| pTimeConfig->PMOC	// PMOC[3:1]
	| pTimeConfig->CPOL
	;
	
	TIMER30->PREDR = pTimeConfig->Prescaler;
	
	TIMER30->CR = reg_val16;
	
	TIMER30->CR |= 0x01; //Clear T30 & Precaler	
	
	return HAL_OK;
}

/**
* @brief		Interrupt Control Register
* @param		TIMER30		Timer 30 peripheral selected, should be:
* 					- TIMER30	:	Timer30 peripheral
* @param		NewState 	New State of Timer30 peripheral's operation, should be:
* 					- ENABLE
* 					- DISABLE
* @param		T30IntCfg 	Interrupt Sourtce Setting
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_ConfigInterrupt(TIMER3n_Type* TIMER3n,uint32_t NewState,uint32_t T30IntCfg)
{
	if (NewState == ENABLE) {
		TIMER3n->INTCR |= T30IntCfg;
	}
	else 	{
		TIMER3n->INTCR &= (~T30IntCfg) & 0x7f;
	}
	return HAL_OK;
}

/**
* @brief		Get Interrupt Flag
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral 					
* @return		return interrept flag
*/
uint32_t HAL_TIMER3n_GetStatus_IT(TIMER3n_Type* TIMER3n)
{
	return TIMER3n->INTFLAG;	
}

/**
* @brief		Interrupt Flag Clear
* @param		TIMER30		Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		T30IntCfg 	Select clear interrupt
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_ClearStatus_IT(TIMER3n_Type* TIMER3n, uint32_t T30IntCfg)
{
	TIMER3n->INTFLAG = T30IntCfg;
	return HAL_OK;
}

/**
* @brief		Enable or Disable PWM start
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		NewState New State of T30 peripheral's operation, should be:
* 					- ENABLE
* 					- DISABLE
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_Start(TIMER3n_Type* TIMER3n, uint32_t NewState)
{
	if (NewState == ENABLE) {
		TIMER3n->CR |= T30_ENABLE;
	}
	else 	{
		TIMER3n->CR &= ~T30_ENABLE;
	}
	return HAL_OK;
}

/**
* @brief		PWM Output Port Control Register (Initial : 6channel enable, output low)
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		NewState New State of MPWMx peripheral's operation, should be:
* 					- ENABLE
* 					- DISABLE
* @param		pwmApol	Timer 30xA Output Polarity Selection
* @param		pwmBpol	Timer 30xB Output Polarity Selection
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_OutputCtrl(TIMER3n_Type* TIMER3n,  uint32_t NewState, uint32_t pwmApol, uint32_t pwmBpol)
{
	uint32_t temp;
	temp = 0
	| (0xE06CuL<<TIMER30_OUTCR_WTIDKY_Pos) 
	| (pwmApol | pwmBpol)
	;	
	if(NewState == ENABLE){
		temp |= (0x3f<<8);
	}
	
	TIMER3n->OUTCR = temp;
	return HAL_OK;
}

/**
* @brief 		Set period data .
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		period			MPWM period data									
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetPeriod(TIMER3n_Type* TIMER3n, uint32_t period)
{
	TIMER3n->PDR = (period&0xffff);  // period. it sould be larger than 0x10
	return HAL_OK;
}

/**
* @brief 		Set duty A data .
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		aduty			Timer30 Aduty data	 							
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetADuty(TIMER3n_Type* TIMER3n, uint32_t aduty)
{
	TIMER3n->ADR = ((aduty) &0xffff);  // if using I/O control function, set period data
	return HAL_OK;
}

/**
* @brief 		Set duty B data .
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		bduty			Timer30 Bduty data	 							
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetBDuty(TIMER3n_Type* TIMER3n, uint32_t bduty)
{
	TIMER3n->BDR = ((bduty) &0xffff);  // if using I/O control function, set period data
	return HAL_OK;
}

/**
* @brief 		Set duty C data .
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		cduty			Timer30 Cduty data	 							
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetCDuty(TIMER3n_Type* TIMER3n, uint32_t cduty)
{
	TIMER3n->CDR = ((cduty) &0xffff);  // if using I/O control function, set period data
	return HAL_OK;
}

/**
* @brief 		Set dead time (delay time)
* @param		TIMER30	Timer 30 peripheral selected, should be:
* 					- TIMER30	:Timer30 peripheral
* @param		dten			dead time enable 
* @param		dtpos		dead timer position (front or back) 
* @param		clkdata		dead time 							
* @return 		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetDelayTime(TIMER3n_Type* TIMER3n, uint32_t dten, uint32_t dtpos, uint32_t clkdata)
{	
	uint32_t temp;
	
	temp = TIMER3n->CR;
	temp &= ~(0x03<<8);
	temp |= dten | dtpos;	
 	TIMER3n->CR = temp;
	
	TIMER3n->DLY =	clkdata&0x03ff;     
	return HAL_OK;
}

/**
* @brief		Set Timer 30 T30HIZCR Register
* @param		u32T30HizSet 			Timer 30 Output High-Impedance Setting Data 
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetHizReg(TIMER3n_Type* TIMER3n, uint32_t u32T30HizSet)
{
	TIMER3n->HIZCR = u32T30HizSet;  //Setting Timer 30 High-Impedance Control Register
	return HAL_OK;
}

/**
* @brief		Set Timer30 ADC Tirgger Source & Timing 
* @param		u32triggerpoint 	Timer 30 Output High-Impedance Setting Data 
* @param		u32triggertime 		Timer 30 A/DC Trigger Setting Data 
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER3n_SetADCTrigger(TIMER3n_Type* TIMER3n, uint32_t u32triggerpoint, uint32_t u32triggertime)
{
	TIMER3n->ADTCR = u32triggerpoint;  //Setting Timer 30 A/DC Trigger Control Register
	TIMER3n->ADTDR = u32triggertime;
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */

