/**
* @file		A31G32x_hal_timer40.c
* @brief	Contains all functions support for timer40 firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_timer40.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/**
* @brief		Initial Timer/Counter device
* 				 	Set Clock frequency for Timer
* 					Set initial configuration for Timer
* @param		TIMER40 Timer selection, should be:
* 					- TIMER40 peripheral n=0
* @param		TimerCounterMode Timer counter mode, should be:
* 					- PERIODIC_MODE			:Timer mode
* 					- PWM_MODE	:Counter rising mode
* 					- ONESHOT_MODE	:Counter falling mode
* 					- CAPTURE_MODE		:Counter on both edges
* @param		TIMER_ConfigStruct pointer to TIMER_ConfigStruct
* 				that contains the configuration information for the
*				specified Timer peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_Init(TIMER40_Type *TIMER4n, TIMER_MODE_OPT TimerCounterMode, void *TIMER_ConfigStruct)
{
	TIMER40_PERIODICCFG_Type *pTimeCfg;
	TIMER40_PWMCFG_Type *pPwmOneshotCfg;
	TIMER40_CAPTURECFG_Type *pCaptureCfg;
	uint16_t reg_val16;
	
	if (TIMER4n== TIMER40)
	{
		SCU->PER1 &= ~(1<<20);
		SCU->PCER1 &= ~(1<<20);
		
		SCU->PER1 |= (1<<20);
		SCU->PCER1 |= (1<<20);		
	}
	else
	{
		
	}
	
	if (TimerCounterMode == PERIODIC_MODE)
	{
		pTimeCfg = (TIMER40_PERIODICCFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			| TIMER40_CR_CKSEL_SET(pTimeCfg->CkSel)
			| TIMER40_CR_MODE_SET(TimerCounterMode)
			| TIMER40_CR_STARTLVL_SET(pTimeCfg->StartLevel)
			;
		if (pTimeCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER40_CR_ECE_SET(pTimeCfg->ECE);
		}
		TIMER4n->CR = reg_val16;
		
		TIMER4n->PREDR = ((pTimeCfg->Prescaler-1) & T40_PRS_MASK);
		TIMER4n->ADR = pTimeCfg->TnADR;
	}
	else if ((TimerCounterMode == PWM_MODE) || (TimerCounterMode == ONESHOT_MODE))
	{
		pPwmOneshotCfg = (TIMER40_PWMCFG_Type *)TIMER_ConfigStruct;

		reg_val16 = 0
			| TIMER40_CR_CKSEL_SET(pPwmOneshotCfg->CkSel)
			| TIMER40_CR_MODE_SET(TimerCounterMode)
			| TIMER40_CR_STARTLVL_SET(pPwmOneshotCfg->StartLevel)
			;
		if (pPwmOneshotCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER40_CR_ECE_SET(pPwmOneshotCfg->ECE);
		}		
		TIMER4n->CR = reg_val16;

		TIMER4n->PREDR = ((pPwmOneshotCfg->Prescaler-1) & T40_PRS_MASK);
		TIMER4n->ADR = pPwmOneshotCfg->TnADR;
		TIMER4n->BDR = pPwmOneshotCfg->TnBDR;		
	}
	else if (TimerCounterMode == CAPTURE_MODE)
	{
		pCaptureCfg = (TIMER40_CAPTURECFG_Type *)TIMER_ConfigStruct;
		
		reg_val16 = 0
			| TIMER40_CR_CKSEL_SET(pCaptureCfg->CkSel)	// (n<<14)
			| TIMER40_CR_MODE_SET(TimerCounterMode)	// (n<<12)
			| TIMER40_CR_CAPCHSEL(pCaptureCfg->Channel) // (n<<9)
			| TIMER40_CR_CPOL_SET(pCaptureCfg->ClrMode) // (n<<6)
			;
		if (pCaptureCfg->CkSel==1) {
			reg_val16 = reg_val16 | TIMER40_CR_ECE_SET(pCaptureCfg->ECE);
		}
		TIMER4n->CR = reg_val16;
		TIMER4n->PREDR = ((pCaptureCfg->Prescaler-1) & T40_PRS_MASK);
		TIMER4n->PREDR2 = ((pCaptureCfg->Prescaler_capture-1) & T40_PRS_MASK);
		TIMER4n->ADR = pCaptureCfg->TnADR;		
		
	}
	TIMER4n->CR |= 0x1; //  timer counter clear
	return HAL_OK;
}

/**
* @brief		Close Timer/Counter device
* @param		TIMER40  Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_DeInit (TIMER40_Type *TIMER4n)
{
	// Disable timer/counter
	TIMER40->CR = 0x00;

	if (TIMER4n== TIMER40)
	{
		SCU->PER1 &= ~(1<<20);
		SCU->PCER1 &= ~(1<<20);
	}
	else
	{
		
	}
	return HAL_OK;
	
}

/**
* @brief	 	Start/Stop Timer/Counter device
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @param		NewState
* 					- ENABLE  	:Set timer enable
* 					- DISABLE 	:Disable timer
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_Cmd(TIMER40_Type *TIMER4n, FunctionalState NewState)
{
	if (NewState == ENABLE) {
		TIMER4n->CR |=  T40_ENABLE;	
	}
	else {
		TIMER4n->CR &= ~T40_ENABLE;
	}
	return HAL_OK;
}

/**
* @brief 		Enable or disable TIMER interrupt.
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @param		TIMERIntCfg	Specifies the interrupt flag,
* 				should be one of the following:
*						- TIMER40_INTCFG_MIE : OVIE Interrupt enable
*						- TIMER40_INTCFG_CIE : MBIE Interrupt enable
* @param		NewState New state of specified TIMER40 interrupt type,
* 				should be:
* 					- ENALBE	:Enable this interrupt type.
* 					- DISALBE 	:Disable this interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_ConfigInterrupt(TIMER40_Type *TIMER4n, TIMER40_INT_Type TIMERIntCfg, FunctionalState NewState)
{
	uint8_t tmp;

	switch(TIMERIntCfg){
		case TIMER40_INTCFG_MIE:
			tmp = T40_MATINTEN;
			break;
		case TIMER40_INTCFG_CIE:
			tmp = T40_CAPINTEN;
			break;			
	}

	if (NewState == ENABLE)
	{
		TIMER4n->CR |= tmp;
	}
	else // DISABLE
	{
		TIMER4n->CR &= (~tmp);
	}
	return HAL_OK;
}

/**
* @brief		Timer 40 Counter and Prescaler clear
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_ClearCounter(TIMER40_Type *TIMER4n)
{
	TIMER4n->CR |= (1<<0);
	return HAL_OK;
}

/**
* @brief		Get flags of Timer 40 Match Interrupt
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		Match Interrupt flags
*/
uint32_t HAL_TIMER4n_GetMatchInterrupt(TIMER40_Type *TIMER4n)
{
	if (TIMER4n->CR & (1<<3)){
		return 1;
	}
	else {
		return 0;
	}
}

/**
* @brief		Clear flags of Timer 40 Match Interrupt
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_ClearMatchInterrupt(TIMER40_Type *TIMER4n)
{
	TIMER4n->CR |= (1<<3);
	return HAL_OK;
}

/**
* @brief		Get Timer 40 Capture Data Register
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		Capture Data
*/
uint32_t HAL_TIMER4n_GetCaptureData(TIMER40_Type *TIMER4n)
{
	return (TIMER4n->CAPDR);
}

/**
* @brief		Get flags of Timer 40 Capture Interrupt
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		Capture Interrupt flags
*/
uint32_t HAL_TIMER4n_GetCaptureInterrupt(TIMER40_Type *TIMER4n)
{
	if (TIMER4n->CR & (1<<2)){
		return 1;
	}
	else {
		return 0;
	}	
}

/**
* @brief		Clear flags of Timer 40 Capture Interrupt
* @param		TIMER40 Pointer to timer device, should be:
* 					- TIMER40 peripheral n=0
* @return		HAL_Status
*/
HAL_Status_Type HAL_TIMER4n_ClearCaptureInterrupt(TIMER40_Type *TIMER4n)
{
	TIMER4n->CR |= (1<<2);
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */
