/**
* @file		A31G32x_hal_usartn.c
* @brief	Contains all functions support for usart firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_usartn.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t UsartBaseClock; 

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
/**
* @brief		Determines best dividers to get a target clock rate
* @param		USTx	Pointer to selected USART peripheral, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		baudrate Desired USART baud rate.
* @return		None
*/
void usart_set_divisors(USART_Type *USTx, uint32_t baudrate)
{
	uint32_t numerator; 
	uint32_t denominator; 
  uint32_t fpc;
  float bdr;
	//------------------------------------------------------------------------------
	// numerator & denominator 
	// 
	// bdr = (UsartBaseClock) / n / baudrate  - 1
	//------------------------------------------------------------------------------
	numerator = UsartBaseClock; 
	denominator = baudrate; 

	bdr = ((float)numerator / (float)denominator) - 1; 
	USTx->BDR = ((uint16_t)bdr)&0xffff;
  fpc = (uint32_t)((bdr - (uint16_t)bdr) * 256);
  USTx->FPCR = fpc;
}

/*******************************************************************************
* Public Function
*******************************************************************************/
/**
* @brief		Initializes the USART1x peripheral according to the specified
*				parameters in the USART_ConfigStruct.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		USART_ConfigStruct Pointer to a USART_CFG_Type structure
*				that contains the configuration information for the
*				specified USART peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_Init(USART_Type *USTx, USART_CFG_Type *USART_ConfigStruct)
{
	uint32_t tmp;

	if(USTx == USART10)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART10_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART10_Msk);		
		/* Set up peripheral clock for USART10 module */
		SCU->PER2   |=  (SCU_PER2_USART10_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART10_Msk);		
	}
	else if(USTx == USART11)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART11_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART11_Msk);		
		/* Set up peripheral clock for USART11 module */
		SCU->PER2   |=  (SCU_PER2_USART11_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART11_Msk);		
	}
    else if(USTx == USART12)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART12_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART12_Msk);		
		/* Set up peripheral clock for USART12 module */
		SCU->PER2   |=  (SCU_PER2_USART12_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART12_Msk);	
	}
    else if(USTx == USART13)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART13_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART13_Msk);		
		/* Set up peripheral clock for USART13 module */
		SCU->PER2   |=  (SCU_PER2_USART13_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART13_Msk);	
	}
    
    tmp = 0
        | ((USART_ConfigStruct->Mode & 0x3) << USART_CR1_USTnMS_Pos)
        | ((USART_ConfigStruct->Parity & 0x3) << USART_CR1_USTnP_Pos)   // only UART mode
        | ((USART_ConfigStruct->Databits & 0x7) << USART_CR1_USTnS_Pos)
        | ((USART_ConfigStruct->Order & 0x1) << USART_CR1_ORDn_Pos)     // only SPI mode
				| ((USART_ConfigStruct->ACK & 0x1) << USART_CR1_CPOLn_Pos)      // Sync. or SPI mode
				| ((USART_ConfigStruct->Edge & 0x1)  << USART_CR1_CPHAn_Pos)    // only SPI mode
        | (UST_CR1_TX_EN)
        | (UST_CR1_RX_EN); 
    
    USTx->CR1   =   tmp;
    
    if(USART_ConfigStruct->Mode == USART_SPI_MODE){     // Sync or SPI Master mode
        UsartBaseClock=SystemCoreClock >> 1;            // UsartBaseClock = SystemCoreClock / 2 
    }
    else {                                              // UART Mode Defalut
        UsartBaseClock=SystemCoreClock >> 4;            // UsartBaseClock = SystemCoreClock / 16
    }
    usart_set_divisors(USTx, USART_ConfigStruct->Baud_rate);
		
		tmp = 0
		| ( UST_CR2_FPC_EN )
		| ( UST_CR2_UST_EN )
		| ((USART_ConfigStruct->Stopbits & 0x1)<<UST_CR2_USTnSB_Pos);
    
    USTx->CR2   =   tmp;
//		USTx->CR2		|= 	(1<<7);
    
    //Dummy Read
    tmp = HAL_USART_ReceiveByte(USTx);
    tmp = HAL_USART_ReceiveByte(USTx);
	return HAL_OK;
}

/**
* @brief		De-initializes the USTx peripheral registers to their
*				default reset values.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_DeInit(USART_Type* USTx)
{
	if(USTx == USART10)
	{
		SCU->PER2   &=  ~(SCU_PRER2_USART10_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART10_Msk);
    USTx->CR2   &=  ~(UST_CR2_RTO_EN|UST_CR2_FPC_EN|UST_CR2_RTO_EN);
	}
	else if(USTx == USART11)
	{
		SCU->PER2   &=  ~(SCU_PRER2_USART11_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART11_Msk);
    USTx->CR2   &=  ~(UST_CR2_RTO_EN|UST_CR2_FPC_EN|UST_CR2_RTO_EN);
	}
	else if(USTx == USART12)
	{
		SCU->PER2   &=  ~(SCU_PRER2_USART12_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART12_Msk);
    USTx->CR2   &=  ~(UST_CR2_RTO_EN|UST_CR2_FPC_EN|UST_CR2_RTO_EN);
	}
	else if(USTx == USART13)
	{
		SCU->PER2   &=  ~(SCU_PRER2_USART13_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART13_Msk);
    USTx->CR2   &=  ~(UST_CR2_RTO_EN|UST_CR2_FPC_EN|UST_CR2_RTO_EN);
	}
	return HAL_OK;
}

/**
* @brief		USTx enable control
*				default reset values.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		state : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
 HAL_Status_Type HAL_USART_Enable(USART_Type* USTx, FunctionalState state)
{
    if(state == ENABLE){
        USTx->CR2 |= UST_CR2_UST_EN;
    }
    else {
        USTx->CR2 &= ~(UST_CR2_UST_EN);
    }
	return HAL_OK;
}

/**
* @brief		USTx RTO enable control
*				default reset values.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		state : enable/disable should be,
* 						- DISABLE	= 0
* 						- ENABLE	= 1
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_RTOEnable(USART_Type* USTx, FunctionalState state)
{
    if(state == ENABLE){
        USTx->CR2 |= UST_CR2_RTO_EN;
    }
    else {
        USTx->CR2 &= ~(UST_CR2_RTO_EN);
    }
		
		USTx->RTO = 0x0000f000;
	return HAL_OK;
}

/**
* @brief		Fills each USART_InitStruct member with its default value:
*				- 38400 bps
*				- 8-bit data
*				- 1 Stopbit
*				- None Parity
* @param		USART_InitStruct Pointer to a USART_CFG_Type structure which will be initialized.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_UART_Mode_Config(USART_CFG_Type *USART_InitStruct)
{
  USART_InitStruct->Mode = USART_UART_MODE;
	USART_InitStruct->Baud_rate = 38400;
	USART_InitStruct->Databits = USART_DATABIT_8;
	USART_InitStruct->Parity = USART_PARITY_NONE;
	USART_InitStruct->Stopbits = USART_STOPBIT_1;
	return HAL_OK;
}

/**
* @brief		Fills each USART_InitStruct member with its default value:
* @param		USART_InitStruct Pointer to a USART_CFG_Type structure which will be initialized.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_SPI_Mode_Config(USART_CFG_Type *USART_InitStruct)
{
    USART_InitStruct->Mode = USART_SPI_MODE;
	USART_InitStruct->Baud_rate = 38400;
	USART_InitStruct->Databits = USART_DATABIT_8;
    
    //only SPI
    USART_InitStruct->Order = USART_SPI_MSB;
#if 0 // CPOLn : 0, CPHAn : 0
    USART_InitStruct->ACK = USART_SPI_TX_RISING;
    USART_InitStruct->Edge = USART_SPI_TX_LEADEDGE_SAMPLE;
#endif
#if 1	// CPOLn : 0, CPHAn : 1
    USART_InitStruct->ACK = USART_SPI_TX_RISING;
    USART_InitStruct->Edge = USART_SPI_TX_LEADEDGE_SETUP;
#endif
    
#if 0 // CPOLn : 1, CPHAn : 0 
    USART_InitStruct->ACK = USART_SPI_TX_FALLING;
    USART_InitStruct->Edge = USART_SPI_TX_LEADEDGE_SAMPLE;
#endif
    
#if 0 // CPOLn : 1, CPHAn : 1
    USART_InitStruct->ACK = USART_SPI_TX_FALLING;
    USART_InitStruct->Edge = USART_SPI_TX_LEADEDGE_SETUP;
#endif
	return HAL_OK;

    
}

/**
* @brief		Fills each USART_InitStruct member with its default value:
*					- 38400 bps
*					- 8-bit data
*					- 1 Stopbit
*					- None Parity
* @param		USART_InitStruct Pointer to a USART_CFG_Type structure which will be initialized.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_USRT_Mode_Config(USART_CFG_Type *USART_InitStruct)
{
  USART_InitStruct->Mode = USART_SYNC_MODE;
	USART_InitStruct->Baud_rate = 38400;
	USART_InitStruct->Databits = USART_DATABIT_8;
	USART_InitStruct->Parity = USART_PARITY_NONE;
	USART_InitStruct->Stopbits = USART_STOPBIT_1;   
	return HAL_OK;
}

/**
* @brief		Transmit a single data through USART peripheral
* @param		USTx	USART peripheral selected, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @param		Data	Data to transmit (must be 8-bit long)
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_TransmitByte(USART_Type* USTx, uint8_t Data)
{
	USTx->DR = Data;
	return HAL_OK;
}

/**
* @brief		Receive a single data from USART peripheral
* @param		USTx	USART peripheral selected, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @return		Data received
*/
uint8_t HAL_USART_ReceiveByte(USART_Type* USTx)
{
	return (USTx->DR);
}

/**
* @brief		Send a block of data via USART peripheral
* @param		USTx	Selected USART peripheral used to send data, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @param		txbuf		Pointer to Transmit buffer
* @param		buflen	Length of Transmit buffer
* @return		Number of bytes sent.
*/
uint32_t HAL_USART_Transmit(USART_Type *USTx, uint8_t *txbuf, uint32_t buflen)
{
	uint32_t bToSend, bSent;
	uint8_t *pChar = txbuf;

	bToSend = buflen;
	bSent = 0;
	while (bToSend){          
		while(!(USTx->ST & UST_SR_DRE)){ }

		HAL_USART_TransmitByte(USTx, (*pChar));

		pChar++;
		bToSend--;
		bSent++;
	}
	
	return bSent;
}

/**
* @brief			Receive a block of data via USART peripheral
* @param		USTx	Selected USART peripheral used to send data,
* 				should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		rxbuf	Pointer to Received buffer
* @param		buflen	Length of Received buffer
* @param		flag		Flag mode, should be:
*					- NONE_BLOCKING
* 					- BLOCKING
* @return		Number of bytes received
*/
uint32_t HAL_USART_Receive(USART_Type *USTx, uint8_t *rxbuf, uint32_t buflen)
{
	uint32_t bToRecv, bRecv;
	uint8_t *pChar = rxbuf;

	bToRecv = buflen;
	bRecv = 0;
	while (bToRecv) {
		while(!(USTx->ST & UST_SR_RXC)){ }
		
		(*pChar++) = HAL_USART_ReceiveByte(USTx);
		bRecv++;
		bToRecv--;
	}
	return bRecv;
}

/**
* @brief 		Enable or disable specified USART interrupt.
* @param		USTx	USART peripheral selected, should be
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		USTIntCfg	Specifies the interrupt flag,
* 				should be one of the following:
* 					- USART_INTCFG_DR   : DR Interrupt enable
* 					- USART_INTCFG_TXC 	: TXC Interrupt enable
* 					- USART_INTCFG_RXC 	: RXC interrupt enable
* 					- USART_INTCFG_WAKE : WAKE Interrupt enable
* 					- USART_INTCFG_DMATX: DMA TX Interrupt enable
* 					- USART_INTCFG_DMARX: DMA RX Interrupt enable
* 					- USART_INTCFG_RTO  : RTO Interrupt enable
* @param		NewState New state of specified USART interrupt type,
* 				should be:
* 					- ENALBE	:Enable this USART interrupt type.
* 					- DISALBE	:Disable this USART interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_ConfigInterrupt(USART_Type *USTx, USART_INT_Type USTIntCfg, FunctionalState NewState)
{
	uint32_t tmp=0;
	
	switch(USTIntCfg){
		case USART_INTCFG_WAKE :
			tmp = UST_CR1_WAKEINT_EN;
			break;
		case USART_INTCFG_RXC:
			tmp = UST_CR1_RXCINT_EN;
			break;
		case USART_INTCFG_TXC:
			tmp = UST_CR1_TXCINT_EN;
			break;
		case USART_INTCFG_DR:
			tmp = UST_CR1_DRINT_EN;
			break;
        case USART_INTCFG_RTO:
			tmp = UST_CR2_RTOINT_EN;
			break;
        case USART_INTCFG_DMARX:
			tmp = UST_CR2_DMA_RXINT_EN;
			break;
        case USART_INTCFG_DMATX:
			tmp = UST_CR2_DMA_TXINT_EN;
			break;
        
	}
    
	if (NewState == ENABLE)
	{
		if(USTIntCfg < (USART_INTCFG_DR + 1 )) {
            USTx->CR1 |= tmp;
        }
        else if ( USTIntCfg <  (USART_INTCFG_DMATX + 1)) {
            USTx->CR2 |= tmp;
        }
	}
	else
	{
		if(USTIntCfg < (USART_INTCFG_DR + 1 )) {
            USTx->CR1 &= ~(tmp & UST_CR1_INT_BITMASK);
        }
        else if ( USTIntCfg <  (USART_INTCFG_DMATX + 1)) {
            USTx->CR1 &= ~(tmp & UST_CR2_INT_BITMASK);
        }
	}
	return HAL_OK;
}

/**
* @brief 		Get current value of Line Status register in USART peripheral.
* @param		USTx	USART peripheral selected, should be:
* 					- USART0	: USART0 peripheral
* 					- USART1	: USART1 peripheral
* 					- USART2	: USART2 peripheral
* 					- USART3	: USART3 peripheral 
* @return		Current value of Status register in USART peripheral.
*/
uint32_t HAL_USART_GetStatus(USART_Type* USTx)
{
	return ((USTx->ST) & UST_SR_BITMASK);
}

/**
* @brief 		Clear Status register in USART peripheral.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
*	 				- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_ClearStatus(USART_Type* USTx, USART_STATUS_Type Status)
{
    uint32_t tmp;
    
    switch(Status){
        case USART_STATUS_WAKE:
            tmp = UST_SR_WAKE;
            break;
        case USART_STATUS_RXC:
            tmp = UST_SR_RXC;
            break;
        case USART_STATUS_TXC:
            tmp = UST_SR_TXC;
            break;
        case USART_STATUS_DRE:
            tmp = UST_SR_DRE;
            break;
        case USART_STATUS_RTO:
            tmp = UST_SR_RTO;
            break;
        case USART_STATUS_DMARX:
            tmp = UST_SR_DMARX;
            break;
        case USART_STATUS_DMATX:
            tmp = UST_SR_DMATX;
            break;
        default:
            break;
    }    
    
    USTx->ST = tmp;
	return HAL_OK;
}

/**
* @brief		Check whether if USART is busy or not
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @return		RESET if USART is not busy, otherwise return SET.
*/
FlagStatus HAL_USART_CheckBusy(USART_Type *USTx)
{
	if (USTx->ST & UST_SR_DRE){
		return RESET;
	} else {
		return SET;
	}
}

/**
* @brief		Configure Data Control mode for USART peripheral
* @param		USTx
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		Data Control mode, should be:
* 					- USART_CR_RTOEN     :Activate USARTn RTO block by supplying.
* 					- USART_CR_FPCEN     :Activate USARTn Floating Point counter.
* 					- USART_CR_USTEN     :Activate USARTn Block by supplying.
* 					- USART_CR_DBLS      :Selects receiver sampling rate. (only UART mode)
* 					- USART_CR_MASTER    :Selects master or slave in SPIn or Synchronous mode and controls the direction of SCKn pin.
* 					- USART_CR_LOOPS     :Control the Loop Back mode of USARTn for test mode.
* 					- USART_CR_DISSCK    :In synchronous mode operation, selects the waveform of SCKn output.
* 					- USART_CR_USTSSEN   :This bit controls the SSn pin operation. (only SPI mode)
* 					- USART_CR_FXCH      :SPIn port function exchange control bit. (only SPI mode)
* 					- USART_CR_USTSB     :Selects the length of stop bit in Asynchronous or Synchronous mode.
* 					- USART_CR_USTTX8    :The ninth bit of data frame in Asynchronous or Synchronous mode of operation. Write this bit first before loading the DR register.
* 					- USART_CR_USTRX8    :The ninth bit of data frame in Asynchronous or Synchronous mode of operation. Read this bit first before reading the receive buffer (only UART mode)
* @param		NewState New State of this mode, should be:
* 					- ENABLE	:Enable this mode.
*					- DISABLE	:Disable this mode.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_DataControlConfig(USART_Type *USTx, USART_CONTROL_Type Mode, FunctionalState NewState)
{
	uint16_t tmp=0;
    
	switch(Mode){
	case USART_CONTROL_USTRX8:
		tmp = UST_CR2_RX8;
		break;
	case USART_CONTROL_USTTX8:
		tmp = UST_CR2_TX8;
		break;
	case USART_CONTROL_USTSB:
		tmp = UST_CR2_STOPBIT;
		break;
	case USART_CONTROL_FXCH:
		tmp = UST_CR2_FXCH;
		break;	
	case USART_CONTROL_USTSSEN:
		tmp = UST_CR2_SSEN;
		break;	
	case USART_CONTROL_DISSCK:
		tmp = UST_CR2_DISSCK;
		break;	
	case USART_CONTROL_LOOPS:
		tmp = UST_CR2_LOOPS;
		break;	
	case USART_CONTROL_MASTER:
		tmp = UST_CR2_MASTER;
		break;	
	case USART_CONTROL_DBLS:
		tmp = UST_CR2_DBLS;
		break;	
	case USART_CONTROL_USTEN:
		tmp = UST_CR2_UST_EN;
		break;	
    case USART_CONTROL_FPCEN:
        tmp = UST_CR2_FPC_EN;
        break;  
    case USART_CONTROL_RTOEN:
        tmp = UST_CR2_RTO_EN;
        break;  
    default:
		break;
	}

	if (NewState == ENABLE)
	{
		USTx->CR2 |= tmp;
	}
	else
	{
		USTx->CR2 &= ~(tmp & UST_CR2_CONTROL_BITMASK) ;
	}
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */
