/**
* @file		A31G32x_hal_usb.c
* @brief	Contains all functions support for usb firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "a31g32x.h"
#include "A31G32x_hal_usb.h"
#include "A31G32x_hal_usb_core.h"
#include "A31G32x_usb_config.h"

#ifdef USB_MSC
#include "memory.h"
#endif

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/
#define __regw(x)       (*(volatile unsigned int *)(x))
#define USBCON			__regw(0x40001540)

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
#ifdef USB_MSC
#define MSC_ImageSize 8192
uint8_t Memory[MSC_ImageSize];                  // MSC Memory in RAM
#endif

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief      Initialize USB peripheral and core
* @param      None
* @return     None
*/
void USB_Init(void)
{
#ifdef USB_MSC
	uint32_t n;

  for (n = 0; n < MSC_ImageSize; n++) 
	{              // Copy Initial Disk Image
    Memory[n] = DiskImage[n];                        //   from Flash to RAM
  }
#endif
	SCU->PCER2 &=  ~(1<<16);
	SCU->PER2   &=  ~(1<<16);
	
	SCU->PCER2 |=  (1<<16);
	SCU->PER2   |=  (1<<16);
	
	NVIC_EnableIRQ(USB_IRQn);
		
	USB_CoreInit();
}

/**
* @brief      Reset USB core for interrupt usb reset event
* @param      None
* @return     None
*/
void USB_GINTSTS_USBRst(){  //[12]
    USB->DOEPCTL0 = (1 << 27)    ;
    USB->DOEPCTL1 = (1 << 27)    ;
    USB->DOEPCTL2 = (1 << 27)    ;
    USB->DOEPCTL3 = (1 << 27)    ;
    USB->DOEPCTL4 = (1 << 27)    ;
    USB->DIEPCTL0 = (1 << 27)    ;
    USB->DIEPCTL1 = (1 << 27)    ;
    USB->DIEPCTL2 = (1 << 27)    ;
    USB->DIEPCTL3 = (1 << 27)    ;
    USB->DIEPCTL4 = (1 << 27)    ;

	
    USB->DAINTMSK = 0
		| ( 1 << 16)	//OUTEP0
		| ( 1 << 0 )	//INEP0
		| ( 1 << 17)	//OUTEP1
		| ( 1 << 1 )	//INEP1
		;
	
    USB->DOEPMSK = 0
		| ( 1 << 3 )	//SetUPMsk
		| ( 1 << 0 )	//XferComplMsk
		;
    USB->DIEPMSK = 0
		| ( 1 << 3 )	//TimeOUTMsk
		| ( 1 << 0 )	//XferComplMsk
		;
		
    USB->GINTMSK = 0
		| (0x1 << 19)	//DOEPINT EN
		| (0x1 << 18)	//DIEPINT EN
		;
	
	USB_CoreInit();
	
	USB->DCFG &= ~(0x7f << 4); //clear
	USB->DCFG |= ((0x0) << 4);
		
	//FIFO size configuration
	USB->GRXFSIZ 	= 0x00000080; 
	USB->GNPTXFSIZ 	= 0x00400080;
	USB->DIEPTXF1 	= 0x004000C0;
    USB->DIEPTXF2 	= 0x00400100;
		
}

/**
* @brief      configure USB core for enumeration completion event
* @param      None
* @return     None
*/
void USB_GINTSTS_EnumDone(){
    // Initialization on Enumeration Completion
#if defined(USB_HID_KBD) || defined(USB_HID_PIPE_LS)
    USB->DIEPCTL0 = (0x3 << 0);      //8 Byte
#else
    USB->DIEPCTL0 = (0x0 << 0);      //64 Byte
#endif
    USB->DOEPCTL0 = (0x1UL << 31);   //EPEna
	
    USB->GINTMSK  |= (0x1<<3 );		//SofMsk Unmask
	
		//Wait Setuppacket
    USB_prepareSetupPacket();
}

/**
* @brief      configure USB core for interrupt suspend event
* @param      None
* @return     None
*/
void USB_GINTSTS_Suspend(){
	USB->PCGCCTL |= (1<<0);
	
	USB->PCGCCTL &= ~(1<<0);
	USB->DCTL |= (1<<1);
	
//	USB->GRSTCTL = 0x80000001;
//	while(USB->GRSTCTL & (0x1 << 0)) { __nop(); }
	
	USB_CoreInit();
}

/**
* @brief      Dummy fucntion(no operation) for interrupt sof(start of frame) event
* @param      None
* @return     None
*/
void USB_GINTSTS_SOF()
{
	__NOP();
}

/**
* @brief      configure USB core for interrupt In-Endpoint event
* @param      None
* @return     None
*/
void USB_GINTSTS_IEPInt(){
    volatile uint32_t usb_daint;
    
	//[IEPInt DAINT]
	usb_daint = USB->DAINT;

    //INEPInt0
    if(usb_daint & (1 <<  0)){
        USB_IEPInt_DAINT_InEPInt0();
    }
	//INEPInt1
    if(usb_daint & (1 <<  1)){
        USB_IEPInt_DAINT_InEPInt1();
    }
}

/**
* @brief      configure USB core for interrupt Out-Endpoint event
* @param      None
* @return     None
*/
void USB_GINTSTS_OEPInt(){
	volatile uint32_t usb_daint;
    usb_daint = USB->DAINT;
	//[OEPInt DAINT]

    //OutEPInt0
    if(usb_daint & (1 << 16)){
        USB_OEPInt_DAINT_OutEPInt0();
    }
	//OutEPInt1
    if(usb_daint & (1 << 17)){
		USB_OEPInt_DAINT_OutEPInt1();
    }
}
