/**
* @file		A31G32x_hal_usb_core.c
* @brief	Contains all functions support for usb core firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_usb_user.h"
#include "A31G32x_hal_usb.h"
#include "A31G32x_hal_usb_core.h"
#include "A31G32x_usb_config.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
volatile uint32_t   USB_DOEP0_DMA_DESC_RX[2];
volatile uint32_t   USB_DOEP0_DMA_BUF_RX[64];
volatile uint32_t   USB_DIEP0_DMA_DESC_TX[2];
volatile uint32_t   USB_DIEP0_DMA_BUF_TX[64];

volatile tSetup_stage_data_packet _setup_data_packet;

volatile uint32_t   USB_EP0_status = 0;

#ifdef USB_MSC
volatile uint8_t BulkStage;
uint8_t  BulkBuf[64]; /* Bulk In/Out Buffer */
#endif
#ifdef USB_CDC
extern volatile tCdc_line_coding _cdc_line_coding;
uint8_t cdc_ch[64];
#endif

#ifdef USB_HID_PIPE
uint8_t hid_ch[64];
#endif

#if defined(USB_HID_KBD) || defined(USB_HID_PIPE_LS)
uint8_t hid_ch[8];
#endif

#ifdef USB_HID
uint8_t hid_protocol_val, hid_idle_val;
#endif

uint16_t config_wValue, remote_wakeup;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief      Initialize USB(Device) core
* @param      None
* @return     None
*/
void USB_CoreInit(){
	uint16_t len;
	
	//USB Configuration
	USB->GUSBCFG = (0x1 << 6); // PHYsel
	
	//Device Configuration
#ifdef USB_HID_PIPE_LS
	USB->DCFG |= (0x2 << 0); //Device Speed : low Speed 6MHz
#else
	USB->DCFG |= (0x3 << 0); //Device Speed : Full Speed 48MHz
#endif
	
//	//Device Control
//	USB->DCTL = 0;
	
	//AHB Configuration
	USB->GAHBCFG = 0
		| (0x1 <<5 )	//DMA Mode Enable
		| (0x5 <<1 )	//DMA Burst length 8(64 words or INCR8)
		| (0x1 <<0 )	//Global interrupt Mask
		;
	
	//Device Threshold Control
	USB->DTHRCTL = 0x0;

	USB->DCTL = 0;

	//USB Interrupt
	USB->GINTSTS  = 0x0; //clear
	
	//Device Configuration
#ifdef USB_HID_PIPE_LS
	USB->DCFG     |= 0x0A841002;
#else
	USB->DCFG     |= 0x0A841003;
#endif

	//Device Control
	USB->DCTL     |= 0x00020000;

	USB->GINTMSK  |= 0
		| (0x1<<12)		//USBRstMsk Unmask
		| (0x1<<13)		//EnumDoneMsk Unmask
		| (0x1<<11)		//SuspMsk Unmask
		| (0x1<<10)		//ErlySuspMsk Unmask
		| (0x1<<3)		 	//SofMsk Unmask
		;


#ifdef USB_MSC
	//Insert Total Config Desc
	len = 32;

	config_desc[2] = LSB(len);
	config_desc[3] = MSB(len);
#endif

#ifdef USB_CDC
	//Insert Total Config Desc
	len = 75;

	config_desc[2] = LSB(len);
	config_desc[3] = MSB(len);
#endif

#ifdef USB_HID
	//Insert Total Config Desc
	len = 41;
	
	config_desc[2] = LSB(len);
	config_desc[3] = MSB(len);
#endif

}

/**
* @brief      Ready to send USB data buffer from Device to Host
* @param      length : data buffer length
* @param      desc : Pointer contains USB data
* @return     None
*/
void USB_ReadyTx(uint16_t length, uint8_t *desc){
	USB_DIEP0_DMA_DESC_TX[0] = 0x0A000000 | length;
    USB_DIEP0_DMA_DESC_TX[1] = (uint32_t)desc;
    
	USB->GRSTCTL = 0x80000020;   // TX FIFO FLUSH
    
	while(USB->GRSTCTL & (0x1 << 5)) { __NOP(); }
        
    USB->DIEPDMA0 = (uint32_t)USB_DIEP0_DMA_DESC_TX;
#ifdef USB_MSC
    USB->DIEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
	
#ifdef USB_CDC
		USB->DIEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
	
#ifdef USB_HID_PIPE_LS
		USB->DIEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif

#ifdef USB_HID_PIPE
		USB->DIEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31)  //EPEna
		;
#endif

#ifdef USB_HID_KBD
		USB->DIEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
}	

/**
* @brief      Ready to receive USB data buffer from Host to Device
* @param      length : data buffer length
* @param      desc : Pointer contains USB data
* @return     None
*/
void USB_ReadyRx(uint16_t length, uint8_t *desc){
	USB_DOEP0_DMA_DESC_RX[0] = 0x0A000000 | length;// | SWAP16(_setup_data_packet.wLength);
	USB_DOEP0_DMA_DESC_RX[1] = (uint32_t)desc;
	
	USB->GRSTCTL = 0x80000010;   // RX FIFO FLUSH
    
	while(USB->GRSTCTL & (0x1 << 4)) { __NOP(); }
	
	USB->DOEPDMA0 = (uint32_t)USB_DOEP0_DMA_DESC_RX;
#ifdef USB_MSC
    USB->DOEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
	
#ifdef USB_CDC
		USB->DOEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
	
#ifdef USB_HID_PIPE_LS
    	USB->DOEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif

#ifdef USB_HID_PIPE
		USB->DOEPCTL0 = 0
		| (0x0   <<  0) //64Byte
		| (0x1UL << 31)  //EPEna
		;
#endif

#ifdef USB_HID_KBD
		USB->DOEPCTL0 = 0
		| (0x3   <<  0) //8Byte
		| (0x1UL << 31)  //EPEna
		;
#endif
}

/**
* @brief      Set State(Data stage IN-Endpoint, from Device to Host) 
* @param      length : data buffer length
* @param      desc : Pointer contains USB data
* @return     None
*/
void USB_DataStageIN(uint16_t length, uint8_t *desc){
	USB_EP0_status = eUSB_EP0_DATA_IN;
	USB_ReadyTx(length, desc);
}

/**
* @brief      Set State(Data stage OUT-Endpoint, from Host to Device)  
* @param      length : data buffer length
* @param      desc : Pointer contains USB data
* @return     None
*/
void USB_DataStageOUT(uint16_t length, uint8_t *desc){
	USB_EP0_status = eUSB_EP0_DATA_OUT;
	USB_ReadyRx(length, desc);
}

/**
* @brief      Prepare Setup Packet State(Receive Setup Packet, from Host to Device)  
* @param      None
* @return     None
*/
void USB_prepareSetupPacket(){
	USB_EP0_status = eUSB_EP0_SETUP;
	USB_ReadyRx(0x08, (uint8_t*)&_setup_data_packet);

}

/**
* @brief      Response status state(Receive Setup Packet, from Host to Device)  
* @param      None
* @return     None
*/
void USB_StatusStageOut(){
	USB_EP0_status = eUSB_EP0_STATUS_OUT;
	USB_prepareSetupPacket();
}

/**
* @brief      Response status state(Send to ZLP(Zero Length Packet), from Device to Host)  
* @param      None
* @return     None
*/
void USB_StatusStageIn(){
	uint8_t tmp_zero = 0;
	USB_ReadyTx(0, &tmp_zero);
}

/**
* @brief      Response get usb descriptor to Host 
* @param      None
* @return     None
*/
void USB_GetDescriptor(){
	volatile uint16_t req, len;
	uint8_t *desc;
	
	req = SWAP16(_setup_data_packet.wValue)&0xFF;
	switch(req){
		case eUSB_DT_DEVICE:
			desc = device_desc;
			len = device_desc[0];
			break;
		case eUSB_DT_CONFIGURATION:
			desc = config_desc;
			len = _setup_data_packet.wLength & 0xffff;
			if(len == 0xff){
				
#ifdef USB_MSC
				len = 0x09;
#endif

#ifdef USB_CDC
				len = config_desc[3]<<8 | config_desc[2];
#endif
				
#ifdef USB_HID
				len = 0x09;
#endif
			}
			break;
		case eUSB_DT_STRING:
		switch(_setup_data_packet.wValue&0xFF){
				case 0:
					desc = lang_desc;
					len = lang_desc[0];
					break;
				case 1:
					desc = __manufact_desc;
					len = __manufact_desc[0];
					break;
				case 2:
					desc = __product_desc;
					len = __product_desc[0];
					break;
				case 3:
					desc = __serial_desc;
					len = __serial_desc[0];
					break;
			}
			break;
		case eUSB_DT_DEVICE_QUALIFIER:
			desc = device_qulifier_desc;
			len = device_qulifier_desc[0];
			break;
		case eUSB_DT_OTHER_SPEED_CONFIGURATION:
#ifdef USB_HID
			desc = other_speed_desc;
			len = _setup_data_packet.wLength & 0xffff;
			//len = other_speed_desc[0];
#endif
			break;
		case eUSB_DT_BOS:
			//stall 
			USB_EP0_status = eUSB_EP0_STALL;
		 break;
		default:
			//control error
			return;
	}
	
	if(req != eUSB_DT_BOS)
	{
		USB_DataStageIN(len, desc);
	}
}

/**
* @brief      Set address requested by host
* @param      None
* @return     None
*/
void USB_SetAddress(){
	volatile uint16_t addr;
	
	addr = _setup_data_packet.wValue & 0x7f;
	USB->DCFG &= ~(0x7f << 4); //clear
	USB->DCFG |= ((addr&0x7f) << 4);
	USB_StatusStageIn();
}

/**
* @brief      Get Remote Wakeup flag
* @param      None
* @return     remote_wakeup
*/
uint16_t USB_GetWakeupStatus(){
		return remote_wakeup;
}

/**
* @brief      Set configuration requested by host
* @param      None
* @return     None
*/
void USB_SetConfiguration(){
	
	//Set configuration data
	config_wValue = _setup_data_packet.wValue;

#ifdef USB_HID
	USB->DIEPCTL1 |= 0
		| (0x1 << 22)	// Tx FIFO Num
		| (0x3 << 18)	// Endpoint Type
		| (0x1 << 15)	// Endpoint is Activated
		;
	
		USB->DOEPCTL1 |= 0
		| (0x3 << 18)	// Endpoint Type
		| (0x1 << 15)	// Endpoint is Activated
		;
	
    USB->DAINTMSK |= 0
		|(1 << 17)	//OUTEP1
		|(1 << 1 )	//INEP1
		;
#else
	//CDC Data
	USB->DIEPCTL1 |= 0
		| (0x1 << 22)	// Tx FIFO Num
		| (0x2 << 18)	// Endpoint Type : BULK
		| (0x1 << 15)	// Endpoint is Activated
		;
    USB->DOEPCTL1 |= 0
		| (0x2 << 18)	// Endpoint Type : BULK
		| (0x1 << 15)	// Endpoint is Activated
		;
    USB->DAINTMSK |= 0
		|(1 << 17)	//OUTEP1
		|(1 << 1 )	//INEP1
		;
#endif	
	
#ifdef USB_MSC
	USB_StatusStageIn();
	USB_Receive_DATA_EP1(BulkBuf);
#endif
	
	
#ifdef USB_CDC
		//CDC Command
	USB->DIEPCTL2 |= 0
		| (0x1 << 22)	// Tx FIFO Num
		| (0x3 << 18)	// Endpoint Type : Interrupt
		| (0x1 << 15)	// Endpoint is Activated
		;
	
    USB->DAINTMSK |= 0
		|(1 << 2 )	//INEP2
		;
		
	USB_StatusStageIn();
	USB_Receive_DATA_EP1(cdc_ch);
#endif

	
#ifdef USB_HID
	USB_StatusStageIn();
	USB_Receive_HID_Report_EP1(hid_ch);
#endif
}

/**
* @brief      Get configuration requested by host
* @param      None
* @return     None
*/
void USB_GetConfiguration(){
	
	volatile uint16_t req, len;
	uint8_t desc;
	
	req = SWAP16(_setup_data_packet.wValue)&0xFF;
	len = _setup_data_packet.wLength & 0xffff;

	desc = (uint8_t)config_wValue;
	
	USB_DataStageIN(len, &desc);
}

/**
* @brief      Get Status requested by host(Standard_Device_Request)
* @param      None
* @return     None
*/
void USB_GetStatus(){
	uint16_t len;
	uint8_t data[2];

	len = _setup_data_packet.wLength;

	data[0] = (uint8_t)(remote_wakeup<<1);
	data[1] = 0x00;
	USB_DataStageIN(len, data);

}

/**
* @brief      Response Standard_Device_Request 
* @param      None
* @return     None
*/
void USB_Standard_Device_Request(){
	uint16_t req;
	req = _setup_data_packet.bRequest;
	switch(req){
		case eUSB_REQ_GET_DESCRIPTOR: 
			USB_GetDescriptor();
        break;
    
		case eUSB_REQ_SET_ADDRESS:
			USB_SetAddress();
		break;
    
		case eUSB_REQ_SET_CONFIGURATION:
			USB_SetConfiguration();
		break;
    
		case eUSB_REQ_GET_CONFIGURATION: 
			USB_GetConfiguration();			
		break;
    
		case eUSB_REQ_GET_STATUS:
			USB_GetStatus();
		break;

		case eUSB_REQ_SET_FEATURE: 
			remote_wakeup = _setup_data_packet.wValue;
		break;
    
		case eUSB_REQ_CLEAR_FEATURE:       
			remote_wakeup = 0x00;
		break;
		
		default:  
		//Controll Error
		break;
  }
}

/**
* @brief      Response Standard_Interface_Request 
* @param      None
* @return     None
*/
void USB_Standard_Interface_Request(){
	uint16_t reqType, req, val, len;
	
	reqType = (_setup_data_packet.bmRequestType>>5) & 0x3;
	req = _setup_data_packet.bRequest;
	val = SWAP16(_setup_data_packet.wValue);
	len = _setup_data_packet.wLength;
	

	
	switch(reqType){
		case eUSB_REQ_TYPE_STANDARD:
			switch(req){
				case eUSB_REQ_GET_DESCRIPTOR:
					switch(val){
						case eUSB_DT_REPORT:
							__NOP();
#if defined(USB_HID_PIPE) || defined(USB_HID_PIPE_LS)
						len = 29;
						USB_DataStageIN(len, AccHIDReportDesc);
#endif

#ifdef USB_HID_KBD
						len = 63;
						USB_DataStageIN(len, AccHIDReportDesc);
#endif
							break;
					case eUSB_HID_GET_DESCRIPTOR:
#ifdef USB_HID
						USB_DataStageIN(len, hid_desc);
#endif
					break;
						
						default:
							break;
					}
					break;
				case eUSB_REQ_SET_INTERFACE:
					USB_DataStageIN(len, (uint8_t*) 0x00);
					break;
				case eUSB_REQ_GET_INTERFACE:
					USB_StatusStageIn();
					break;

				default:
					break;
			}
			break;
			
			case eUSB_REQ_TYPE_CLASS:

			switch(req){
#ifdef USB_MSC
				case eUSB_MSC_MASS_STORAGE_RESET:
					BulkStage = MSC_BS_CBW;
					USB_StatusStageIn();
					break;
				case eUSB_MSC_GET_MAX_LUN:
					//lun = 0x00;
					USB_DataStageIN(len, (uint8_t*) 0x00);
					break;
#endif
				
#ifdef USB_CDC
				case eUSB_CDC_SEND_ENCAPSULATED_COMMAND:
					break;
				case eUSB_CDC_GET_ENCAPSULATED_RESPONSE:
					break;
				case eUSB_CDC_SET_COMM_FEATURE:
					break;
				case eUSB_CDC_GET_COMM_FEATURE:
					break;
				case eUSB_CDC_CLEAR_COMM_FEATURE:
					break;
				case eUSB_CDC_SET_LINE_CODING:
					USB_DataStageOUT(len, (uint8_t*) &_cdc_line_coding);
					Process_line_coding();
					break;
				case eUSB_CDC_GET_LINE_CODING:
					USB_DataStageIN(len, (uint8_t*) &_cdc_line_coding);
					break;
				case eUSB_CDC_SET_CONTROL_LINE_STATE:
					break;
				case eUSB_CDC_SEND_BREAK:
					break;
#endif
				
#ifdef USB_HID
				case eUSB_HID_GET_PROTOCOL:
						USB_DataStageIN(len, &hid_protocol_val);
					break;
				case eUSB_HID_SET_PROTOCOL:
						hid_protocol_val = (uint8_t)(_setup_data_packet.wValue & 0xFF);
					break;
				case eUSB_HID_GET_IDLE:
						USB_DataStageIN(len, &hid_idle_val);
					break;
				case eUSB_HID_SET_IDLE:
						hid_idle_val  = val;
					break;
#endif
				default:
					break;
			}
					
			break;
	}
}

/**
* @brief      Response Standard_Endpoint_Request 
* @param      None
* @return     None
*/
void USB_Standard_EndPoint_Request(){
	uint16_t req, len;
	uint8_t data[2];
	static uint8_t chk = 0;
	
	req = _setup_data_packet.bRequest;
	len = _setup_data_packet.wLength;

	switch(req)
	{
			case eUSB_REQ_GET_STATUS:
				data[0] = chk;
				data[1] = 0x00;
				USB_DataStageIN(len, data);
				break;
			case eUSB_REQ_CLEAR_FEATURE:
				chk = 0x00;
				__NOP();
				break;
			case eUSB_REQ_SET_FEATURE:
				chk++;
				__NOP();
				break;
			default:
				break;
	}
		
}

/**
* @brief      Response USB Setup Stage(Device, Interface, Endpoint)
* @param      None
* @return     None
*/
void USB_SetupStage(){
	uint16_t req;
	req = _setup_data_packet.bmRequestType;
	USB_EP0_status = eUSB_EP0_SETUP;
	switch(req&0x1F){
		case eUSB_REQ_RECIPIENT_DEVICE:
			USB_Standard_Device_Request();
			break;
		case eUSB_REQ_RECIPIENT_INTERFACE:
			USB_Standard_Interface_Request();
			break;
		case eUSB_REQ_RECIPIENT_ENDPOINT:
			USB_Standard_EndPoint_Request();
			break;
		default:
			//Stall
			break;
		
	}
}

/**
* @brief      Response interrupt related to OUT Endpoint0(Control Transfer) 
* @param      None
* @return     None
*/
void USB_OEPInt_DAINT_OutEPInt0_Combination(){
    volatile unsigned int    usb_outepint0;
    volatile unsigned int    DESC;
    volatile unsigned int    IOC, SPD, SI;
	
	
    IOC = 0;
    SPD = 0;
	SI = 0;
	
    //IN Setup Stage
    usb_outepint0 = USB->DOEPINT0;

    IOC = (usb_outepint0 >> 0) & 0x1;
    SPD = (usb_outepint0 >> 3) & 0x1;
	SI = (usb_outepint0 >> 5) & 0x1;

    //CASE A
    if(SI == 0 && SPD == 0 && IOC == 1)
		{
				USB->DOEPINT0 = (0x1 << 0);
        DESC = USB_DOEP0_DMA_DESC_RX[0];
			
				if(USB_EP0_status == eUSB_EP0_STALL)
				{
						USB->DIEPCTL0 &= ~((0x1<<21) //Stall
																				|(0x1 << 27));	// NAK
				}			
				
				//SR
				if(((DESC >> 24) & 0x01) == 1)
				{					
					USB_SetupStage();
				}
				else
				{
						if(USB_EP0_status == eUSB_EP0_SETUP){
							//Status IN Stage
							USB_StatusStageIn();
							USB_prepareSetupPacket();
							
						}
						else
						{
							//Status IN Stage
							USB_StatusStageIn();
							USB_prepareSetupPacket();
//							//Data OUT Stage
//							USB->DOEPCTL0 = 0
//								| (0x3   <<  0) //64Byte
//								| (0x1UL << 31) //EPEna
//								;
						}
				}
				
    //CASE B
			}
			else if(SI == 0 && SPD == 1 && IOC == 0)
			{
					USB->DOEPINT0 = (0x1 << 3);
					DESC = USB_DOEP0_DMA_DESC_RX[0];
								//SR
				
					if(USB_EP0_status == eUSB_EP0_DATA_IN)
					{
						// Data IN & Status OUT Stage
						USB->DIEPCTL0 |= (0x1<<26); //CNAK
						USB->DOEPCTL0 |= (0x1<<26); //CNAK
						
						USB_prepareSetupPacket();	
					}
					else if(USB_EP0_status == eUSB_EP0_DATA_OUT)
					{
						// Data Out Stage
						USB->DIEPCTL0 |= (0x1<<26); //CNAK
						USB->DOEPCTL0 |= (0x1<<26); //CNAK

						USB->DOEPCTL0 = 0
							| (0x3   <<  0) //64Byte
							| (0x1UL << 31) //EPEna
							;
						USB_EP0_status = eUSB_EP0_STATUS_IN;
						
					}
					else
					{
							// L : last descriptor
							DESC = USB_DOEP0_DMA_DESC_RX[0];

							USB->DIEPCTL0 |= (0x1<<26); //CNAK
							USB->DOEPCTL0 |= (0x1<<26); //CNAK

							if((DESC >> 27) & 0x1)
							{
								// Status IN Stage
								if(USB_EP0_status == eUSB_EP0_STALL)
								{
										USB->DIEPCTL0 |= (0x1<<21) //Stall
																						| (0x1 << 27);	// NAK
										//
										USB_prepareSetupPacket();
										USB_EP0_status = eUSB_EP0_STALL;
								}
								else
								{
									USB_StatusStageIn(); 
									USB_prepareSetupPacket();
								}
								
							}
							else
							{
								// Data Out Stage
								USB->DOEPCTL0 = 0
									| (0x3   <<  0) //64Byte
									| (0x1UL << 31) //EPEna
									;
							}
					}
		
		
    //CASE C
    }
		else if(SI == 0 && SPD == 1 && IOC == 1)
		{
        USB->DOEPINT0 = (0x1 << 3) | (0x1 << 0);
		
        DESC = USB_DOEP0_DMA_DESC_RX[0];
        //Prepare IN DATA and Status Stage
        
				//SR
				if(((DESC >> 24) & 0x01) == 1)
				{
					USB_SetupStage();
				}
				USB->DOEPCTL0 |= (0x1<<26); //CNAK
				USB->DIEPCTL0 |= (0x1<<26); //CNAK
				
	//CASE D
    }
		else if(SI == 1 && SPD == 0 && IOC == 0)
		{
				USB->DOEPINT0 = (0x1 << 5);
				USB->DIEPCTL0 |= (0x1<<26); //CNAK
		//CASE E
    } 
		else if(SI == 1 && SPD == 0 && IOC == 1)
		{
				DESC = USB_DOEP0_DMA_DESC_RX[0];
				//Data IN Stage 
				USB->DOEPINT0 = (0x1 << 5) | (0x01 << 0);
				USB->DIEPCTL0 |= (0x1<<26); //CNAK

				if((DESC >> 27) & 0x1){
					USB_StatusStageIn(); 
					USB_prepareSetupPacket();
				}

		}

}


/**
* @brief      Process OUT Endpoint0 interrupt
* @param      None
* @return     None
*/
void USB_OEPInt_DAINT_OutEPInt0(){
    volatile uint32_t usb_outepint0;
    usb_outepint0 = USB->DOEPINT0;

	if (((usb_outepint0 & (1 <<  5))) || ((usb_outepint0 & (1 <<  3))) || (usb_outepint0 & (1 <<  0))){
        USB_OEPInt_DAINT_OutEPInt0_Combination();
    }
}

/**
* @brief      Process OUT Endpoint1 interrupt
* @param      None
* @return     None
*/
void USB_OEPInt_DAINT_OutEPInt1(){
  volatile uint32_t usb_outepint1;
    usb_outepint1 = USB->DOEPINT1;

	if (usb_outepint1 & (1 << 0))
	{ // XferCompl
			USB->DOEPINT1 = (1 << 0);
#ifdef USB_MSC
			MSC_BulkOut();
#endif
	
#ifdef USB_CDC
		rx_USB_UART(cdc_ch, Get_rx_length());
		USB_Receive_DATA_EP1(cdc_ch);
#endif
		
#if defined(USB_HID_PIPE) || defined(USB_HID_PIPE_LS)
			//TODO:source CODE
		USB_Receive_HID_Report_EP1(hid_ch);
//		_DBH(ch[0]);
//		_DBH(ch[1]);
//		_DBH(ch[2]);
		USB_Send_HID_Report_EP1(hid_ch);
#endif

#ifdef USB_HID_KBD
		
		USB_Receive_HID_Report_EP1(hid_ch);
//			_DBH(ch[0]);
//			_DBH(ch[1]);
//			_DBH(ch[2]);
//			_DBH(ch[3]);
#endif
	}

}

/**
* @brief      Process IN Endpoint0 interrupt
* @param      None
* @return     None
*/
void USB_IEPInt_DAINT_InEPInt0(){
    volatile uint32_t usb_inepint0;

    usb_inepint0 = USB->DIEPINT0;

    //XferCompl
    if(usb_inepint0 & (1 << 0 )){ 
        USB->DIEPINT0 = (0x1 << 0);
		
			if(USB_EP0_status == eUSB_EP0_STATUS_IN)
			{
				USB_prepareSetupPacket();
			}

    }
}

/**
* @brief      Process IN Endpoint1 interrupt
* @param      None
* @return     None
*/
void USB_IEPInt_DAINT_InEPInt1(){
	volatile uint32_t usb_inepint1;
	usb_inepint1 = USB->DIEPINT1;

    //XferCompl
	if(usb_inepint1 & (1<<0))
	{
		USB->DIEPINT1 = (0x1 << 0);
#ifdef USB_MSC
		MSC_BulkIn();
#endif
	}

}

/**
* @brief      Handle USB_IRQHandler
* @param      None
* @return     None
*/
void USB_IRQHandler(){
	volatile uint32_t usb_global_int_sts;
	usb_global_int_sts = USB->GINTSTS;
		
	//OEPInt
	if(usb_global_int_sts & (1 << 19)){ 
		USB_GINTSTS_OEPInt();
		USB->GINTSTS |= (1 << 19);
	}
	
	//IEPInt
	if(usb_global_int_sts & (1 << 18)){ 
		USB_GINTSTS_IEPInt();
		USB->GINTSTS|= (1 << 18);
	}
	
	//EnumDone
	if(usb_global_int_sts & (1 << 13)){ 
		USB_GINTSTS_EnumDone();
		USB->GINTSTS |= (1 << 13);
	}
	
	//USBRst
	if(usb_global_int_sts & (1 << 12)){ 
		USB_GINTSTS_USBRst();
		USB->GINTSTS |= (1 << 12);
	}
	
	//USBSusp
	if(usb_global_int_sts & (1 << 11)){ 
		USB_GINTSTS_Suspend();
		USB->GINTSTS |= (1 << 11);
	}
	
	//ErlySusp
	if(usb_global_int_sts & (1 << 10)){ 
		USB->GINTSTS |= (1 << 10);
	}
	
	//Sof
	if(usb_global_int_sts & (1 <<  3)){ 
		USB_GINTSTS_SOF();
		USB->GINTSTS |= (1 << 3);
	}
}
