/**
* @file		A31G32x_hal_wdt.c
* @brief	Contains all functions support for wdt firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_wt.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t _WTCLK;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/**
* @brief 		Initial for Watch Timer function
* @param		wtCfg WT_CFG_Type
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_Init(WT_CFG_Type wtCfg)
{
	/* Check the IWDG handle allocation */


	SCU->PER1 &= ~(1UL<<31); // WT 31 bit
	SCU->PCER1 &= ~(1UL<<31);	

	SCU->PER1 |= (1UL<<31);
	SCU->PCER1 |= (1UL<<31);	
	

	WT->CR = 0
		|((wtCfg.wtClkDiv&0x03)<<4) // WTINTV[1:0]
		|WTCR_WTCLR
		;
	WT->DR = (wtCfg.wtTmrConst & 0xFFF);
	
	return HAL_OK;
}

/**
* @brief 		deinit for Watch Timer function
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_DeInit(void)
{
	WT->CR = 0;
	return HAL_OK;
}

/**
* @brief 		Enable WT activity
* @param		FunctionalState 	ctrl 
*					- DISABLE : wt enable
*					- ENABLE  : wt disable 
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_Start(FunctionalState ctrl)
{
	uint32_t tmp_reg;
	
	tmp_reg = WT->CR&0xFF;
	tmp_reg &= ~(0x1<<7); // Disable watch-dog timer counter
	
	if (ctrl == ENABLE){
		tmp_reg |= (0x1<<7); // Enable watch timer counter
	}
	
	tmp_reg |= WTCR_WTCLR; // clear the counter and divider

	WT->CR = tmp_reg; 
	return HAL_OK;
}

/**
* @brief 		Stop WT activity
* @param		Not Support
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_Stop(void)
{
	return HAL_OK;
}

/**
* @brief		Get the current value of WT
* @param		None
* @return		current value of WT
*/
uint32_t HAL_WT_GetCurrentCount(void)
{
	return (WT->CNT & 0xFFF);
}

/**
* @brief 		Get the timer status register of WT
* @param		None
* @return		the status register of WT
*/
uint32_t HAL_WT_GetStatus(void)
{
	return (WT->CR & WT_STATUS_BITMASK);
}

/**
* @brief		Clear the timer status register of WT
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_ClearStatus(void)
{
	WT->CR |= WTCR_WTIFLAG; //(1<<1) : clear for WTIFLAG
	return HAL_OK;
}

/**
* @brief 		Enable or disable WT interrupt.
* @param		NewState New state of specified WT interrupt type,
* 				should be:
* 					- ENALBE	: Enable this interrupt type.
* 					- DISALBE	: Disable this interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_WT_ConfigInterrupt(FunctionalState NewState)
{
	uint32_t reg_val=0;
	
	reg_val =(WT->CR &0xFF);
	
	if (NewState == ENABLE) {
		reg_val |= WTCR_WTIEN; // WTIEN bit
	}
	else	{
		reg_val &= ~WTCR_WTIEN;
	}
	
	WT->CR = reg_val | WTCR_WTIFLAG; //(1<<1) : clear for WTIFLAG
	return HAL_OK;
}

/* --------------------------------- End Of File ------------------------------ */
