/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void ADC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
__IO	uint32_t i;
__IO	uint32_t count;

ADC_CFG_Type ADC_config;

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using ADC in Burst mode \n\r"
"************************************************\n\r";



/**********************************************************************
 * @brief		ADC_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);
	
	if(status & ADC_STAT_TRIGGER)	//	TRIGGER
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_TRIGGER);
	}
	
	if(status & ADC_STAT_SEQ)		//	EOS
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SEQ);
		count=1;
	}
	
	if(status & ADC_STAT_SINGLE)		//	EOC
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SINGLE);
	}
	
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);		// ADC Channel AN0
	HAL_GPIO_ConfigFunction(PA, 0, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
}

/**********************************************************************
 * @brief		TIMER40_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure( void)
{
	TIMER40_PERIODICCFG_Type timer;
	
	timer.TnADR = 9500;
	timer.TnBDR = 0;
	timer.Prescaler = 479;
	timer.StartLevel = START_HIGH;
	timer.CkSel = T40_CLKINT;
	timer.ECE = T40_FEDGE;
	
	HAL_SCU_Timer40_ClockConfig(T40CLK_PCLK);
	
	HAL_TIMER4n_Init(TIMER40, PERIODIC_MODE, (TIMER40_PERIODICCFG_Type *) &timer);
	
	/*TIMER40 Enable */
	HAL_TIMER4n_Cmd(TIMER40, ENABLE);
}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG
	_DBG("DR0 = "); _DBH16(ADC->DR0>>4); _DBG(" ");
	_DBG("DR1 = "); _DBH16(ADC->DR1>>4); _DBG(" ");
	_DBG("DR2 = "); _DBH16(ADC->DR2>>4); _DBG(" ");
	_DBG("DR3 = "); _DBH16(ADC->DR3>>4); _DBG_(" ");	
	_DBG("DR4 = "); _DBH16(ADC->DR4>>4); _DBG(" ");
	_DBG("DR5 = "); _DBH16(ADC->DR5>>4); _DBG(" ");
	_DBG("DR6 = "); _DBH16(ADC->DR6>>4); _DBG(" ");	
	_DBG("DR7 = "); _DBH16(ADC->DR7>>4); _DBG(" ");	
	#endif
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	GPIO_Configure();
	
	ADC_config.Mode = ADC_BURST_MODE;
	ADC_config.SamplingTime=0x1f;
	ADC_config.SeqCnt = 8;
	ADC_config.RestartEn = 0;
	ADC_config.TrgSel = ADC_TRIGGER_TIMER;
	ADC_config.InClkDiv = 0x7f; 
	
	if (HAL_ADC_Init(ADC, &ADC_config) != HAL_OK)
	{
		/* Initialization Error */
      Error_Handler();
	}

	HAL_ADC_TriggerlSel(ADC, 0x7);	// ADC Trigger
	HAL_ADC_ChannelSel_1(ADC, 0x00000000);		
	HAL_ADC_ChannelSel_2(ADC, 0x00000000);
	
	HAL_ADC_ClearStatus(ADC, 0x0f); // before adc interrupt enable, you must clear adc status!! 
	HAL_ADC_ConfigInterrupt(ADC, ADC_INTEN_SEQ | ADC_INTEN_SINGLE, ENABLE);
	
	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);
}

/**********************************************************************
 * @brief		ADC_Burst_Loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Burst_Loop(void)
{
	HAL_ADC_Start(ADC);	// start

	while(count==0);
	count=1;
	 
	UART_DebugMessage();
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize ADC peripheral*/
	ADC_Configure(); 
	
	/*TIMER40 Init*/
	TIMER_Configure();

	 /*  Enable IRQ Interrupts */
	 __enable_irq();      
	 
	/* ADC Function Loop */
	ADC_Burst_Loop();
	
      /* Infinite Loop */
	while(1)
	{			
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
   /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	 
	/* Initialize Debug frame work through initializing USART port  */
  DEBUG_Init();

	/* Infinite loop */
	mainloop();   
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif



