/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define CRC16_BYPASS	0x8005	//	CRC-16
#define CRC16_CCITT	0x1021	//	CRC-CCITT

#define POLYNOM	CRC16_CCITT

#define WORD_LENGTH	0
#define BUF_LENGTH	1

#define CRC_MSB_FIRST	0
#define CRC_LSB_FIRST	1
#define NUM_OF_CRC_DUMMY	10
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void gen_crc_table(void);
unsigned short crc_gen(unsigned short crc_init, unsigned long crc_in, unsigned char ind1, unsigned char ind2);
void flash_crc_read (void);
void CRC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t	check1, check2;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"CRC CCITT DMA RAM\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
//uint32_t crc_dummy_data[NUM_OF_CRC_DUMMY] = {0x12345678, 0x9ABCDEF1, 0x23456789, 0xABCDEF12, 0x3456789A, 0xBCDEF123, 0x456789AB, 0xCDEF1234, 0x56789ABC, 0xDEF12345};
uint32_t crc_dummy_data[16] = {0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab, 0xabababab,};
static unsigned short crc_table[256];
unsigned short polynominal = POLYNOM;
unsigned long crc_start_adr;
unsigned long crc_end_adr;


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief	CRC_IRQHandler
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_IRQHandler_IT(void)
{
	uint32_t status;
	status = HAL_CRC_GetState_IT();
	if((status & CRC_FLAG) == CRC_FLAG)
	{
		HAL_CRC_ClearState_IT();
		__NOP();
	}
}


/**********************************************************************
 * @brief		gen_crc_table
 * @param[in]	None
 * @return	None
 **********************************************************************/
void gen_crc_table(void) {
	unsigned short i, j;
	unsigned short crc_accum;
	
	for( i = 0; i < 256; i++ ) {
		crc_accum = i<<8;
		for (j=0; j<8; j++) {
			if(crc_accum & 0x8000) crc_accum = (crc_accum<<1 ) ^ polynominal;
			else crc_accum <<= 1;
		}
		crc_table[i] = crc_accum;
	}		
}


/**********************************************************************
 * @brief		crc_gen
 * @param[in]	None
 * @return	None
 **********************************************************************/
unsigned short crc_gen(unsigned short crc_init, unsigned long crc_in, unsigned char ind1, unsigned char ind2) 
{
	unsigned char i;
	unsigned long tmp=0, *st_adr, *ed_adr;
	
	gen_crc_table();
	if(ind1 == WORD_LENGTH) 
	{
		if(ind2 != CRC_MSB_FIRST) 
		{
			for(i=0 ; i<32 ; i++) 
			{
				tmp <<= 1;
				if(crc_in & 0x00000001ul) tmp |= 0x00000001;
				crc_in >>= 1;
			}
			crc_in = tmp;
		}
		for(i=0 ; i<4 ; i++) crc_init = (crc_init<<8) ^ crc_table[((crc_init>>8) ^ (unsigned short)(crc_in>>(24-i*8)))&0x00FF];
	}
	else 
	{
		st_adr = (unsigned long *)crc_start_adr;
		ed_adr = (unsigned long *)crc_end_adr;
		while(st_adr <= ed_adr) 
		{
			crc_in = *st_adr++;
			if(ind2 != CRC_MSB_FIRST) 
			{
				for(i=0 ; i<32 ; i++)
				{
					tmp <<= 1;
					if(crc_in & 0x00000001ul) tmp |= 0x00000001;
					crc_in >>= 1;
				}
				crc_in = tmp;
			}
			for(i=0 ; i<4 ; i++) crc_init = (crc_init<<8) ^ crc_table[((crc_init>>8) ^ (unsigned short)(crc_in>>(24-i*8)))&0x00FF];
		}
	}
	return crc_init;
}

/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Configure(void)
{
	uint16_t 					ccitt_sum_init;		

	
	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}

	
	ccitt_sum_init = 0;			// CRC_INIT Value
	
	// CRC Calculation by CRC block
	if(HAL_CRC_SetUserInput(ccitt_sum_init) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();						
	}

	#ifdef CRC_INTERRUPT
	HAL_CRC_ConfigInterrupt(ENABLE);
	#endif
	
	HAL_CRC_ConfigUserMode(CRC_CRC, CRC_CCITT, CRC_MSB);
	
	#ifdef CRC_INTERRUPT
	NVIC_SetPriority(CRC_IRQn, 3);
    NVIC_EnableIRQ(CRC_IRQn);
	#endif


}

/**********************************************************************
 * @brief		DMAC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void DMAC_Configure(void)
{
	DMAC_Ch_CFG_Type DMACU1Tx;

	if(HAL_DMAC_Init() != HAL_OK)
{
	/* Initialization Error */
	Error_Handler();				
}
	DMACU1Tx.TransferCnt = NUM_OF_CRC_DUMMY;
	DMACU1Tx.PeriSel = DMAC_PERISEL_CRC;
	DMACU1Tx.TransferWidth = DMAC_WIDTH_WORD;
	DMACU1Tx.Dir = DMAC_DIR_TX;	
	DMACU1Tx.PeriAddr = CRC_BASE+4;
	DMACU1Tx.MemAddr = (uint32_t)(&crc_dummy_data[0]);	
	HAL_DMAC_Setup(DMAC2, &DMACU1Tx);	
}
/**********************************************************************
 * @brief		CRC_CCITT_DMA_Loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_CCITT_DMA_Loop(void)
{
	uint16_t 					ccitt_sum_init;		
	uint16_t 					ccitt_sum;
	uint32_t 					i;

	//CRC Calculation by core
	ccitt_sum_init = 0;			// CRC_INIT Value
	ccitt_sum = ccitt_sum_init;
	
	for(i=0;i<NUM_OF_CRC_DUMMY;i++)
	{
		ccitt_sum = crc_gen(ccitt_sum, crc_dummy_data[i], WORD_LENGTH, CRC_MSB_FIRST);
	}	
	
	check1 = ccitt_sum;
	

	CRCRun();
	
	HAL_DMAC_ChCmd(DMAC2, ENABLE);
	
	while (HAL_DMAC_GetState(DMAC2) == RESET) ;
	
	HAL_DMAC_ChCmd(DMAC2, DISABLE);
	
	ccitt_sum = HAL_CRC_GetData();
	
	CRCStop();
	
	check2 = ccitt_sum;
	
	_DBG("\r\n sw crc value = "); _DBH16(check1);
	_DBG("\r\n hw crc value = "); _DBH16(check2);
	
	if(check1 == check2) {
		_DBG("\r\n correct CRC!!\r\n");		
	} else {
		_DBG("\r\n incorrect CRC!!\r\n");		
	}
	
}
/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
  /*Initialize CRC peripheral*/
	CRC_Configure();

	DMAC_Configure();
	 /*Enable IRQ Interrupts */
   __enable_irq(); 
	
	CRC_CCITT_DMA_Loop();
	
	/* Infinite Loop */
	while(1)
	{
	}	

		
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 
  
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif



