/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
#define CRC16_BYPASS	0x8005		//	CRC-16
#define CRC16_CCITT	0x1021		//	CRC-CCITT

#define POLYNOM	CRC16_BYPASS

#define WORD_LENGTH	0
#define BUF_LENGTH	1

#define CRC_MSB_FIRST	0
#define CRC_LSB_FIRST	1

/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void gen_crc_table(void);
unsigned short crc_gen(unsigned short crc_init, unsigned long crc_in, unsigned char ind1, unsigned char ind2);
void CRC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t check1, check2;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"CRC CRC16 \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
// Internal Variables Definition
static unsigned short crc_table[256];
unsigned short polynominal = POLYNOM;
unsigned long crc_start_adr;
unsigned long crc_end_adr;


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		gen_crc_table
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void gen_crc_table(void)
{
	unsigned short i, j;
	unsigned short crc_accum;
	
	for( i = 0; i < 256; i++ ) {
		crc_accum = i<<8;
		for (j=0; j<8; j++) {
			if(crc_accum & 0x8000) crc_accum = (crc_accum<<1 ) ^ polynominal;
			else crc_accum <<= 1;
		}
		crc_table[i] = crc_accum;
	}
}

/**********************************************************************
 * @brief		crc_gen
 * @param[in]	None
 * @return 		None
 **********************************************************************/
unsigned short crc_gen(unsigned short crc_init, unsigned long crc_in, unsigned char ind1, unsigned char ind2) 
{
	unsigned char i;
	unsigned long tmp=0, *st_adr, *ed_adr;
	
	gen_crc_table();
	if(ind1 == WORD_LENGTH) {
		if(ind2 != CRC_MSB_FIRST) {
			for(i=0 ; i<32 ; i++) {
				tmp <<= 1;
				if(crc_in & 0x00000001ul) tmp |= 0x00000001;
				crc_in >>= 1;
			}
			crc_in = tmp;
		}
		for(i=0 ; i<4 ; i++) crc_init = (crc_init<<8) ^ crc_table[((crc_init>>8) ^ (unsigned short)(crc_in>>(24-i*8)))&0x00FF];
	}
	else {
		st_adr = (unsigned long *)crc_start_adr;
		ed_adr = (unsigned long *)crc_end_adr;
		while(st_adr <= ed_adr) {
			crc_in = *st_adr++;
			if(ind2 != CRC_MSB_FIRST) {
				for(i=0 ; i<32 ; i++) {
					tmp <<= 1;
					if(crc_in & 0x00000001ul) tmp |= 0x00000001;
					crc_in >>= 1;
				}
				crc_in = tmp;
			}
			for(i=0 ; i<4 ; i++) crc_init = (crc_init<<8) ^ crc_table[((crc_init>>8) ^ (unsigned short)(crc_in>>(24-i*8)))&0x00FF];
		}
	}
	return crc_init;
}



/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Configure(void)
{
	uint16_t 					crc16_sum_init;	
	
	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	
	crc16_sum_init = 0;			// CRC_INIT Value
	// CRC Calculation by CRC Block
	HAL_CRC_SetUserInput(crc16_sum_init);

	HAL_CRC_ConfigUserMode(CRC_CRC, CRC_16, CRC_MSB);
	
}

/**********************************************************************
 * @brief		CRC_CRC16_Loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_CRC16_Loop(void)
{
	uint32_t 					crc16_in;
	uint16_t 					crc16_sum;
	uint16_t 					crc16_sum_init;	
	
	// CRC Calculation by Core
	crc16_sum_init = 0;			// CRC_INIT Value
	crc16_sum = crc16_sum_init;
	crc16_in = 0x98763254;				// CRC_IN Value
	crc16_sum = crc_gen(crc16_sum_init, crc16_in, WORD_LENGTH, CRC_MSB_FIRST);
	
	check1 = crc16_sum;
	
	CRCRun();
	HAL_CRC_UserInput(crc16_in);
	crc16_sum =  HAL_CRC_GetData();
	CRCStop();
	
	check2 = crc16_sum;
	
	
	_DBG("\r\n sw crc value = "); _DBH16(check1);
	_DBG("\r\n hw crc value = "); _DBH16(check2);
	
	if(check1 == check2) {
		_DBG("\r\n correct CRC!!\r\n");		
	} else {
		_DBG("\r\n incorrect CRC!!\r\n");		
	}
	
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{	
	
	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
  
	/*Initialize CRC peripheral*/
	CRC_Configure();

	
	CRC_CRC16_Loop();
	
	/* Infinite Loop */
	while(1)
	{
	}
		
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

