/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define NUM_OF_CRC_DUMMY	10
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void CRC_Configure(void);
void DMAC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"CRC Checksum DMA RAM \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
uint32_t crc_dummy_data[NUM_OF_CRC_DUMMY] = {0x12345678, 0x9ABCDEF1, 0x23456789, 0xABCDEF12, 0x3456789A, 0xBCDEF123, 0x456789AB, 0xCDEF1234, 0x56789ABC, 0xDEF12345};


/**********************************************************************
 * @brief		CRC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void CRC_IRQHandler_IT(void){
	uint32_t status;
	status = HAL_CRC_GetState_IT();
	if((status & CRC_FLAG) == CRC_FLAG){
		HAL_CRC_ClearState_IT();
		__NOP();
	}
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Configure(void)
{
	uint16_t	chk_sum_init;	

	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}

	chk_sum_init = 0;			// CRC_INIT Value
	// Checksum calculation by CRC Block
	HAL_CRC_SetUserInput(chk_sum_init);
	
//#ifdef CRC_INTERRUPT
	HAL_CRC_ConfigInterrupt(ENABLE);
//#endif
	
	HAL_CRC_ConfigUserMode(CRC_CHECKSUM, CRC_CCITT, CRC_MSB);
	
//#ifdef CRC_INTERRUPT
		NVIC_SetPriority(CRC_IRQn, 3);
    NVIC_EnableIRQ(CRC_IRQn);
//#endif

}


/**********************************************************************
 * @brief		DMAC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void DMAC_Configure(void)
{
	DMAC_Ch_CFG_Type DMACU1Tx;
	
	if(HAL_DMAC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	DMACU1Tx.TransferCnt = NUM_OF_CRC_DUMMY;
	DMACU1Tx.PeriSel = DMAC_PERISEL_CRC;
	DMACU1Tx.TransferWidth = DMAC_WIDTH_WORD;
	DMACU1Tx.Dir = DMAC_DIR_TX;	
	DMACU1Tx.PeriAddr = CRC_BASE+4;
	DMACU1Tx.MemAddr = (uint32_t)(&crc_dummy_data[0]);	
	HAL_DMAC_Setup(DMAC3, &DMACU1Tx);	
	
}

/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Checksum_DMA_Loop(void)
{
	uint32_t	i, j;
	uint32_t	chk1, chk2;
	uint16_t	chk_sum;
	uint16_t	chk_sum_init;	
	
	// Checksum calculation by Core
	chk_sum_init = 0;			// CRC_INIT Value
	chk_sum = chk_sum_init;
	for(i=0;i<NUM_OF_CRC_DUMMY;i++)
	{
		for (j=0; j<4; j++) {
			chk_sum += (uint16_t) (crc_dummy_data[i] >> (24-j*8)) & 0x00FF;
		}
	}
	
	chk1 = chk_sum;

	
	CRCRun();
	
	HAL_DMAC_ChCmd(DMAC3, ENABLE);
	
	while (HAL_DMAC_GetState(DMAC3) == RESET) ;

	HAL_DMAC_ChCmd(DMAC3, DISABLE);
	
	chk_sum = HAL_CRC_GetData();
	CRCStop();

	chk2 = chk_sum;
	
	_DBG("\r\n sw crc value = "); _DBH16(chk1);
	_DBG("\r\n hw crc value = "); _DBH16(chk2);
	
	if(chk1 == chk2) {
		_DBG("\r\n correct CRC!!\r\n");		
	} else {
		_DBG("\r\n incorrect CRC!!\r\n");		
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
  
	/*Initialize CRC peripheral*/
	CRC_Configure(); 
	
	/*Initialize DMAC peripheral*/
	DMAC_Configure();
  
 	 /*Enable IRQ Interrupts */
   __enable_irq(); 
	
	CRC_Checksum_DMA_Loop();
	
	/* Infinite Loop */
	while(1)
	{
	}
		
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


