/**
* @file		Debug_usartn.c
* @brief	Contains all functions support for usart firmware library on A31G324
* @author	ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "Debug_usartn.h"
#include "A31G32x_hal_scu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t UsartBaseClock; 
USARTBUFFER	USART_Buffer[4];

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*********************************************************************
 * @brief		Uart of Usart get buffer
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		USARTBUFFER
 **********************************************************************/
USARTBUFFER *usart_get_buffer(USART_Type *USART1n)
{
	USARTBUFFER	*ub;

	if (USART1n == USART10) {
		ub = &USART_Buffer[0];
	} else if (USART1n == USART11) {
		ub = &USART_Buffer[1];
	} else if (USART1n == USART12) {
		ub = &USART_Buffer[2];
	} else if (USART1n == USART13) {
		ub = &USART_Buffer[3];
	} else {
		ub = NULL;
	}

	return ub;
}

/*********************************************************************
 * @brief		Uart of Usart set buffer
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		None
 **********************************************************************/
void	usart_set_buffer(USART_Type *USART1n, uint8_t *buffer, uint32_t rxqsize, uint32_t txqsize)
{
	USARTBUFFER	*ub;

	ub = usart_get_buffer(USART1n);

	if (ub != NULL) {
		ub->status = 0;
		ub->rxhead = ub->rxtail = ub->rxsize = 0;
		ub->txhead = ub->txtail = ub->txsize = 0;

		if (buffer == NULL) {
			ub->buffer = NULL;
			ub->rxqsize = 0;
			ub->txqsize = 0;
		} else {
			ub->buffer = buffer;
			ub->rxqsize = rxqsize;
			ub->txqsize = txqsize;
		}
	}
}

/*********************************************************************
 * @brief		UART Handler routine
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART0	:UART0 peripheral
 * 					- UART1	:UART1 peripheral
 * 				ub		UART buffer
 * @return 		None
 **********************************************************************/
void	USART_Handler(USART_Type *USARTn)
{
	int			ch;
	uint32_t	sr;
	USARTBUFFER	*ub;

	ub = usart_get_buffer(USARTn);
	sr = USARTn->ST;

	if (sr & UST_SR_RXC) {
		ch = USARTn->DR;
		if (ub->rxsize < ub->rxqsize) {
			ub->rxsize++;
			ub->buffer[ub->rxtail] = ch;
			if (++ub->rxtail >= ub->rxqsize)
				ub->rxtail = 0;
		}
		//USARTn->USARTn_ST_b.RXCn = 1;
	}
	if (sr & UST_SR_DRE) {
		if (ub->txsize == 0) {
			USARTn->CR1 &= ~(1<<5);	// = 0uawrite(port,IA_IER,uaread(port,IA_IER)&~IER_TIE);
		} else if (ub->txqsize > 0) {
			ch = ub->buffer[ub->rxqsize + ub->txhead];
			if (++ub->txhead >= ub->txqsize)
				ub->txhead = 0;
			USARTn->DR = ch;	//uawrite(port,IA_THR,ch);
			ub->txsize--;
		}
	}
	if (sr & UST_SR_DOR) {
		ub->status |= UST_SR_DOR;		//Read-Only
	}
	if (sr & UST_SR_FE) {
		ub->status |= UST_SR_FE;
		USARTn->ST |= (1<<1);
	}
	if (sr & UST_SR_PE) {
		ub->status |= UST_SR_PE;
		USARTn->ST|= (1<<0);
	}
}


/**
* @brief		Determines best dividers to get a target clock rate
* @param		USTx	Pointer to selected USART peripheral, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		baudrate Desired USART baud rate.
* @return		None
*/
void usart_set_divisors(USART_Type *USTx, uint32_t baudrate)
{
	uint32_t numerator; 
	uint32_t denominator; 
  uint32_t fpc;
  float bdr;
	//------------------------------------------------------------------------------
	// numerator & denominator 
	// 
	// bdr = (UsartBaseClock) / n / baudrate  - 1
	//------------------------------------------------------------------------------
	numerator = UsartBaseClock; 
	denominator = baudrate; 

	bdr = ((float)numerator / (float)denominator) - 1; 
	USTx->BDR = ((uint16_t)bdr)&0xffff;
  fpc = (uint32_t)((bdr - (uint16_t)bdr) * 256);
  USTx->FPCR = fpc;
}

/*******************************************************************************
* Public Function
*******************************************************************************/
/**
* @brief		Initializes the USART1x peripheral according to the specified
*				parameters in the USART_ConfigStruct.
* @param		USTx	USART peripheral selected, should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		USART_ConfigStruct Pointer to a USART_CFG_Type structure
*				that contains the configuration information for the
*				specified USART peripheral.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_Init(USART_Type *USTx, USART_CFG_Type *USART_ConfigStruct)
{
	uint32_t tmp;

	if(USTx == USART10)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART10_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART10_Msk);		
		/* Set up peripheral clock for USART10 module */
		SCU->PER2   |=  (SCU_PER2_USART10_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART10_Msk);		
	}
	else if(USTx == USART11)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART11_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART11_Msk);		
		/* Set up peripheral clock for USART11 module */
		SCU->PER2   |=  (SCU_PER2_USART11_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART11_Msk);		
	}
    else if(USTx == USART12)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART12_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART12_Msk);		
		/* Set up peripheral clock for USART12 module */
		SCU->PER2   |=  (SCU_PER2_USART12_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART12_Msk);	
	}
    else if(USTx == USART13)
	{
		SCU->PER2   &=  ~(SCU_PER2_USART13_Msk);
		SCU->PCER2  &=  ~(SCU_PCER2_USART13_Msk);		
		/* Set up peripheral clock for USART13 module */
		SCU->PER2   |=  (SCU_PER2_USART13_Msk);
		SCU->PCER2  |=  (SCU_PCER2_USART13_Msk);	
	}
    
    tmp = 0
        | ((USART_ConfigStruct->Mode & 0x3) << USART_CR1_USTnMS_Pos)
        | ((USART_ConfigStruct->Parity & 0x3) << USART_CR1_USTnP_Pos)   // only UART mode
        | ((USART_ConfigStruct->Databits & 0x7) << USART_CR1_USTnS_Pos)
        | ((USART_ConfigStruct->Order & 0x1) << USART_CR1_ORDn_Pos)     // only SPI mode
				| ((USART_ConfigStruct->ACK & 0x1) << USART_CR1_CPOLn_Pos)      // Sync. or SPI mode
				| ((USART_ConfigStruct->Edge & 0x1)  << USART_CR1_CPHAn_Pos)    // only SPI mode
        | (UST_CR1_TX_EN)
        | (UST_CR1_RX_EN); 
    
    USTx->CR1   =   tmp;
    
    if(USART_ConfigStruct->Mode == USART_SPI_MODE){     // Sync or SPI Master mode
        UsartBaseClock=SystemCoreClock >> 1;            // UsartBaseClock = SystemCoreClock / 2 
    }
    else {                                              // UART Mode Defalut
        UsartBaseClock=SystemCoreClock >> 4;            // UsartBaseClock = SystemCoreClock / 16
    }
    usart_set_divisors(USTx, USART_ConfigStruct->Baud_rate);
		
		tmp = 0
		| ( UST_CR2_FPC_EN )
		| ( UST_CR2_UST_EN )
		| ((USART_ConfigStruct->Stopbits & 0x1)<<UST_CR2_USTnSB_Pos);
    
    USTx->CR2   =   tmp;
//		USTx->CR2		|= 	(1<<7);
    
    //Dummy Read
    tmp = HAL_USART_ReceiveByte(USTx);
    tmp = HAL_USART_ReceiveByte(USTx);

	// add
    usart_set_buffer(USTx, USART_ConfigStruct->RxTxBuffer, USART_ConfigStruct->RxBufferSize, USART_ConfigStruct->TxBufferSize);
	//Dummy Read and flush buffer
	usflush(USTx);

	if (USART_ConfigStruct->RxTxBuffer != NULL) {
		//HAL_USART_ConfigInterrupt(USTx, USART_INTCFG_TXC, ENABLE);
		HAL_USART_ConfigInterrupt(USTx, USART_INTCFG_RXC, ENABLE);

		if (USTx == USART10) {
			NVIC_EnableIRQ(USART10_IRQn);
		//	EInt_MaskDisable(UST10_MASK);
		} else if (USTx == USART11) {
			NVIC_EnableIRQ(USART11_IRQn);
		//	EInt_MaskDisable(UST11_MASK);
		} else if (USTx == USART12) {
			NVIC_EnableIRQ(USART12_IRQn);
		//	EInt_MaskDisable(UST12_MASK);
		} else if (USTx == USART13) {
			NVIC_EnableIRQ(USART13_IRQn);
		//	EInt_MaskDisable(UST13_MASK);
		}
	}
    
    return HAL_OK;
}

/**
* @brief		Fills each USART_InitStruct member with its default value:
*				- 38400 bps
*				- 8-bit data
*				- 1 Stopbit
*				- None Parity
* @param		USART_InitStruct Pointer to a USART_CFG_Type structure which will be initialized.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_UART_Mode_Config(USART_CFG_Type *USART_InitStruct)
{
  USART_InitStruct->Mode = USART_UART_MODE;
	USART_InitStruct->Baud_rate = 38400;
	USART_InitStruct->Databits = USART_DATABIT_8;
	USART_InitStruct->Parity = USART_PARITY_NONE;
	USART_InitStruct->Stopbits = USART_STOPBIT_1;
	USART_InitStruct->RxTxBuffer = NULL;
	USART_InitStruct->RxBufferSize = 0;
	USART_InitStruct->TxBufferSize = 0;
	return HAL_OK;
}

/**
* @brief		Transmit a single data through USART peripheral
* @param		USTx	USART peripheral selected, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @param		Data	Data to transmit (must be 8-bit long)
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_TransmitByte(USART_Type* USTx, uint8_t Data)
{
	USTx->DR = Data;
	return HAL_OK;
}

/**
* @brief		Receive a single data from USART peripheral
* @param		USTx	USART peripheral selected, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @return		Data received
*/
uint8_t HAL_USART_ReceiveByte(USART_Type* USTx)
{
	return (USTx->DR);
}

/**
* @brief		Send a block of data via USART peripheral
* @param		USTx	Selected USART peripheral used to send data, should be:
*					- USART10	: USART10 peripheral
*					- USART11	: USART11 peripheral
*					- USART12	: USART12 peripheral
*					- USART13	: USART13 peripheral 
* @param		txbuf		Pointer to Transmit buffer
* @param		buflen	Length of Transmit buffer
* @return		Number of bytes sent.
*/
uint32_t HAL_USART_Transmit(USART_Type *USTx, uint8_t *txbuf, uint32_t buflen)
{
	uint32_t bToSend, bSent;
	uint8_t *pChar = txbuf;

	bToSend = buflen;
	bSent = 0;
	while (bToSend){          
		while(!(USTx->ST & UST_SR_DRE)){ }

		HAL_USART_TransmitByte(USTx, (*pChar));

		pChar++;
		bToSend--;
		bSent++;
	}
	
	return bSent;
}

/**
* @brief			Receive a block of data via USART peripheral
* @param		USTx	Selected USART peripheral used to send data,
* 				should be:
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		rxbuf	Pointer to Received buffer
* @param		buflen	Length of Received buffer
* @param		flag		Flag mode, should be:
*					- NONE_BLOCKING
* 					- BLOCKING
* @return		Number of bytes received
*/
uint32_t HAL_USART_Receive(USART_Type *USTx, uint8_t *rxbuf, uint32_t buflen)
{
	uint32_t bToRecv, bRecv;
	uint8_t *pChar = rxbuf;

	bToRecv = buflen;
	bRecv = 0;
	while (bToRecv) {
		while(!(USTx->ST & UST_SR_RXC)){ }
		
		(*pChar++) = HAL_USART_ReceiveByte(USTx);
		bRecv++;
		bToRecv--;
	}
	return bRecv;
}

/**
* @brief 		Enable or disable specified USART interrupt.
* @param		USTx	USART peripheral selected, should be
* 					- USART10	: USART10 peripheral
* 					- USART11	: USART11 peripheral
* 					- USART12	: USART12 peripheral
* 					- USART13	: USART13 peripheral 
* @param		USTIntCfg	Specifies the interrupt flag,
* 				should be one of the following:
* 					- USART_INTCFG_DR   : DR Interrupt enable
* 					- USART_INTCFG_TXC 	: TXC Interrupt enable
* 					- USART_INTCFG_RXC 	: RXC interrupt enable
* 					- USART_INTCFG_WAKE : WAKE Interrupt enable
* 					- USART_INTCFG_DMATX: DMA TX Interrupt enable
* 					- USART_INTCFG_DMARX: DMA RX Interrupt enable
* 					- USART_INTCFG_RTO  : RTO Interrupt enable
* @param		NewState New state of specified USART interrupt type,
* 				should be:
* 					- ENALBE	:Enable this USART interrupt type.
* 					- DISALBE	:Disable this USART interrupt type.
* @return		HAL_Status
*/
HAL_Status_Type HAL_USART_ConfigInterrupt(USART_Type *USTx, USART_INT_Type USTIntCfg, FunctionalState NewState)
{
	uint32_t tmp=0;
	
	switch(USTIntCfg){
		case USART_INTCFG_WAKE :
			tmp = UST_CR1_WAKEINT_EN;
			break;
		case USART_INTCFG_RXC:
			tmp = UST_CR1_RXCINT_EN;
			break;
		case USART_INTCFG_TXC:
			tmp = UST_CR1_TXCINT_EN;
			break;
		case USART_INTCFG_DR:
			tmp = UST_CR1_DRINT_EN;
			break;
        case USART_INTCFG_RTO:
			tmp = UST_CR2_RTOINT_EN;
			break;
        case USART_INTCFG_DMARX:
			tmp = UST_CR2_DMA_RXINT_EN;
			break;
        case USART_INTCFG_DMATX:
			tmp = UST_CR2_DMA_TXINT_EN;
			break;
        
	}
    
	if (NewState == ENABLE)
	{
		if(USTIntCfg < (USART_INTCFG_DR + 1 )) {
            USTx->CR1 |= tmp;
        }
        else if ( USTIntCfg <  (USART_INTCFG_DMATX + 1)) {
            USTx->CR2 |= tmp;
        }
	}
	else
	{
		if(USTIntCfg < (USART_INTCFG_DR + 1 )) {
            USTx->CR1 &= ~(tmp & UST_CR1_INT_BITMASK);
        }
        else if ( USTIntCfg <  (USART_INTCFG_DMATX + 1)) {
            USTx->CR1 &= ~(tmp & UST_CR2_INT_BITMASK);
        }
	}
	return HAL_OK;
}

/*********************************************************************
 * @brief		usflush
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		None
 **********************************************************************/
void	usflush(USART_Type* USARTn)
{
	uint32_t temp;
	USARTBUFFER	*ub;

	ub = usart_get_buffer(USARTn);

	temp = USARTn->DR;
	temp = USARTn->DR;

	if (ub != NULL) {
		ub->status=0;
		ub->rxhead=ub->rxtail=ub->rxsize=0;
		ub->txhead=ub->txtail=ub->txsize=0;
	}
}

/*********************************************************************
 * @brief		usputc
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		None
 **********************************************************************/
void usputc(USART_Type* USARTn, uint8_t Data)
{

	USARTBUFFER	*ub;

	ub = usart_get_buffer(USARTn);

	if (ub != NULL) {

		if (ub->txqsize > 0) {
			if (ub->txsize == 0 && (USARTn->ST & UST_SR_DRE) != 0) {
				USARTn->DR = Data;
				//USARTn->USARTn_ST_b.DREn = 1;
			}
			else {
				while (ub->txsize == ub->txqsize) ;
				__disable_irq();
				ub->buffer[ub->rxqsize + ub->txtail] = Data;
				if (++ub->txtail >= ub->txqsize)
					ub->txtail = 0;
				ub->txsize++;
				USARTn->CR1 |= (1<<5);	//uawrite(port,IA_IER,uaread(port,IA_IER)|IER_TIE);
				__enable_irq();
			}
		}
		else {
			while ((USARTn->ST & UST_SR_DRE) == 0);
			USARTn->DR = Data;
		}
	} else {
		while ((USARTn->ST & UST_SR_DRE) != 0);
		USARTn->DR = Data;
	}
}

/*********************************************************************
 * @brief		uswrite
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	uswrite(USART_Type *USART1n,void *p,int size)
{
	int32_t i;
	uint8_t	*dp = p;
	for (i = 0; i < size; i++) {
		usputc(USART1n, *dp++);
	}
	return i;
}

/*********************************************************************
 * @brief		usread
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	usread(USART_Type *USARTn,void *p,int size)
{
	int32_t i;
	uint8_t	*dp = p;
	for (i = 0; i < size; i++) {
		*dp++ = usgetc(USARTn);
	}
	return i;
}

/*********************************************************************
 * @brief		usputs
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t usputs(USART_Type *USARTn,char *s)
{
	int	i;
	int	ch;
	for (i = 0; ;i++) {
		ch = *s++;
		if (ch == 0)
			break;
		usputc(USARTn, ch);
	}
	return i;
}

/*********************************************************************
 * @brief		UART of USART SendByte
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		None
 **********************************************************************/
void UST_SendByte(USART_Type* USTx, uint8_t Data)
{
	usputc(USTx, Data);
}

/*********************************************************************
 * @brief		usgetq
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	usgetq(USARTBUFFER *ub)
{
	int	ch;

	if (ub->rxsize == 0)
		return -1;
	__disable_irq();
	ch = ub->buffer[ub->rxhead];
	if (++ub->rxhead>=ub->rxqsize)
		ub->rxhead=0;
	ub->rxsize--;
	__enable_irq();

	return ch;
}

/*********************************************************************
 * @brief		uscheck
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	uscheck(USART_Type* USARTn)
{
	USARTBUFFER *ub;

	ub = usart_get_buffer(USARTn);
	if (ub->buffer != NULL) {
		return (ub->rxsize);
	}
	return USARTn->ST & UST_SR_RXC;
}

/*********************************************************************
 * @brief		usgetc
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	usgetc(USART_Type* USARTn)
{
	USARTBUFFER *ub;

	ub = usart_get_buffer(USARTn);
	if (ub->buffer != NULL) {
		int	ch;
		while ((ch = usgetq(ub)) < 0);
		return ch;
	}

	while ((USARTn->ST & UST_SR_RXC) == 0);
	return USARTn->DR;
}

/*********************************************************************
 * @brief		usgetchar
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	usgetchar(USART_Type* USARTn)
{
	int	ch = -1;

	if (uscheck(USARTn))
		ch = usgetc(USARTn);

	return ch;
}

/*********************************************************************
 * @brief		usgets
 * @param[in]	UARTn	Pointer to selected UART peripheral, should be:
 * 					- UART10	:UART10 peripheral
 * 					- UART11	:UART11 peripheral
 *					- UART12	:UART12 peripheral
 *					- UART13	:UART13 peripheral
 * 				ub		UART buffer
 * @return 		Number of bytes received
 **********************************************************************/
int32_t	usgets(USART_Type* USARTn, char *strin, int32_t maxsize)
{
	int i;

	for (i = 0; i < maxsize-1; i++) {
		int32_t	ch;

		ch = usgetc(USARTn);
		if (ch == '\r')
			break;
		strin[i] = ch;
	}
	strin[i++] = 0;
	return i;
}


/* --------------------------------- End Of File ------------------------------ */
