/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "A31G32x_hal_pcu.h"
#include "A31G32x_hal_scu.h"
#include "EEPROM.h"
#include "A31G32x_hal_usartn.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define  READ		0x03          //  READ instruction  
#define  WRITE	0x02          //  WRITE instruction
#define  WRDI		0x04          //  Reset Write 
#define  WREN		0x06          //  WRITE ENABLE instruction  
#define  RDSR		0x05          //  READ STATUS register instruction 
#define  WRSR		0x01          //  WRITE STATUS REGISTER instruction  
#define  NOPROT	0x00          //  disable all write protections 
#define  PGSZ		0x10          //  size of PAGE = 16 bytes  
#define  MAXLEN	0x70          //  maximum length of a string = 128 bytes
#define DUMMY	0x00	//Dummy read

#define USARTx	USART11

/* Private function prototypes -----------------------------------------------*/
void Delay(uint32_t count);
uint8_t USART_SPI_Send_Receive_byte(uint8_t data);
void Write_Enable(void);
void Write_Disable(void);
uint8_t Read_State(void);
void EEPROM_Write_Byte(uint16_t addr, uint8_t data);
uint8_t EEPROM_Read_Byte(uint16_t addr);
void EEPROM_Erase_Byte(uint16_t addr);
void EEPROM_Write_Page(uint16_t addr, uint8_t* data);
void EEPROM_Read_Page(uint16_t addr, uint8_t* data);
void EEPROM_Erase_Page(uint16_t addr);
void EEPROM_Init(void);

/* Private variables ---------------------------------------------------------*/
uint8_t Receive_data[8]={0,};

/**********************************************************************
 * @brief		Delay
 * @param[in]	count
 * @return	None
 **********************************************************************/
void Delay(uint32_t count)
{
	uint8_t i,j;
	for(i=0;i<200;i++)
	{
		for(j=0;j<count;j++);
	}
}

/**********************************************************************
 * @brief		USART_SPI_Send_Receive_byte
 * @param[in]	data		This Parameter contains the value of Send Data
 * @return	recieve	The value of Recieve Data
 **********************************************************************/
uint8_t USART_SPI_Send_Receive_byte(uint8_t data)
{
	uint8_t recieve;
	while((HAL_USART_GetStatus(USARTx)&UST_SR_DRE) != UST_SR_DRE);
	HAL_USART_TransmitByte(USARTx, data);	
	while(!(HAL_USART_GetStatus(USARTx)&UST_SR_TXC));
	
	while((HAL_USART_GetStatus(USARTx)&UST_SR_RXC) != UST_SR_RXC);
	recieve=HAL_USART_ReceiveByte(USARTx);
	Delay(1);
	return recieve;
}

/**********************************************************************
 * @brief		Write_Enable
 * @param[in]	None
 * @return	None
 **********************************************************************/
void Write_Enable()
{
	CS_L;
	USART_SPI_Send_Receive_byte(WREN); // Manufactured and Device ID Read
	CS_H;
	Delay(10);
}

/**********************************************************************
 * @brief		Write_Disable
 * @param[in]	None
 * @return	None
 **********************************************************************/
void Write_Disable()
{
	CS_L;
	USART_SPI_Send_Receive_byte(WRDI); // Manufactured and Device ID Read
	CS_H;
	Delay(10);
}

/**********************************************************************
 * @brief		Read_State
 * @param[in]	None
 * @return	result	The value of EEPROM Read State
 **********************************************************************/
uint8_t Read_State()
{
	uint8_t result;
	
	CS_L;
	USART_SPI_Send_Receive_byte(RDSR); // Manufactured and Device ID Read
	result=USART_SPI_Send_Receive_byte(DUMMY);
	CS_H;	
	Delay(10);
	return result;
}

/**********************************************************************
 * @brief		EEPROM_Write_Byte
 * @param[in]	addr		This Parameter contains the value of Write Address
 * @param[in]	data		This Parameter contains the value of Wirte Data
 * @return	None
 **********************************************************************/
void EEPROM_Write_Byte(uint16_t addr, uint8_t data)
{
	Write_Enable();
	while((Read_State()&0x03)!=0x02);
	// write
	CS_L;
	USART_SPI_Send_Receive_byte(WRITE); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	USART_SPI_Send_Receive_byte(data);
	CS_H;
	Delay(100);
}

/**********************************************************************
 * @brief		EEPROM_Read_Byte
 * @param[in]	addr		This Parameter contains the value of Read Address
 * @return	result	The value of EEPROM Read Data
 **********************************************************************/
uint8_t EEPROM_Read_Byte(uint16_t addr)
{
	uint8_t result;
	// read
	CS_L;
	USART_SPI_Send_Receive_byte(READ); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	result=USART_SPI_Send_Receive_byte(0x00);
	CS_H;
	Delay(50);
	return result;
}

/**********************************************************************
 * @brief		EEPROM_Erase_Byte
 * @param[in]	addr		This Parameter contains the value of Erase Address
 * @return	None
 **********************************************************************/
void EEPROM_Erase_Byte(uint16_t addr)
{
	uint8_t i;
	Write_Enable();
	
	// write
	CS_L;
	USART_SPI_Send_Receive_byte(WRITE); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	USART_SPI_Send_Receive_byte(0xFF);
	CS_H;
	Delay(100);
}

/**********************************************************************
 * @brief		EEPROM_Write_Page
 * @param[in]	addr		This Parameter contains the value of Write Address
 * @param[in]	data		This Parameter contains the value of Wirte Data
 * @return	None
 **********************************************************************/
void EEPROM_Write_Page(uint16_t addr, uint8_t* data)
{
	uint8_t i;
	Write_Enable();
	while((Read_State()&0x03)!=0x02);
	// write
	CS_L;
	USART_SPI_Send_Receive_byte(WRITE); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	for(i=0; i<PAGE_SIZE; i++)
	{
		USART_SPI_Send_Receive_byte(*data);
		data++;
	}
	CS_H;
	Delay(100);
}

/**********************************************************************
 * @brief		EEPROM_Read_Page
 * @param[in]	addr		This Parameter contains the value of Read Address
 * @return	None
 **********************************************************************/
void EEPROM_Read_Page(uint16_t addr, uint8_t* data)
{
	uint8_t i;
	// read
	CS_L;
	USART_SPI_Send_Receive_byte(READ); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	for(i=0; i<PAGE_SIZE; i++)
	{
		data[i]=USART_SPI_Send_Receive_byte(DUMMY);
	}
	CS_H;
	Delay(50);
}

/**********************************************************************
 * @brief		EEPROM_Erase_Page
 * @param[in]	addr		This Parameter contains the value of Erase Address
 * @return	None
 **********************************************************************/
void EEPROM_Erase_Page(uint16_t addr)
{
	uint8_t i;
	Write_Enable();
	
	// write
	CS_L;
	USART_SPI_Send_Receive_byte(WRITE); // Manufactured and Device ID Read
	USART_SPI_Send_Receive_byte((addr & 0xFF00)>>8);
	USART_SPI_Send_Receive_byte(addr & 0xFF);
	for(i=0; i<PAGE_SIZE; i++)
	{
		USART_SPI_Send_Receive_byte(0xFF);
	}
	CS_H;
	Delay(100);
}

/**********************************************************************
 * @brief		EEPROM_Init
 * @param[in]	None
 * @return	None
 **********************************************************************/
void EEPROM_Init()
{
	Write_Enable();	
	while((Read_State()&0x03)!=0x02);
}
