/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SPI_Configure(void);
void SPI_to_EEPROM(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint8_t result;
uint8_t write_buf[PAGE_SIZE], read_buf[PAGE_SIZE], i;
USART_CFG_Type USARTConfigStruct;

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using SPI Polling to EEPROM \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Initialize USART11
	
	// MOSI11 (PA2)	 
	HAL_GPIO_ConfigOutput(PA, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 2, FUNC2);
	
	// MISO11 (PA3)	
	HAL_GPIO_ConfigOutput(PA, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 3, FUNC2);
	HAL_GPIO_ConfigPullup(PA, 3, ENPU);
	
	//SCK11 (PA4) 
	HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 4, FUNC2);

	//SS11 (PA1)   : master control as GPIO 
	HAL_GPIO_ConfigOutput(PA, 1, PUSH_PULL_OUTPUT);
}

/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_Configure(void)
{
	// USART Init
	USARTConfigStruct.Mode = USART_SPI_MODE;
	USARTConfigStruct.Baud_rate = 38400;
	USARTConfigStruct.Databits = USART_DATABIT_8;
	USARTConfigStruct.Order = USART_SPI_MSB;
	USARTConfigStruct.ACK = USART_SPI_TX_FALLING;
	USARTConfigStruct.Edge = USART_SPI_TX_LEADEDGE_SETUP;

	// USART11 initialize
	if(HAL_USART_Init(USART11, &USARTConfigStruct) != HAL_OK)
	{
		Error_Handler();
	}

	// SPI Master
	HAL_USART_DataControlConfig(USART11, USART_CONTROL_MASTER, ENABLE);
	CS_H;
}

/**********************************************************************
 * @brief		SPI_to_EEPROM
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_to_EEPROM(void)
{ 
	_DBG_("USART SPI Write to EEPORM");
	EEPROM_Init();
	_DBG_("EEPROM Write READY");
	
	// EEPROM Write Byte
	_DBG_("EEPROM Write Byte");
	EEPROM_Write_Byte(0x1C, 0x55);
	result=EEPROM_Read_Byte(0x1C);
	_DBG("MASTER GET DATA = "); _DBG(" = ");_DBH(result); _DBG_(" ");

	// EEPROM Erase Byte
	_DBG_("EEPROM Erase Byte");
	EEPROM_Erase_Byte(0x1C);
	result=EEPROM_Read_Byte(0x1C);
	_DBG("MASTER GET DATA = "); _DBG(" = ");_DBH(result); _DBG_(" ");
	
	// EEPROM Write Page
	_DBG_("EEPROM Write Page");
	for(i=0;i<PAGE_SIZE;i++) write_buf[i]=i;
	EEPROM_Write_Page(0x00, write_buf);
	EEPROM_Read_Page(0x00, read_buf);
		
	for(i=0;i<PAGE_SIZE;i++)
	{
		_DBG("MASTER GET DATA = "); _DBG(" = ");_DBH(read_buf[i]); _DBG_(" ");
	}
	
	// EEPROM Erase Page
	_DBG_("EEPROM Erase Page");
	EEPROM_Erase_Page(0x06);
	EEPROM_Read_Page(0x00, read_buf);
		
	for(i=0;i<PAGE_SIZE;i++)
	{
		_DBG("MASTER GET DATA = "); _DBG(" = ");_DBH(read_buf[i]); _DBG_(" ");
	}
	
	while(1)
	{
             
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
    /*Initialize menu prinf*/
    DEBUG_Print_menu();
    
    /*Initialize port peripheral*/
    GPIO_Configure();
    
    /*Initialize ADC peripheral*/
    SPI_Configure();

   /*Disable IRQ Interrupts */
   __disable_irq(); 
	 
   /* Infinite Loop */
    while(1)
    {
        SPI_to_EEPROM();
    }
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	 DEBUG_Init();  
	
	/* Infinite loop */
	mainloop();  
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

