#ifndef _USB_H_
#define _USB_H_

#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"
#include "A31G32x_hal_usb.h"
#include "A31G32x_hal_usb_core.h"
#include "memory.h"

/*---------------------------------------------------------------------*/
/*  MACRO                                                              */
/*---------------------------------------------------------------------*/
#define SWAP16(s) 					(((((s) & 0xff) << 8) | (((s) >> 8) & 0xff)))
#define SWAP32(l) 					(((((l) & 0xff000000) >> 24) | (((l) & 0x00ff0000) >> 8)  | (((l) & 0x0000ff00) << 8)  | (((l) & 0x000000ff) << 24)))

#define MSB(word) 					(uint8_t)(((uint16_t)(word) >> 8) & 0xff)
#define LSB(word)					(uint8_t)((uint16_t)(word) & 0xff)


/*---------------------------------------------------------------------*/
/*  Define                                                             */
/*---------------------------------------------------------------------*/
#define USBD_VID 0x1A29	// VID
#define USBD_PID 0x7002 // PID

// USB Spec version
#define USB_11_SPEC_VER				(0x0110)
#define USB_20_SPEC_VER				(0x0200)

// HID Class 1.0 Spec version
#define USB_HID_SPEC_VER			(0x0100)

#define USB_EP0_MAX_PKT_SZ			(0x08)
#define eUSB_DESC_CONFIG_LEN    	(0x09)
#define eUSB_DESC_INTERFACE_LEN 	(0x09)
#define eUSB_DESC_HID_LEN       	(0x09)
#define eUSB_DESC_EP_LEN        	(0x07)

#define USB_EP0_CTRL				(0x00)  /* EP0 for Contol */
#define USB_EP1_MSC_IN				(0x81)  /* EP1 for data IN */
#define USB_EP1_MSC_OUT				(0x01)  /* EP1 for data OUT */
//#define USB_EP2_CDC_CMD				(0x82)  /* EP2 for CDC commands */

/* USB standard class */
#define USB_IN						(0x80)
#define USB_IN_MASK					(0x7F)

#define CDC_CMD_PACKET_SIZE 		(8)  /* Control Endpoint Packet size */ 
#define CDC_DATA_FS_MAX_PACKET_SIZE	(64) /* Endpoint IN & OUT Packet size */


/* MSC Bulk-only Stage */
#define MSC_BS_CBW                      0       /* Command Block Wrapper */
#define MSC_BS_DATA_OUT                 1       /* Data Out Phase */
#define MSC_BS_DATA_IN                  2       /* Data In Phase */
#define MSC_BS_DATA_IN_LAST             3       /* Data In Last Phase */
#define MSC_BS_DATA_IN_LAST_STALL       4       /* Data In Last Phase with Stall */
#define MSC_BS_CSW                      5       /* Command Status Wrapper */
#define MSC_BS_ERROR                    6       /* Error */


/* CSW Status Definitions */
#define CSW_CMD_PASSED                  0x00
#define CSW_CMD_FAILED                  0x01
#define CSW_PHASE_ERROR                 0x02


/* SCSI Commands */
#define SCSI_TEST_UNIT_READY            0x00
#define SCSI_REQUEST_SENSE              0x03
#define SCSI_FORMAT_UNIT                0x04
#define SCSI_INQUIRY                    0x12
#define SCSI_MODE_SELECT6               0x15
#define SCSI_MODE_SENSE6                0x1A
#define SCSI_START_STOP_UNIT            0x1B
#define SCSI_MEDIA_REMOVAL              0x1E
#define SCSI_READ_FORMAT_CAPACITIES     0x23
#define SCSI_READ_CAPACITY              0x25
#define SCSI_READ10                     0x28
#define SCSI_WRITE10                    0x2A
#define SCSI_VERIFY10                   0x2F
#define SCSI_MODE_SELECT10              0x55
#define SCSI_MODE_SENSE10               0x5A


/* Bulk-only Command Block Wrapper */
typedef struct _MSC_CBW {
  uint32_t dSignature;
  uint32_t dTag;
  uint32_t dDataLength;
  uint8_t  bmFlags;
  uint8_t  bLUN;
  uint8_t  bCBLength;
  uint8_t  CB[16];
} MSC_CBW;

/* Bulk-only Command Status Wrapper */
typedef struct _MSC_CSW {
  uint32_t dSignature;
  uint32_t dTag;
  uint32_t dDataResidue;
  uint8_t  bStatus;
} MSC_CSW;

#define MSC_CBW_Signature               0x43425355
#define MSC_CSW_Signature               0x53425355

#define MSC_MemorySize  8192
#define MSC_BlockSize   512
#define MSC_BlockCount  (MSC_MemorySize / MSC_BlockSize)

/* MSC Bulk Callback Functions */
extern void MSC_GetCBW (void);
extern void MSC_SetCSW (void);
extern void MSC_BulkIn (void);
extern void MSC_BulkOut(void);

/*---------------------------------------------------------------------*/
/*  Enumerator                                                         */
/*---------------------------------------------------------------------*/

/* USB Request Type */
typedef enum{
	eUSB_REQ_TYPE_STANDARD	= 0x0,
	eUSB_REQ_TYPE_CLASS		= 0x1,
	eUSB_REQ_TYPE_VENDOR	= 0x2,
}eUSB_ReqType;
	
/* USB Request */
typedef enum{
	eUSB_REQ_RECIPIENT_DEVICE	=0x00,
	eUSB_REQ_RECIPIENT_INTERFACE,
	eUSB_REQ_RECIPIENT_ENDPOINT,
	eUSB_REQ_RECIPIENT_MASK
}eUSB_Request;

/* USB Standard Device Request */
typedef enum{
	eUSB_REQ_GET_STATUS			=0x00,
	eUSB_REQ_CLEAR_FEATURE		=0x01,
	eUSB_REQ_SET_FEATURE		=0x03,
	eUSB_REQ_SET_ADDRESS		=0x05,
	eUSB_REQ_GET_DESCRIPTOR		=0x06,
	eUSB_REQ_SET_DESCRIPTOR		=0x07,
	eUSB_REQ_GET_CONFIGURATION	=0x08,
	eUSB_REQ_SET_CONFIGURATION	=0x09,
	eUSB_REQ_GET_INTERFACE		=0x0A,
	eUSB_REQ_SET_INTERFACE		=0x0B,
	eUSB_REQ_SYNCH_FRAME		=0x0C
}eUSB_StdDevReq;

/* USB Descriptor Type */
typedef enum{
	eUSB_DT_DEVICE			= 0x01,
	eUSB_DT_CONFIGURATION	= 0x02,
	eUSB_DT_STRING			= 0x03,
	eUSB_DT_INTERFACE		= 0x04,
	eUSB_DT_ENDPOINT		= 0x05,
	eUSB_DT_DEVICE_QUALIFIER= 0x06,
	eUSB_DT_OTHER_SPEED_CONFIGURATION = 0x07,
	eUSB_DT_BOS			= 				0x0F,
	eUSB_DT_HID         =  0x21,
	eUSB_DT_REPORT			= 0x22,
	eUSB_DT_PHYSICAL		= 0x23,
	eUSB_DT_CS_INTERFACE	= 0x24,   // Audio Class-Specific Descriptor
	eUSB_DT_CS_ENDPOINT		= 0x25,   // Audio Class-Specific Descriptor
	eUSB_DT_DFU_FUNCTIONAL	= 0x21,
	eUSB_DT_DFU_BOS			= 0xFF,
}eUSB_DescriptorType;

/* USB MSC Type */
typedef enum{
	eUSB_MSC_MASS_STORAGE_RESET = 0xFF,
	eUSB_MSC_GET_MAX_LUN	= 0xFE,
}eUSB_MSCCmdType;

/* USB HID class Type */
typedef enum{
	eUSB_HID_GET_PROTOCOL			= 0x03,
	eUSB_HID_SET_PROTOCOL			= 0x0B,
	eUSB_HID_GET_IDLE		= 0x02,
	eUSB_HID_SET_IDLE			= 0x0A,
	eUSB_HID_GET_DESCRIPTOR			= 0x21,
}eUSB_HIDCmdType;

/* USB Transfer Type */
typedef enum{
	eUSB_XferCTRL	= 0x00,
	eUSB_XferISO	= 0x01,
	eUSB_XferBULK	= 0x02,
	eUSB_XferINTR	= 0x03,
}eUSB_XferType;

/* USB Class Code */
typedef enum{
	eUSB_CLASS_CDC	= 0x02,
	eUSB_CLASS_HID	= 0x03,
}eUSB_ClassCode;

typedef enum{
	eUSB_EP0_IDLE=0,
	eUSB_EP0_SETUP,
	eUSB_EP0_DATA_IN,
	eUSB_EP0_DATA_OUT,
	eUSB_EP0_STATUS_IN,
	eUSB_EP0_STATUS_OUT,
	eUSB_EP0_STALL,
}eUSB_EP0Status;

typedef enum{
	eUSB_EP1_IDLE=0,
	eUSB_EP1_SETUP,
	eUSB_EP1_DATA_IN,
	eUSB_EP1_DATA_OUT,
	eUSB_EP1_STALL,
}eUSB_EP1Status;

/*---------------------------------------------------------------------*/
/*  Struct                                                             */
/*---------------------------------------------------------------------*/

typedef struct{
    uint8_t bmRequestType;
    uint8_t bRequest;
    uint16_t wValue;
    uint16_t wIndex;
    uint16_t wLength;
}tSetup_stage_data_packet;

typedef struct{
    uint32_t bitrate;
    uint8_t format;
    uint8_t paritytype;
    uint8_t datatype;
}tCdc_line_coding;

/*---------------------------------------------------------------------*/
/*  Variables                                                          */
/*---------------------------------------------------------------------*/
extern uint8_t device_desc[];
extern uint8_t device_qulifier_desc[];
extern uint8_t other_speed_desc[];
extern uint8_t config_desc[];
extern uint8_t AccHIDReportDesc[];
extern uint8_t lang_desc[];
extern uint8_t _cdc_line_coding_def[];
extern uint8_t __manufact_desc[];
extern uint8_t __product_desc[];
extern uint8_t __serial_desc[];
extern uint8_t hid_desc[];
/*---------------------------------------------------------------------*/
/*  Functions                                                          */
/*---------------------------------------------------------------------*/
void USB_Send_CSW_EP1(uint8_t *ch);
void USB_Send_DATA_EP1(uint8_t *ch, uint8_t len);
void USB_Receive_DATA_EP1(uint8_t *ch);
#endif
